@@ -608,7 +608,8 @@ def getADQLGrammarCopy(nodes):
 			- ','
 			- g.characterStringLiteral
 			- ')')
-		g.scalarArray1FunctionName = Regex("(?i)arr_(min|max|avg|stddev|sum)")
+		g.scalarArray1FunctionName = Regex(
+			"(?i)arr_(min|max|avg|stddev|sum|count)")
 		g.scalarArray1Function = (
 			g.scalarArray1FunctionName("fName")
 			- '(' - g.nvArgs - ')')
@@ -796,7 +797,7 @@ def getADQLGrammarCopy(nodes):
 			g.setGeneratingFunction
 			| g.possiblyAliasedTable
 			| g.derivedTable
-			| g.subJoin)
+			| g.subJoin).setName("Join Operand")
 		g.tableReference = (
 			g.joinedTable
 			| g.possiblyAliasedTable
@@ -826,7 +827,7 @@ def getADQLGrammarCopy(nodes):
 		g.joinedTable << (g.joinOperand
 			+ ZeroOrMore(g.joinOperator
 				+ g.joinOperand
-				+ Optional(g.joinSpecification)))
+				+ Optional(g.joinSpecification))).setName("Joined Table")
 
 # Detritus in table expressions
 		g.groupByClause = (CaselessKeyword("GROUP") + CaselessKeyword("BY")
@@ -852,7 +853,7 @@ def getADQLGrammarCopy(nodes):
 			+ Optional(g.orderByClause))
 
 # Common Table Expressions
-		g.selectExpression = Forward()
+		g.selectExpression = Forward().setName("Select expression")
 		g.querySetExpression = Forward()
 		g.querySetTerm = Forward()
 
@@ -892,7 +893,8 @@ def getADQLGrammarCopy(nodes):
 		g.selectExpression << (
 			g.querySetExpression
 			+  Optional(g.offsetSpec))
-		g.queryExpression << (g.selectExpression | g.joinedTable)
+		g.queryExpression << (g.selectExpression | g.joinedTable).setName(
+			"Query Expression")
 
 		g.querySpecification = Optional(g.withSpecification) + g.selectExpression
 
