@@ -2,7 +2,7 @@
 Helpers for morphing modules
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -35,8 +35,8 @@ class State(object):
 
 
 _BOOLEANIZER_TABLE = {
-	('=', '0'): "NOT ",
-	('!=', '1'): "NOT ",
+	('=', '0'): "NOT",
+	('!=', '1'): "NOT",
 	('=', '1'): "",
 	('!=', '0'): "",}
 
@@ -45,7 +45,7 @@ def addNotToBooleanized(expr, operator, operand):
 	be one for ADQL integerized boolean expressions.
 
 	The function will return None for unknown combinations of operator and
-	operand, and it will simply hand through Nones for expr, so calling 
+	operand, and it will simply hand through Nones for expr, so calling
 	functions can just return addNotToBooleanized(...).
 	"""
 	if expr is None:
@@ -55,8 +55,12 @@ def addNotToBooleanized(expr, operator, operand):
 	if prefix is None:
 		# weird non-boolean-looking condition
 		return None
-	
-	return prefix+expr
+
+	elif prefix:
+		return nodes.TransparentNode(children=[prefix, '(', expr, ')'])
+
+	else:
+		return expr
 
 
 def analyzeFuncComp(node, acceptableOperators=["=", "!="]):
@@ -70,7 +74,7 @@ def analyzeFuncComp(node, acceptableOperators=["=", "!="]):
 	* the operator is not symmetric
 	* none of the operators is a FunctionNode
 	"""
-	if (node.type!="comparisonPredicate" 
+	if (node.type!="comparisonPredicate"
 			or node.opr not in acceptableOperators):
 		return None, None
 
@@ -144,8 +148,8 @@ class Morpher(object):
 	should return the node if they do not with to change it.
 	state is a State instance.
 
-	The main entry point is morph(origTree) -> state, tree.  origTree is not 
-	modified, the return value can be flattened but can otherwise be severely 
+	The main entry point is morph(origTree) -> state, tree.  origTree is not
+	modified, the return value can be flattened but can otherwise be severely
 	damaged.
 
 	For special effects, there's also earlyMorphers.  These will be called
