@@ -10,11 +10,12 @@ for full ADQL geometry support, we'll phase this out (although it's a good
 deal faster when it works).
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import functools
 
 from gavo.adql import common
@@ -113,7 +114,7 @@ def _booleanizeContainsQ3C(node, operator, operand):
 
 		# the way PredicateGeometryFunction.optimize sorts the arguments
 		# has constants last, and here we want constants first.
-		expr = ("q3c_join(%s, %s, %s, %s, %s)"%tuple(map(_flatAndMorph, 
+		expr = ("q3c_join(%s, %s, %s, %s, %s)"%tuple(map(_flatAndMorph,
 			(shape.center.x, shape.center.y, p.x, p.y, shape.radius))))
 
 	elif shape.type=="polygon":
@@ -139,7 +140,7 @@ def _distanceToQ3C(node, state):
 	# to tell the comparison to disappear.
 
 	parent = state.nodeStack[-1]
-	if (parent.type!="comparisonPredicate" 
+	if (parent.type!="comparisonPredicate"
 			or parent.opr not in ['<', '<=', '>', '>=']):
 		# no supported comparison; pass this on to pgsphere (which probably
 		# isn't much slower, as q3c_dist is bad with indices, too).
@@ -257,7 +258,7 @@ def _transformSystems(pgLiteral, fromSystem, toSystem):
 
 
 def _booleanizeGeoPredPGS(node, geoOp):
-	"""returns an expression node for pgsphere for an ADQL 
+	"""returns an expression node for pgsphere for an ADQL
 	CONTAINS/INTERSECTS call.
 	"""
 	expr = None
@@ -361,8 +362,8 @@ def _distanceToPG(node, state):
 
 	parent.OVERRIDE_RESULT = "%s (%s) <@ scircle(%s, RADIANS(%s))"%(
 		negation,
-		flatten(args[0]), 
-		flatten(args[1]), 
+		flatten(args[0]),
+		flatten(args[1]),
 		flatten(compareAgainst))
 
 	return expr
@@ -510,8 +511,8 @@ _miscMorphers = {
 }
 
 
-class _PGSC(nodes.SelectNoParens):
-	"""A modifield selectNoParens that fixes the syntactic differences
+class _PGSC(nodes.SelectQuery):
+	"""A modifield SelectQuery that fixes the syntactic differences
 	between ADQL and postgres.
 	"""
 	def flatten(self):
@@ -528,7 +529,7 @@ class _PGSC(nodes.SelectNoParens):
 
 
 class _PGQS(nodes.ADQLNode):
-	"""A wrapper for a postgres query specification.  
+	"""A wrapper for a postgres query specification.
 	
 	The only function here is to make sure there's just one LIMIT part
 	at the very end (except, of course, in deeper subqueries).
@@ -605,7 +606,7 @@ def _fixSetLimit(node, state):
 		n.setLimit = None
 	offset = node.offset
 	node.offset = None
-	return _PGQS(original=node, 
+	return _PGQS(original=node,
 		setLimit=node.setLimit and str(node.setLimit),
 		offset=offset)
 
@@ -637,15 +638,15 @@ def _materializeIfNecessary(node, state):
 		index += 1
 		node.children = node.children[:index]+(
 			"MATERIALIZED",)+node.children[index:]
-		return node
+	return node
 
 
 _syntaxMorphers = {
-	"selectNoParens": _insertPGSC,
+	"selectQuery": _insertPGSC,
 	'comparisonPredicate': morphhelpers.booleanizeComparisons,
 	'selectList': _expandStars,
 	'derivedColumn': _forceAlias,
-	"selectQuery": _fixSetLimit,
+	"selectExpression": _fixSetLimit,
 	"withQuery": _materializeIfNecessary,
 }
 
