@@ -1075,6 +1075,21 @@ class SetExpression(SetOperationNode):
 	type = "querySetExpression"
 	collapsible = True
 
+	# we hand the various introspection items through to our first child;
+	# if one day we want to make sure the relations of all children are
+	# reconcilable, this probably should happen in a _polish method.
+
+	@property
+	def fromClause(self):
+		return self.children[0].fromClause
+	
+	def getContributingNames(self):
+		res = []
+		for child in self.children:
+			if hasattr(child, "getContributingNames"):
+				res.extend(child.getContributingNames())
+		return res
+
 
 class WithQuery(SetOperationNode):
 	"""A query from a with clause.
@@ -1169,8 +1184,8 @@ class QuerySpecification(TransparentNode):
 		return self.children[-1].setLimit
 	setLimit = property(_getSetLimit, _setSetLimit)
 
-
 	def __getattr__(self, attrName):
+		# our last child always is a querySpecification
 		return getattr(self.children[-1], attrName)
 
 
@@ -1785,13 +1800,12 @@ class ScalarArrayFunction(FunctionNode):
 	type = "scalarArrayFunction"
 
 	funcDefs = {
-		# the second column was intended for unit handling but isn't used
-		# for now.  Encode unit changes there if they become necessary.
 		'ARR_MIN': ('stat.min;{u}', None, 1),
 		'ARR_MAX': ('stat.max;{u}', None, 1),
 		'ARR_AVG': ('{u};stat.mean', None, 1),
 		'ARR_STDDEV': ('stat.stdev;{u}', None, 1),
 		'ARR_SUM': ('{u};arith.sum', None, 1),
+		'ARR_COUNT': ('meta.number;{u}', "", 1),
 		'ARR_DOT': (None, None, 2),
 	}
 
@@ -1804,16 +1818,19 @@ class ScalarArrayFunction(FunctionNode):
 			raise common.Error(
 				"Grammar vs. annotation function arity mismatch!")
 
+		# we're being very liberal with upcasting to double precision
+		# here since that may help fend off integer overflows.
+		type = "double precision"
 		if nPar==1:
 			baseFI = infoChildren[0].fieldInfo
 			mat = fieldinfo.isArray(baseFI.type)
 			if not mat:
 				raise common.Error(
 					"Scalar array function called on non-array?")
-			type = mat.group(1)
+			if self.funName=="ARR_COUNT":
+				type = "integer"
 
 		elif nPar==2:
-			type = "double precision"
 			baseFI = fieldinfo.FieldInfo(type, "", "",
 				userData=infoChildren)
 		else:
@@ -1823,6 +1840,8 @@ class ScalarArrayFunction(FunctionNode):
 		ucd, unit = "", baseFI.unit
 		if ucdchange is not None and baseFI.ucd:
 			ucd = ucdchange.format(u=baseFI.ucd)
+		if unitchange is not None:
+			unit = unitchange
 
 		self.fieldInfo = fieldinfo.FieldInfo(type, unit, ucd,
 			*collectUserData(infoChildren))
