@@ -18,7 +18,11 @@ from gavo.adql import common
 from gavo.adql import fieldinfo
 from gavo.adql import morphhelpers
 from gavo.adql import nodes
+from gavo.base import sqlsupport
 
+if sqlsupport.GAVOConnection.extensionFunctions is None:
+	raise ImportError("Early import of adql.ufunctions:"
+		"  sqlsupport not initialised yet")
 
 UFUNC_REGISTRY = {}
 
@@ -87,11 +91,16 @@ def userFunction(name,
 		f.adqlUDF_returntype = returntype
 		f.adqlUDF_unit = unit
 		f.adqlUDF_ucd = ucd
-		f.adqlUDF_depends = depends
 		f.adqlUDF_additionalSignatures = [
 				{"signature": f.adqlUDF_name+signature.strip(), "doc": doc}
 			for signature, doc in polymorphism]
 
+		if depends:
+			if depends not in sqlsupport.GAVOConnection.extensionFunctions:
+				# don't register if the function would be broken because of
+				# missing dependencies.
+				return f
+
 		UFUNC_REGISTRY[f.adqlUDF_name.upper()] = f
 		for alias in additionalNames:
 			UFUNC_REGISTRY[alias.upper()] = f
@@ -248,7 +257,7 @@ def _interval_has(args):
 _makeTwoArgBooleanizer("ivo_interval_has", "((%(1)s) <@ (%(2)s))")
 
 
-@userFunction("gavo_to_mjd",
+@userFunction("ivo_to_mjd",
 	"(d TIMESTAMP) -> DOUBLE PRECISION",
 	"""
 	The function converts a postgres timestamp to modified julian date.
@@ -256,14 +265,15 @@ _makeTwoArgBooleanizer("ivo_interval_has", "((%(1)s) <@ (%(2)s))")
 	scales or the like are done; you can thus not expect this to be
 	good to second-precision unless you are careful in the construction
 	of the timestamp.
-	""")
+	""",
+	additionalNames=["gavo_to_mjd"])
 def _to_mjd(args):
 	if len(args)!=1:
 		raise common.UfuncError("gavo_to_mjd takes exactly one timestamp argument")
 	return "ts_to_mjd(%s)"%nodes.flatten(args[0])
 
 
-@userFunction("gavo_to_jd",
+@userFunction("ivo_to_jd",
 	"(d TIMESTAMP) -> DOUBLE PRECISION",
 	"""
 	The function converts a postgres timestamp to julian date.
@@ -271,7 +281,8 @@ def _to_mjd(args):
 	scales or the like are done; you can thus not expect this to be
 	good to second-precision unless you are careful in the construction
 	of the timestamp.
-	""")
+	""",
+	additionalNames=["gavo_to_jd"])
 def _to_jd(args):
 	if len(args)!=1:
 		raise common.UfuncError("gavo_to_jd takes exactly one timestamp argument")
@@ -368,7 +379,7 @@ class _TransformNode(nodes.FunctionNode):
 		self.fieldInfo.ucd = self.ucdMap.get(self.toSys, "pos")
 
 
-@userFunction("gavo_transform",
+@userFunction("ivo_geom_transform",
 	"(from_sys TEXT, to_sys TEXT, geo GEOMETRY) -> GEOMETRY",
 	"""
 	The function transforms ADQL geometries between various reference systems.
@@ -386,7 +397,8 @@ class _TransformNode(nodes.FunctionNode):
 	is treated as ICRS), FK4 (for B1950. without epoch-dependent corrections),
 	GALACTIC.  Reference frame names are case-sensitive.
 	""",
-	returntype="GEOMETRY")
+	returntype="GEOMETRY",
+	additionalNames=["gavo_transform"])
 def _gavo_transform(args):
 	if len(args)!=3:
 		raise common.UfuncError(
@@ -412,7 +424,7 @@ def _gavo_transform(args):
 			toSys=toSys))
 
 
-@userFunction("gavo_normal_random",
+@userFunction("ivo_normal_random",
 	"(mu REAL, sigma REAL) -> REAL",
 	"""The function returns a random number drawn from a normal distribution
 	with mean mu and width sigma.
@@ -422,7 +434,8 @@ def _gavo_transform(args):
 	very precise or fast.  It might work for some use cases, and we
 	will provide a better implementation if this proves inadequate.
 	""",
-	returntype="real")
+	returntype="real",
+	additionalNames=["gavo_normal_random"])
 def _gavo_normal_random(args):
 	if len(args)!=2:
 		raise common.UfuncError(
@@ -917,22 +930,3 @@ class UserFunction(nodes.FunctionNode):
 
 
 nodes.registerNode(UserFunction)
-
-
-def _cleanForConn(conn):
-	"""removes UDFs requiring postgres functions not available in what conn
-	points to.
-
-	This is run once during database initialisation.
-	"""
-	toRemove = set()
-	for name, funcDef in UFUNC_REGISTRY.items():
-		if funcDef.adqlUDF_depends:
-			if funcDef.adqlUDF_depends not in conn.extensionFunctions:
-				toRemove.add(name)
-	
-	for name in toRemove:
-		del UFUNC_REGISTRY[name]
-
-
-base.onDBInit(_cleanForConn)
