@@ -22,7 +22,7 @@ There is a non-expanding version of STREAM, too, but that has to take
 special precautions.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -45,8 +45,8 @@ from gavo.base import structure
 # the following is a sentinel for values that have been expanded
 # by an active tag already.  When active tags are nested, only the
 # innermost active tag must expand macros so one can be sure that
-# double-escaped macros actually end up when the events are finally 
-# replayed at the top level.  _EXPANDED_VALUE must compare true to 
+# double-escaped macros actually end up when the events are finally
+# replayed at the top level.  _EXPANDED_VALUE must compare true to
 # value since it is used as such in event triples.
 
 class _ExValueType(object):
@@ -168,7 +168,7 @@ class RecordingBase(structure.Structure):
 	def feedEvent(self, ctx, type, name, value):
 		# keep _EXPANDED_VALUE rather than feed the value to protect it from
 		# further expansion by subordinate structures (except if we, the
-		# active tag, is the final recipient, in which case we gobble the thing
+		# active tag, are the final recipient, in which case we gobble the thing
 		# ourselves).
 		if (type is _EXPANDED_VALUE and name not in self.managedAttrs):
 			self.events_.append((_EXPANDED_VALUE, name, value, ctx.pos))
@@ -347,14 +347,14 @@ class Prune(ActiveTag, structure.Structure):
 class Edit(EmbeddedStream):
 	"""an event stream targeted at editing other structures.
 
-	When replaying a stream in the presence of EDITs, the elements are 
+	When replaying a stream in the presence of EDITs, the elements are
 	are continually checked against ref.  If an element matches, the
 	children of edit will be played back into it.
 	"""
 	name_ = "EDIT"
 
 	_ref = attrdef.UnicodeAttribute("ref", description="Destination of"
-		" the edits, in the form elementName[<name or id>]", 
+		" the edits, in the form elementName[<name or id>]",
 		default=utils.Undefined)
 
 	refPat = re.compile(
@@ -363,7 +363,7 @@ class Edit(EmbeddedStream):
 	def onElementComplete(self):
 		mat = self.refPat.match(self.ref)
 		if not mat:
-			raise common.LiteralParseError("ref", self.ref, 
+			raise common.LiteralParseError("ref", self.ref,
 				hint="edit references have the form <element name>[<value of"
 					" name or id attribute>]")
 		self.triggerEl, self.triggerId = mat.groups()
@@ -421,7 +421,7 @@ class ReplayBase(ActiveTag, structure.Structure, macros.StandardMacroMixin):
 		with ctx.replaying():
 			for type, name, val, pos in events:
 				if (self._expandMacros
-						and type=="value" 
+						and type=="value"
 						and (self.reexpand or type is not _EXPANDED_VALUE)
 						and "\\" in val):
 					try:
@@ -590,14 +590,16 @@ class GeneratorAttribute(attrdef.UnicodeAttribute):
 			raise common.RestrictedElement("codeItems")
 		attrdef.UnicodeAttribute.feed(self, ctx, instance, literal)
 		src = utils.fixIndentation(
-			getattr(instance, self.name_), 
+			getattr(instance, self.name_),
 			"  ", governingLine=1)
 		src = "def makeRows(context):\n"+src+"\n"
 		instance.iterRowsFromCode = utils.compileFunction(
 			src, "makeRows", useGlobals={
-				"definitionContext": ctx, 
+				"definitionContext": ctx,
 				"utils": utils,
-				"base": base})
+				"base": base},
+			uniqueName="<{} at {}>".format(
+				instance.name_, instance.getSourcePosition()))
 
 
 class Loop(ReplayedEventsWithFreeAttributesBase):
