@@ -14,10 +14,11 @@ import re
 import shlex
 import sys
 
+# You can't import base or anything from base here, as that would
+# import sqlsupport, and sqlsupport needs config initialised to
+# even come up.
+
 from gavo import utils
-from gavo.base import attrdef
-from gavo.base import meta
-from gavo.base import structure
 from gavo.utils import fancyconfig
 from gavo.utils.fancyconfig import (StringConfigItem, #noflake: exported names
 	EnumeratedConfigItem, IntConfigItem, PathConfigItem, ListConfigItem,
@@ -28,11 +29,6 @@ fancyconfig.BAD_CONFIG_ITEM_JUST_WARNS = True
 
 defaultSettingsPath = "/etc/gavo.rc"
 
-addMeta = meta.configMeta.addMeta
-setMeta = meta.configMeta.setMeta
-getMeta = meta.configMeta.getMeta
-
-
 
 class RootRelativeConfigItem(PathRelativeConfigItem):
 	baseKey = "rootDir"
@@ -111,26 +107,21 @@ class ProfileParseError(Error):
 from configparser import NoOptionError
 
 
-class DBProfile(structure.Structure):
-	"""is a profile for DB access.
+class DBProfile:
+	"""A parsed form of the postgres connection string.
 	"""
-	name_ = "dbProfile"
 	profileName = "anonymous"
+	name = None
+	host = ""
+	port = None
+	database = ""
+	user = ""
+	password = ""
+	sslmode = "allow"  # or disable, allow, prefer, require, verify-*)")
 
-	_name = attrdef.UnicodeAttribute("name", default=attrdef.Undefined,
-		description="An identifier for this profile")
-	_host = attrdef.UnicodeAttribute("host", default="", description="Host"
-		" the database runs on")
-	_port = attrdef.IntAttribute("port", default=None, description=
-		"Port the DB server listens on")
-	_database = attrdef.UnicodeAttribute("database", default=attrdef.Undefined,
-		description="Name of the database to connect to")
-	_user = attrdef.UnicodeAttribute("user", default="", description=
-		"User to log into DB as")
-	_pw = attrdef.UnicodeAttribute("password", default="", description=
-		"Password for user")
-	_sslmode = attrdef.UnicodeAttribute("sslmode", default="allow", description=
-		"SSL negotiation mode (disable, allow, prefer, require, verify-*)")
+	def __init__(self, **kwargs):
+		for k, v in kwargs.items():
+			setattr(self, k, v)
 
 	def getArgs(self):
 		"""returns a dictionary suitable as keyword arguments to psycopg2's
@@ -163,7 +154,7 @@ class DBProfile(structure.Structure):
 					" allowed when DaCHS is called interactively.")
 
 
-class ProfileParser(object):
+class ProfileParser:
 	r"""is a parser for DB profiles.
 
 	The profiles are specified in simple text files that have a shell-like
@@ -206,7 +197,7 @@ class ProfileParser(object):
 
 		self.parser = shlex.shlex(stream, sourceName, posix=True)
 		self.parser.whitespace = " \t\r"
-		self.profile = DBProfile(None, name=profileName)
+		self.profile = DBProfile(name=profileName)
 		while True:
 			tok = self.parser.get_token()
 			if not tok:
@@ -608,23 +599,6 @@ getitem = _config.getitem
 getDBProfile = _config.getDBProfile
 
 
-def makeFallbackMeta(reload=False):
-	"""fills meta.configMeta with items from $configDir/defaultmeta.txt.
-
-	This is called from the module __init__ once typed meta from base
-	is in (which means that any typed meta defined in other places won't
-	become magic in fallback; regrettably, we can't hold off loading fallback
-	meta much longer).
-	"""
-	srcPath = os.path.join(get("configDir"), "defaultmeta.txt")
-	if not os.path.exists(srcPath):
-		# this will be made a bit later in dachs init
-		return
-	with open(srcPath, "rb") as f:
-		content = f.read().decode("utf-8", "ignore")
-	meta.parseMetaStream(meta.configMeta, content, clearItems=reload)
-
-
 def getRSTReference(underlineChar="-"):
 	"""returns a ReStructuredText reference of configuration items.
 
