@@ -1,11 +1,11 @@
 """
-(Mostly deprecated) code to handle coordinate systems and transform 
-between them.  
+(Mostly deprecated) code to handle coordinate systems and transform
+between them.
 
 Basically all of this should be taken over by stc and astropysics.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -189,7 +189,7 @@ def getWCS(wcsFields, naxis=(1,2), relax=True):
 
 	# pywcs used to invent identity transforms if no WCS keys are present.
 	# Hence. we do some sanity checking up front to weed those out.
-	if ("CD1_1" not in wcsFields 
+	if ("CD1_1" not in wcsFields
 			and "CDELT1" not in wcsFields
 			and "PC1_1" not in wcsFields
 			and "CNPIX1" not in wcsFields):
@@ -306,6 +306,9 @@ def getCenterFromWCSFields(wcsFields, spatialAxes=(1,2)):
 	the pixel lengths of the axes.
 	"""
 	wcs = getWCS(wcsFields)
+	if wcs is None:
+		raise ValueError("No WCS solution found")
+
 	center1 = wcs._dachs_header["NAXIS%s"%spatialAxes[0]]/2.
 	center2 = wcs._dachs_header["NAXIS%s"%spatialAxes[1]]/2.
 	return pix2sky(wcs, (center1, center2))
@@ -329,7 +332,7 @@ def getCoveringCircle(wcsFields, spatialAxes=(1,2)):
 
 
 def getSkyWCS(hdr):
-	"""returns a pair of a wcs.WCS instance and a sequence of 
+	"""returns a pair of a wcs.WCS instance and a sequence of
 	the spatial axes.
 
 	This will be None, () if no WCS could be discerned.  There's some
@@ -380,7 +383,7 @@ def getPixelLimits(cooPairs, wcsFields):
 	# due to spherical magic this is not enough to ensure +/-Inf behaviour
 	# for SODA; TODO: re-evaluate this for astropy.wcs at some point.
 	cooPairs = [(
-			min(359.99999, max(-89.9999, ra)), 
+			min(359.99999, max(-89.9999, ra)),
 			min(89.9999, max(-89.9999, dec)))
 		for ra, dec in cooPairs]
 
