@@ -16,6 +16,7 @@ base.typesystems.ToPythonCodeConverter.
 
 
 import functools
+import json
 import re
 
 # let's depend on psycopg2 for now as regards ranges.  We can always
@@ -108,6 +109,7 @@ def parseBytes(literal):
 	This will interpret hex and octal byte escapes, and it'll support lists
 	of integer-like things; not sure if that's actually more harmful than good.
 	But then people can always override the default behaviour.
+
 	>>> parseBytes("abc")
 	b'abc'
 	>>> parseBytes(r"\xab\000")
@@ -138,6 +140,7 @@ def parseBytes(literal):
 	else:
 		raise ValueError("No idea how to make bytes from %s"%repr(literal))
 
+
 def parseCooPair(soup):
 	"""returns a pair of RA, DEC floats if they can be made out in soup
 	or raises a value error.
@@ -286,6 +289,14 @@ def _numericRangeFactory(colDesc):
 utils.registerDefaultMF(_numericRangeFactory)
 
 
+def parseJSON(val):
+	# we allow JSON input as dicts or lists; keep psycopg from serialising
+	# things that probably came from json in non-json ways
+	if isinstance(val, (dict, list)):
+		return json.dumps(val)
+	return val or None
+
+
 @functools.lru_cache(1)
 def getDefaultValueParsers():
 	"""returns a dict containing all exported names from this module.
@@ -311,4 +322,4 @@ if __name__=="__main__":
 __all__ = ["parseInt", "parseFloat", "parseBooleanLiteral", "parseUnicode",
 	"parseDefaultDate", "parseDefaultTime", "parseDefaultDatetime",
 	"parseCooPair", "getDefaultValueParsers", "parseSPoint", "parseSimpleSTCS",
-	"NumericRange", "originalOrIdentity", "parseBytes"]
+	"NumericRange", "originalOrIdentity", "parseBytes", "parseJSON"]
