@@ -37,12 +37,14 @@ through @meta.forKeys).
 #c COPYING file in the source distribution.
 
 
+import os
 import re
 import textwrap
 
 from gavo import utils
 from gavo.base import attrdef
 from gavo.base import common
+from gavo.base import config
 from gavo.utils import stanxml
 
 
@@ -488,7 +490,7 @@ class MetaMixin(common.StructCallbacks):
 				return self.__metaParent._getMeta(atoms, propagate,
 					acceptSequence=acceptSequence)
 			else:
-				return configMeta._getMeta(atoms, propagate=False,
+				return CONFIG_META._getMeta(atoms, propagate=False,
 					acceptSequence=acceptSequence)
 
 		raise NoMetaKey("No meta item %s"%".".join(atoms), carrier=self)
@@ -557,7 +559,7 @@ class MetaMixin(common.StructCallbacks):
 			if self.__hasMetaParent():
 				val = self.__metaParent.iterMeta(key, propagate=True)
 			else:
-				val = configMeta.iterMeta(key, propagate=False)
+				val = CONFIG_META.iterMeta(key, propagate=False)
 		return val
 
 	def getAllMetaPairs(self):
@@ -692,9 +694,6 @@ class MetaMixin(common.StructCallbacks):
 			del item.copied
 
 
-# Global meta, items get added from config
-configMeta = MetaMixin()
-
 
 class ComputedMetaMixin(MetaMixin):
 	"""A MetaMixin for classes that want to implement defaults for
@@ -1235,3 +1234,24 @@ class ModelBasedBuilder(object):
 		if macroPackage is None:
 			macroPackage = metaContainer
 		return self._build(self.constructors, metaContainer, macroPackage)
+
+
+# Global meta, items get added
+CONFIG_META = MetaMixin()
+
+
+def makeFallbackMeta(reload=False):
+	"""fills CONFIG_META with items from $configDir/defaultmeta.txt.
+
+	This is called from the module __init__ once typed meta from base
+	is in (which means that any typed meta defined in other places won't
+	become magic in fallback; regrettably, we can't hold off loading fallback
+	meta much longer).
+	"""
+	srcPath = os.path.join(config.get("configDir"), "defaultmeta.txt")
+	if not os.path.exists(srcPath):
+		# this will be made a bit later in dachs init
+		return
+	with open(srcPath, "rb") as f:
+		content = f.read().decode("utf-8", "ignore")
+	parseMetaStream(CONFIG_META, content, clearItems=reload)
