@@ -53,10 +53,6 @@ _PG_TIME_UNITS = {
 	"d": 86400.,}
 
 
-# Keep track of whether we have installed our extensions
-# (this is to not require a DB connection on just importing this)
-_PSYCOPG_INITED = False
-
 class SqlSetAdapter(object):
 	"""is an adapter that formats python sequences as SQL sets.
 
@@ -251,11 +247,25 @@ class GAVOConnection(psycopg2.extensions.connection):
 	arguments to the connection; it is used when recovering from
 	a database restart.
 	"""
-	# extensionFunctions is filled by _initPsycopg on the class level
-	# (and contains names of postgres extension functions that might
-	# modify our behaviour
+	# extensionFunctions is filled senseEnvironment (and contains names of
+	# postgres extension functions that might modify our behaviour)
 	extensionFunctions = None
 
+	@classmethod
+	def senseEnvironment(cls, conn):
+		"""configures us depending on what is in the database.
+
+		The argument needs to be a connection to the database we will
+		connect to.  In practice, _initPsycopg calls this once during
+		DaCHS startup.
+		"""
+		cls.extensionFunctions = frozenset(r[0] for r in
+			conn.query(
+     		"SELECT proname FROM pg_proc WHERE"
+     		"  proname in ('epoch_prop', 'q3c_ang2ipix',"
+     		"    'smoc_union', 'healpix_nest')"))
+
+
 	def getParameter(self, key, cursor=None):
 		"""returns the value of the postgres parameter key.
 
@@ -453,9 +463,6 @@ def getDBConnection(profile, debug=debug, autocommitted=False):
 				" in $GAVO_DIR/etc or your database config (in particular,"
 				" pg_hba.conf).")
 
-	if not _PSYCOPG_INITED:
-		_initPsycopg()
-
 	if autocommitted:
 		conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
 	conn.set_client_encoding("UTF8")
@@ -1085,97 +1092,6 @@ def getDBMeta(key):
 		return res[0][0]
 
 
-JOIN_FUNCTION_BODY = """
-SELECT (
-    (
-       ((q3c_ang2ipix($3,$4)>=(q3c_nearby_it($1,$2,$5,0))) AND (q3c_ang2ipix($3,$4)<=(q3c_nearby_it($1,$2,$5,1))))
-    OR ((q3c_ang2ipix($3,$4)>=(q3c_nearby_it($1,$2,$5,2))) AND (q3c_ang2ipix($3,$4)<=(q3c_nearby_it($1,$2,$5,3))))
-    OR ((q3c_ang2ipix($3,$4)>=(q3c_nearby_it($1,$2,$5,4))) AND (q3c_ang2ipix($3,$4)<=(q3c_nearby_it($1,$2,$5,5))))
-    OR ((q3c_ang2ipix($3,$4)>=(q3c_nearby_it($1,$2,$5,6))) AND (q3c_ang2ipix($3,$4)<=(q3c_nearby_it($1,$2,$5,7))))
-    ) AND
-    (
-       ((q3c_ang2ipix($1,$2)>=(q3c_nearby_it($3,$4,$5,0))) AND (q3c_ang2ipix($1,$2)<=(q3c_nearby_it($3,$4,$5,1))))
-    OR ((q3c_ang2ipix($1,$2)>=(q3c_nearby_it($3,$4,$5,2))) AND (q3c_ang2ipix($1,$2)<=(q3c_nearby_it($3,$4,$5,3))))
-    OR ((q3c_ang2ipix($1,$2)>=(q3c_nearby_it($3,$4,$5,4))) AND (q3c_ang2ipix($1,$2)<=(q3c_nearby_it($3,$4,$5,5))))
-    OR ((q3c_ang2ipix($1,$2)>=(q3c_nearby_it($3,$4,$5,6))) AND (q3c_ang2ipix($1,$2)<=(q3c_nearby_it($3,$4,$5,7))))
-    )
-    )
-    AND q3c_sindist($1,$2,$3,$4)<POW(SIN(RADIANS($5)/2),2)
-' LANGUAGE SQL IMMUTABLE;
-"""
-
-
-def onDBInit(func):
-	"""runs func(conn) with an administrator connection during database
-	initialisation (or immediately if the database is already initialised).
-
-	This is done in order to let modules do database initialisation while
-	they are imported without colliding with the module lock problems of
-	the connection pools.
-
-	If func fails, it will stop the DaCHS startup, so be extra careful
-	with what you do in there.
-	"""
-	if _PSYCOPG_INITED:
-		with getWritableAdminConn() as conn:
-			func(conn)
-
-	else:
-		onDBInit.initFuncs.append(func)
-
-onDBInit.initFuncs = []
-
-
-def _initPsycopg():
-	"""does any DaCHS-specific database setup necessary.
-
-	This will always open an admin connection.
-	"""
-# collect all DB setup in this function or use onDBInit.  XXX TODO:
-# in particular, the Box mess from coords (if we still want it)
-	global _PSYCOPG_INITED
-
-	conn = psycopg2.connect(connection_factory=GAVOConnection,
-		**config.getDBProfile("feed").getArgs())
-	try:
-		try:
-			from gavo.utils import pgsphere
-			pgsphere.preparePgSphere(conn)
-		except:  # prama: no cover
-			warnings.warn("pgsphere missing -- ADQL, pg-SIAP, and SSA will not work")
-	
-		# Add symmetrised q3c_joins if q3c is in use and the functions are
-		# not already defined
-		# TODO: Delete this when q3c is fixed.
-		funcs = set(r[0] for r in
-			conn.query("SELECT DISTINCT proname FROM pg_proc"
-			" WHERE proname IN ('q3c_join', 'q3c_join_symmetric')"))
-		if funcs==frozenset(["q3c_join"]):
-			# q3c is there, but not our extension
-			conn.execute("""CREATE OR REPLACE FUNCTION q3c_join_symmetric(
-				leftra double precision, leftdec double precision,
-      	rightra double precision, rightdec double precision,
-      	radius double precision) RETURNS boolean AS '"""+JOIN_FUNCTION_BODY)
-			conn.execute("""CREATE OR REPLACE FUNCTION q3c_join_symmetric(
-				leftra double precision, leftdec double precision,
-      	rightra real, rightdec real,
-      	radius double precision) RETURNS boolean AS '"""+JOIN_FUNCTION_BODY)
-
-		GAVOConnection.extensionFunctions = frozenset(r[0] for r in
-			conn.query(
-     		"SELECT proname FROM pg_proc WHERE"
-     		"  proname in ('epoch_prop', 'q3c_ang2ipix',"
-     		"    'smoc_union', 'healpix_nest')"))
-
-		for func in onDBInit.initFuncs:
-			func(conn)
-	finally:
-		conn.commit()
-		conn.close()
-
-	_PSYCOPG_INITED = True
-
-
 class CustomConnectionPool(psycopg2.pool.ThreadedConnectionPool):
 	"""A threaded connection pool that returns connections made via
 	profileName.
@@ -1330,3 +1246,29 @@ getWritableTableConn = _makeConnectionManager("trustedquery",
 	autocommitted=False, singleton=True)
 getWritableAdminConn = _makeConnectionManager("admin",
 	autocommitted=False, singleton=True)
+
+
+def initPsycopg():
+	"""does any DaCHS-specific database setup necessary.
+
+	This is executed on sqlsupport import unless we are in initdachs (or
+	setting up the testbed); see the foot of this module for how this is done.
+
+	This needs to call the GAVOConnection.senseEnvironment.
+	"""
+	conn = psycopg2.connect(connection_factory=GAVOConnection,
+		**config.getDBProfile("feed").getArgs())
+	try:
+		try:
+			from gavo.utils import pgsphere
+			pgsphere.preparePgSphere(conn)
+		except:  # prama: no cover
+			warnings.warn("pgsphere missing -- ADQL, pg-SIAP, and SSA will not work")
+	
+		GAVOConnection.senseEnvironment(conn)
+	finally:
+		conn.close()
+
+
+if "GAVO_INIT_RUNNING" not in os.environ:
+	initPsycopg()
