@@ -118,14 +118,22 @@ class RelatedResourceMeta(meta.MetaValue):
 		isSupplementTo: GAVO TAP service
 		isSupplementTo.ivoId: ivo://org.gavo.dc
 
+	It is also possible to include an altIdentifier (such as a DOI) to declare
+	a relationship to a resource not declared in the IVOA registry.
+
 	``isServedBy`` and ``isServiceFor`` are somewhat special cases, as
 	the service attribute of data publications automatically takes care
 	of them; so, you shouldn't usually need to bother with these two manually.
 	"""
-	def __init__(self, title, format="plain", ivoId=None):
+	def __init__(self, title,
+			format="plain",
+			ivoId=None,
+			altIdentifier=None):
 		meta.MetaValue.__init__(self, title, format)
 		if ivoId is not None:
 			self._addMeta(["ivoId"], meta.MetaValue(ivoId))
+		if altIdentifier is not None:
+			self._addMeta(["altIdentifier"], meta.MetaValue(altIdentifier))
 	
 
 @meta.forKeys("_news")
@@ -232,7 +240,7 @@ class DatetimeMeta(meta.MetaValue):
 				# backwards compatibility hack: legacy (1.0) userconfigs would try
 				# to pull authority.creationDate via unexpanded macros, and
 				# that breaks now.  Hack around it for now.
-				content = meta.getMetaText(meta.configMeta, "authority.creationDate")
+				content = meta.getMetaText(meta.CONFIG_META, "authority.creationDate")
 
 			content = utils.parseISODT(content)
 
