@@ -201,10 +201,13 @@ class ToPythonBase(FromSQLConverter):
 		"pql-date": "identity",
 		"pql-upload": "identity",
 		"int4range": "parseInterval",
+		"json": "parseJSON",
+		"jsonb": "parseJSON",
 	}
 
 	def mapComplex(self, type, length):
-		if type in self._charTypes:
+		if type in self._charTypes and not "x" in length:
+			# a plain sequence of chars is close enough to a string
 			return "parseUnicode"
 		else:
 			return "identity"  # Anything sufficiently complex is python anyway :-)
