@@ -19,6 +19,7 @@ http://svn.ari.uni-heidelberg.de/svn/gavo/python/trunk/tests/unitconvtest.py
 #c COPYING file in the source distribution.
 
 
+import functools
 import math
 
 from gavo import utils
@@ -499,95 +500,91 @@ def evalAll(s, p, toks):
 	return eval("".join(str(tok) for tok in toks))
 
 
-class getUnitGrammar(utils.CachedResource):
+@functools.cache
+def getUnitGrammar(debugging=False):
 	"""the grammar to parse VOUnits.
 
 	After initialization, the class has a "symbols" dictionary containing
 	the individual nonterminals.
 	"""
-	@classmethod
-	def impl(cls):
-		from gavo.utils.parsetricks import (Word, Literal, Regex,
-			Optional, ZeroOrMore, alphas,
-			Suppress, Forward, pyparsingWhitechars)
+	from gavo.utils.parsetricks import (Word, Literal, Regex,
+		Optional, ZeroOrMore, alphas,
+		Suppress, Forward, pyparsingWhitechars, ParserElement)
 
-		with pyparsingWhitechars(''):
-			unit_atom = Word(alphas).addParseAction(UnitNode.fromToks)
-			unit_atom.setName("atomic unit")
-			quoted_unit_atom = ("'" + Word(alphas) + "'"
-				).addParseAction(QuotedUnitNode.fromToks)
-			quoted_unit_atom.setName("quoted atomic unit")
+	with pyparsingWhitechars(''):
+		unit_atom = Word(alphas).addParseAction(UnitNode.fromToks)
+		unit_atom.setName("atomic unit")
+		quoted_unit_atom = ("'" + Word(alphas) + "'"
+			).addParseAction(QuotedUnitNode.fromToks)
+		quoted_unit_atom.setName("quoted atomic unit")
 
-			OPEN_P = Literal('(')
-			CLOSE_P = Literal(')')
-			SIGN = Literal('+') | Literal('-')
-			FUNCTION_NAME = Word(alphas)
-			UNSIGNED_INTEGER = Word("01234567890")
-			SIGNED_INTEGER = SIGN + UNSIGNED_INTEGER
-			FLOAT = Regex(r"[+-]?([0-9]+(\.[0-9]*)?)")
-			VOFLOAT = Regex(r"0.[0-9]+([eE][+-]?[0-9]+)?"
-				"|[1-9][0-9]*(\.[0-9]+)?([eE][+-]?[0-9]+)?")
+		OPEN_P = Literal('(')
+		CLOSE_P = Literal(')')
+		SIGN = Literal('+') | Literal('-')
+		FUNCTION_NAME = Word(alphas)
+		UNSIGNED_INTEGER = Word("01234567890")
+		SIGNED_INTEGER = SIGN + UNSIGNED_INTEGER
+		FLOAT = Regex(r"[+-]?([0-9]+(\.[0-9]*)?)")
+		VOFLOAT = Regex(r"0.[0-9]+([eE][+-]?[0-9]+)?"
+			"|[1-9][0-9]*(\.[0-9]+)?([eE][+-]?[0-9]+)?")
 
-			integer = (SIGNED_INTEGER | UNSIGNED_INTEGER).setName("integer")
-			power_operator = Literal('**')
-			multiplication_operator = Literal(".")
-			division_operator = Literal("/")
-			numeric_power = (integer
-				| OPEN_P + integer + CLOSE_P
-				| OPEN_P + FLOAT + CLOSE_P
-				| OPEN_P + integer + '/'
-					+ UNSIGNED_INTEGER.addParseAction(lambda s, p, t: t[0]+".")
-					+ CLOSE_P)
-			numeric_power.setParseAction(evalAll)
+		integer = (SIGNED_INTEGER | UNSIGNED_INTEGER).setName("integer")
+		power_operator = Literal('**')
+		multiplication_operator = Literal(".")
+		division_operator = Literal("/")
+		numeric_power = (integer
+			| OPEN_P + integer + CLOSE_P
+			| OPEN_P + FLOAT + CLOSE_P
+			| OPEN_P + integer + '/'
+				+ UNSIGNED_INTEGER.addParseAction(lambda s, p, t: t[0]+".")
+				+ CLOSE_P)
+		numeric_power.setParseAction(evalAll)
 
-			pow_10 = Literal("10") + power_operator + numeric_power
-			scale_factor = (pow_10 | VOFLOAT
-				).setName("scale_factor"
-				).setParseAction(evalAll)
+		pow_10 = Literal("10") + power_operator + numeric_power
+		scale_factor = (pow_10 | VOFLOAT
+			).setName("scale_factor"
+			).setParseAction(evalAll)
 
-			any_unit_atom = unit_atom | quoted_unit_atom
-			factor = (any_unit_atom
-				+ Optional( Suppress(power_operator) + numeric_power )
-				).setName("factor"
-				).addParseAction(Factor.fromToks)
+		any_unit_atom = unit_atom | quoted_unit_atom
+		factor = (any_unit_atom
+			+ Optional( Suppress(power_operator) + numeric_power )
+			).setName("factor"
+			).addParseAction(Factor.fromToks)
 
-			complete_expression = Forward()
-			function_application = (FUNCTION_NAME +
-					Suppress(OPEN_P) + complete_expression + Suppress(CLOSE_P)
-				).setName("function application")
-			function_application.addParseAction(FunctionApplication.fromToks)
-
-			unit_expression = (
+		complete_expression = Forward()
+		function_application = (FUNCTION_NAME +
 				Suppress(OPEN_P) + complete_expression + Suppress(CLOSE_P)
-				| ( factor
-					^ function_application )).setName("unit expression")
+			).setName("function application")
+		function_application.addParseAction(FunctionApplication.fromToks)
 
-			product_of_units = (unit_expression
-					+ ZeroOrMore(multiplication_operator + unit_expression)
-				).setName("units term"
-				).setParseAction(_buildTerm)
+		unit_expression = (
+			Suppress(OPEN_P) + complete_expression + Suppress(CLOSE_P)
+			| ( factor
+				^ function_application )).setName("unit expression")
 
-			complete_expression << (
-				product_of_units + Optional(division_operator + unit_expression) )
-			complete_expression.setParseAction(Term.fromToks)
+		product_of_units = (unit_expression
+				+ ZeroOrMore(multiplication_operator + unit_expression)
+			).setName("units term"
+			).setParseAction(_buildTerm)
 
-			input = (Optional(scale_factor)
-				+ complete_expression
-				).setName("unit expression"
-				).setParseAction(Expression.fromToks)
+		complete_expression << (
+			product_of_units + Optional(division_operator + unit_expression) )
+		complete_expression.setParseAction(Term.fromToks)
 
-			cls.symbols = locals()
-			return input
+		input = (Optional(scale_factor)
+			+ complete_expression
+			).setName("unit expression"
+			).setParseAction(Expression.fromToks)
 
-	@classmethod
-	def enableDebuggingOutput(cls):
-		"""(not user-servicable)
-		"""
-		from gavo.utils.parsetricks import ParserElement
-		for name, sym in cls.symbols.items():
+	if debugging:
+		for name, sym in locals().items():
 			if isinstance(sym, ParserElement):
 				sym.setDebug(True)
 				sym.setName(name)
+	del debugging
+	getUnitGrammar.symbols = locals()
+
+	return input
 
 
 def asSequence(unitDict):
@@ -678,7 +675,6 @@ def computeColumnConversions(newColumns, oldColumns):
 
 
 if __name__=="__main__":
-	getUnitGrammar.enableDebuggingOutput()
-	g = getUnitGrammar()
+	g = getUnitGrammar(debugging=True)
 	res = g.parseString("10**28s", parseAll=True)[0]
 	print(res)
