@@ -20,7 +20,6 @@ effort, for now it reproduces that module's interface.
 import functools
 import re
 
-from gavo import adql
 from gavo import utils
 from gavo.utils.serializers import ( #noflake: for (compatibility) export
 	ValueMapperFactoryRegistry, defaultMFRegistry, AnnotatedColumn,
@@ -189,6 +188,15 @@ def needsQuoting(identifier, forRowmaker=False):
 	return False
 
 
+@functools.cache
+def getADQLReservedWords():
+	# as a cached function because the adql module wants a fully initialised
+	# database connection (in ufunctions), and we may not have that when
+	# valuemappers is imported.
+	from gavo import adql
+	return adql.ALL_RESERVED_WORDS
+
+
 @functools.lru_cache(2)
 def getNameBlacklist(forRowmaker=False):
 	"""returns a set of names not suitable for table column names.
@@ -197,7 +205,7 @@ def getNameBlacklist(forRowmaker=False):
 	is true, also some names damaging row makers (e.g. python reserved
 	words).
 	"""
-	res = set(k.lower() for k in adql.ALL_RESERVED_WORDS)
+	res = set(k.lower() for k in getADQLReservedWords())
 	if forRowmaker:
 		import keyword
 		from gavo.rscdef import rmkfuncs
