@@ -6,7 +6,7 @@ of DaCHS tables is an ObjectAnnotation, the other Annotation classes
 (conceptually, all are key-value pairs) make up their inner structure.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -117,7 +117,7 @@ class AnnotationBase(object):
 
 
 class TableRelativeAnnotation(AnnotationBase):
-	"""A base class for annotations that must be adapted or discarded when 
+	"""A base class for annotations that must be adapted or discarded when
 	an annotation is copied.
 	"""
 	# This currently is only used as a sentinel for such annotations.
@@ -129,14 +129,14 @@ class AtomicAnnotation(AnnotationBase):
 
 	These can take optional metadata.
 	"""
-	def __init__(self, name=None, value=None, unit=None, 
+	def __init__(self, name=None, value=None, unit=None,
 			ucd=None, instance=None):
 		AnnotationBase.__init__(self, name, instance)
 		self.value, self.unit, self.ucd = value, unit, ucd
 
 	def copy(self, newInstance):
 		return self.__class__(
-			self.name, self.value, self.unit, self.ucd, 
+			self.name, self.value, self.unit, self.ucd,
 			newInstance)
 
 	def getVOT(self, ctx, instance):
@@ -148,7 +148,7 @@ class AtomicAnnotation(AnnotationBase):
 				"ucd": self.ucd})
 
 			param = V.PARAM(name=self.name,
-				id=ctx.getOrMakeIdFor(self.value), 
+				id=ctx.getOrMakeIdFor(self.value),
 					dmrole=completeVODMLId(ctx, self.name),
 					**attrs)
 			ctx.getEnclosingContainer()[
@@ -197,12 +197,12 @@ class _WithMapCopyMixin(object):
 		not mentioned in annotationMap) will be discarded.
 
 		This is used when annotation tables with columns copied from
-		other tables.  annotationMap, normally generated by 
+		other tables.  annotationMap, normally generated by
 		dmrd.SynthesizedRoles, then maps the old annotations to the elements
 		that should be annotated in the new table.
 		"""
 		copy = self.copyEmpty(instance)
-		if instance is None: # I'm the new root 
+		if instance is None: # I'm the new root
 			instance = copy
 
 		for role in self.iterChildRoles():
@@ -312,6 +312,8 @@ class ObjectAnnotation(_AttributeGroupAnnotation):
 	Objects are used for actual DM instances.  In particular,
 	every annotation of a DaCHS table is rooted in an object.
 	"""
+	def __repr__(self):
+		return f"<ObjectAnnotation {self.type}>"
 
 
 class CollectionAnnotation(AnnotationBase, _WithMapCopyMixin):
@@ -366,7 +368,7 @@ class CollectionAnnotation(AnnotationBase, _WithMapCopyMixin):
 		res = V.COLLECTION()
 		for c in self.children:
 			with ctx.settingAttribute(
-					V.ITEM()) as item:
+					V.ATTRIBUTE()) as item:
 				item[c.getVOT(ctx, instance)]
 				res[item]
 
