@@ -3,11 +3,11 @@ Writing annotations in RDs.
 
 This module provides the glue between annotations (typically in SIL)
 and the rest of the RDs.  It provides the ResAnnotation struct, which
-contains the SIL, and the makeAttributeAnnotation function at is a factory
+contains the SIL, and the makeAttributeAnnotation function that is a factory
 for attribute annotations.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -20,6 +20,12 @@ from gavo import base
 from gavo.dm import annotations
 from gavo.dm import common
 from gavo.dm import sil
+from gavo.utils import stanxml
+from gavo.utils import algotricks
+
+
+stanxml.registerPrefix("vo-dml", "http://www.ivoa.net/xml/VODML/v1",
+	stanxml.schemaURL("vo-dml.xsd"))
 
 
 class SynthesizedRoles(base.Structure):
@@ -48,10 +54,9 @@ class SynthesizedRoles(base.Structure):
 		# side effect, the dmRoles of the new items are cleared.
 		for item in itertools.chain(self.parent.params, self.parent.columns):
 			if item.parent!=self.parent:
-				# Don't annotate things that don't belong to us.
-				# This is going to hide problems big time.  But touching these
-				# certainly is worse, and if people are missing annotations,
-				# they'll probably notice they have to copy these.
+				# Don't annotate things that don't belong to us.  We need to
+				# be careful there because in the current architecture, we
+				# would be changing dmRoles
 				continue
 
 			if isinstance(item.dmRoles, list):
@@ -60,7 +65,11 @@ class SynthesizedRoles(base.Structure):
 				continue
 
 			for annotation in item.dmRoles.oldRoles:
-				annotationMap[annotation()] = item
+				# annotation is a weakref here; skip the annotation if the
+				# referenced thing has gone away
+				resolved = annotation()
+				if resolved is not None:
+					annotationMap[resolved] = item
 			item.dmRoles = []
 
 		# Then, collect the annotations we have to copy
@@ -86,11 +95,14 @@ class DataModelRoles(base.Structure):
 	The content of this element is a Simple Instance Language clause.
 	"""
 
-# We defer the parsing of the contained element to (hopefully) the
-# end of the parsing of the RD to enable forward references with
-# too many headaches (stubs don't cut it: we (may) need to know types).
-# 
-# There's an additional complication in that we may want to 
+# We used to defer the SIL parsing until the end of the XML parse
+# in order to be relaxed about forward references.  That, however,
+# is not a good idea because later elements in the RD may copy the table,
+# and its annotations will be incomplete then.  Instead, we're now
+# building the annotation when the element is complete and worry
+# about cross-table forward references when they turn up.
+#
+# There's an additional complication in that we may want to
 # access parsed annotations while parsing other annotations
 # (e.g., when processing foreign keys).
 # To allow the thing to "parse itself" in such situations, we do
@@ -103,33 +115,30 @@ class DataModelRoles(base.Structure):
 	def getAnnotation(self, roleName, container, instance):
 		return annotations.GroupRefAnnotation(roleName, self.parse(), instance)
 
-	def completeElement(self, ctx):
-		def _buildAnnotation():
-			try:
-				self._parsedAnnotation = sil.getAnnotation(
-					self.content_, getAnnotationMaker(self.parent))
-				self.parent.annotations.append(self._parsedAnnotation)
-			except Exception as ex:
-				base.ui.notifyInfo("While parsing the SIL below: %s"%str(ex))
-				base.ui.notifyInfo(self.content_)
-				raise base.ui.logOldExc(base.StructureError(str(ex),
-					pos=self.getSourcePosition()))
-			self._buildAnnotation = lambda: None
-		self._buildAnnotation = _buildAnnotation
-
-		ctx.addExitFunc(lambda rd, ctx: self._buildAnnotation())
-		super().completeElement(ctx)
+	def onParentComplete(self):
+		self.parent.annotations.append(self.parse())
 
 	def parse(self):
 		"""returns a parsed version of the embedded annotation.
 
-		Do not call this while the RD is still being built, as dm
-		elements may contain forward references, and these might
-		not yet be available during the parse.
+		The parse really is only run once, but you should only call it once
+		the embedding table is complete, because this needs to resolve table
+		params and columns (and possibly other dm elements).
 		"""
-		self._buildAnnotation()
+		if hasattr(self, "_parsedAnnotation"):
+			return self._parsedAnnotation
+
+		try:
+			self._parsedAnnotation = sil.getAnnotation(
+				self.content_, getAnnotationMaker(self.parent))
+		except Exception as ex:
+			base.ui.notifyInfo("While parsing the SIL below: %s"%str(ex))
+			base.ui.notifyInfo(self.content_)
+			raise base.ui.logOldExc(base.StructureError(str(ex),
+				pos=self.getSourcePosition()))
 		return self._parsedAnnotation
 
+
 	def copy(self, newParent, ctx):
 		# we use the general mechanism used for recovering annotations from
 		# columns and params in tables here so we're independent of
@@ -218,3 +227,89 @@ def getAnnotationMaker(container):
 	within container.
 	"""
 	return functools.partial(makeAttributeAnnotation, container)
+
+
+def _splitPath(path:str):
+	"""returns a pair of (prefix)/(stuff-without-slash) for path.
+	"""
+	splitpoint = path.rfind("/")
+	if splitpoint==-1:
+		return "", path
+	else:
+		return path[:splitpoint], path[splitpoint+1:]
+
+
+def _makeDMVal(stcObj):
+	"""returns a value palatable to makeAttributeAnnotation from stcObj
+
+	(which essentially will be an set.ColRef or some literal).
+	"""
+	if hasattr(stcObj, "dest"):
+		return sil.Reference(stcObj.dest)
+	
+	else:
+		return sil.Atom(stcObj)
+
+
+def buildAdhocSTC(table, vals):
+	"""builds a modern adhoc STC annotation in table based on a mapping
+	of attribute paths to annotation targets in vals.
+
+	This is only necessary as long as we want to support <stc> specs.  The
+	keys here are, in effect defined in rscdef.tabledef; see _utypeToModern.
+	"""
+	if vals.pop("!!!flavor!!!", None)!="SPHERICAL":
+		# anything else would need a lot more thought, and that's certainly not
+		# worth it.  STC-S should die.
+		return
+
+	# Sigh. STC1 split off the J/B from the epoch value.  Let's cope in
+	# some way.
+	if vals.pop("!!!yearDef!!!", "J")!="J":
+		raise NotImplementedError("We can only deal with Julian epochs")
+	if "/space/frame/epoch" in vals:
+		if isinstance(vals["/space/frame/epoch"], str):
+			vals["/space/frame/epoch"] = "J"+vals["/space/frame/epoch"]
+
+	annMaker = getAnnotationMaker(table)
+	# astid is the id of the STC1 AST that has produced the original utypes.
+	# We want to record this because in votablewrite we want to re-use the
+	# COOSYS that already exists when there is an STC1 annotation.
+	astid = None
+	objs = {}
+
+	for path in algotricks.pathSort(vals.keys()):
+		prefix, attname = _splitPath(path)
+		parent = objs[prefix] if path else None
+
+		if path in buildAdhocSTC.typesForPaths:
+			# it's an intermediate object: create it
+			objs[path] = common.ObjectAnnotation(
+				attname, buildAdhocSTC.typesForPaths[path], objs.get(""))
+			if parent is not None:
+				parent.add(objs[path])
+
+		else:
+			# it's an attribute; translate stc's references and literals
+			parent.add(annMaker(objs[""], attname, _makeDMVal(vals[path])))
+			if astid is None:
+				# if value is a ColRef, whatever it refers to will define our
+				# STC1 AST
+				try:
+					destCol = table.getElementForName(vals[path].dest)
+					astid = str(id(destCol.stc))
+					objs[""].add(annMaker(objs[""], "astid", sil.Atom(astid)))
+				except (base.NotFoundError, AttributeError):
+					# not a ColRef, apparently, try next val
+					pass
+
+	table.annotations.append(objs[""])
+
+
+buildAdhocSTC.typesForPaths = {
+	"": "votable:Coords",
+	"/space": "votable:SphericalCoordinate",
+	"/space/frame": "votable:SpaceFrame",
+	"/time": "votable:TimeCoordinate",
+	"/time/frame": "votable:TimeFrame",
+}
