@@ -9,6 +9,7 @@ data model instances written in a simple, JSON-like language.
 #c COPYING file in the source distribution.
 
 
+import functools
 import re
 
 from gavo import utils
@@ -80,72 +81,67 @@ def _pa_nullLiteral(s, p, toks):
 	return [None]
 
 
-class getGrammar(utils.CachedResource):
+@functools.cache
+def getGrammar(debug=False):
 	"""returns a grammar for parsing a SIL object description.
 	"""
-	@classmethod
-	def impl(cls):
-		from gavo.utils.parsetricks import (Word, Literal, alphas, alphanums,
-			QuotedString, Forward, ZeroOrMore, Group, Optional, cStyleComment,
-			pyparsingWhitechars)
+	from gavo.utils.parsetricks import (Word, Literal, alphas, alphanums,
+		QuotedString, Forward, ZeroOrMore, Group, Optional, cStyleComment,
+		pyparsingWhitechars, ParserElement)
 
-		with pyparsingWhitechars("\t\n\r "):
-			qualifiedIdentifier = Word(alphas+"_:", alphanums+"-._:")
-			plainIdentifier = Word(alphas+"_", alphanums+"-._")
-			externalIdentifier = Word(alphas+"_", alphanums+"._/#-")
-			plainLiteral = Word(alphanums+"_-.")
-			quotedLiteral = QuotedString(quoteChar='"', escQuote='""')
-			nullLiteral = Literal("__NULL__")
-			reference = (Literal('@') + externalIdentifier)
+	with pyparsingWhitechars("\t\n\r "):
+		qualifiedIdentifier = Word(alphas+"_:", alphanums+"-._:")
+		plainIdentifier = Word(alphas+"_", alphanums+"-._")
+		externalIdentifier = Word(alphas+"_", alphanums+"._/#-")
+		plainLiteral = Word(alphanums+"_-.")
+		quotedLiteral = QuotedString(quoteChar='"', escQuote='""')
+		nullLiteral = Literal("__NULL__")
+		reference = (Literal('@') + externalIdentifier)
 
-			complexImmediate = Forward()
-			simpleImmediate = plainLiteral | quotedLiteral
-			value = (nullLiteral | reference | complexImmediate | simpleImmediate)
+		complexImmediate = Forward()
+		simpleImmediate = plainLiteral | quotedLiteral
+		value = (nullLiteral | reference | complexImmediate | simpleImmediate)
 
-			valueWithFallback = (value
-				+ "|"
-				- value)
-			attributeDef = (plainIdentifier
-				+ Literal(":")
-				+ (valueWithFallback | value))
+		valueWithFallback = (value
+			+ "|"
+			- value)
+		attributeDef = (plainIdentifier
+			+ Literal(":")
+			+ (valueWithFallback | value))
 
-			typeAnnotation = (Literal('(')
-				+ qualifiedIdentifier
-				+ Literal(')'))
-			objectBody = (Literal('{')
-				+ Group(ZeroOrMore( attributeDef ))
-				+ Literal('}'))
-			obj = Optional(typeAnnotation) + objectBody
+		typeAnnotation = (Literal('(')
+			+ qualifiedIdentifier
+			+ Literal(')'))
+		objectBody = (Literal('{')
+			+ Group(ZeroOrMore( attributeDef ))
+			+ Literal('}'))
+		obj = Optional(typeAnnotation) + objectBody
 
-			sequenceBody = (Literal('[')
-				+ Group(ZeroOrMore(
-					valueWithFallback | value | objectBody))
-				+ Literal(']'))
-			collection = Optional(typeAnnotation) + sequenceBody
+		sequenceBody = (Literal('[')
+			+ Group(ZeroOrMore(
+				valueWithFallback | value | objectBody))
+			+ Literal(']'))
+		collection = Optional(typeAnnotation) + sequenceBody
 
-			complexImmediate << ( obj | collection )
+		complexImmediate << ( obj | collection )
 
-			for sym in [complexImmediate, collection, sequenceBody,
-					objectBody, typeAnnotation, attributeDef]:
-				sym.ignore(cStyleComment)
+		for sym in [complexImmediate, collection, sequenceBody,
+				objectBody, typeAnnotation, attributeDef]:
+			sym.ignore(cStyleComment)
 
-		for n, func in globals().items():
-			if n.startswith("_pa_"):
-				locals()[n[4:]].setParseAction(func)
+	for n, func in globals().items():
+		if n.startswith("_pa_"):
+			locals()[n[4:]].setParseAction(func)
 
-		cls.symbols = locals()
-		return obj
-
-	@classmethod
-	def enableDebuggingOutput(cls):
-		"""(not user-servicable)
-		"""
-		from gavo.utils.parsetricks import ParserElement
-		for name, sym in cls.symbols.items():
+	if debug:
+		for name, sym in locals().items():
 			if isinstance(sym, ParserElement):
 				sym.setDebug(True)
 				sym.setName(name)
 
+	return obj
+
+
 
 def _getAttributeEvents(attrNode):
 	"""returns a list of events for an attr node in the parse tree.
