@@ -1,11 +1,12 @@
 """
-Parsing VO-DML files and validating against the rules obtained in this way.
+Parsing VO-DML files and (perhaps one day) validating against the
+rules obtained in this way.
 
 Validation is something we expect to do only fairly rarely, so none of
 this code is expected to be efficient.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -13,23 +14,20 @@ this code is expected to be efficient.
 
 import functools
 
+from lxml import etree
+
 from gavo import base
-from gavo.utils import ElementTree
 from gavo.votable import V
 
 
 KNOWN_MODELS = {
-# maps the canonical prefix to the file name within resources/dm and
-# (for now) the canonical URI (which isn't available anywhere else so far).
-	"NDcube": ("CubeDM-1.0.vo-dml.xml", 
-		"http://www.ivoa.net/dm/CubeDM-1.0.vo-dml.xml"),
-	"ds": ("DatasetMetadata-1.0.vo-dml.xml",
-		"http://www.ivoa.net/dm/DatasetMetadata-1.0.vo-dml.xml"),
-	"ivoa": ("IVOA.vo-dml.xml", "http://www.ivoa.net/dm/ivoa.vo-dml.xml"),
-	"vo-dml": ("VO-DML.vo-dml.xml", "http://www.ivoa.net/dm/VO-DML.vo-dml.xml"),
-	"dachstoy": ("dachstoy.vo-dml.xml", "http://docs.g-vo.org/dachstoy"),
-	"stc2": ("STC.vo-dml.xml", "http://www.ivoa.net/dm/STC.vo-dml.xml"),
-	"geojson": ("geojson.vo-dml.xml", "http://docs.g-vo.org/geojson"),
+# maps the canonical prefix to the file name within resources/dm
+	"ivoa": "IVOA.vo-dml.xml",
+	"meas": "meas.vo-dml.xml",
+	"coords": "coords.vo-dml.xml",
+	"dachstoy": "dachstoy.vo-dml.xml",
+	"geojson": "geojson.vo-dml.xml",
+	"phot": "phot.vo-dml.xml",
 }
 
 
@@ -39,7 +37,7 @@ def openModelFile(prefix):
 	This will raise a NotFoundError for an unknown prefix.
 	"""
 	try:
-		fName, _ = KNOWN_MODELS[prefix]
+		fName = KNOWN_MODELS[prefix]
 	except KeyError:
 		raise base.NotFoundError(prefix, "VO-DML file for prefix",
 			"data models known to DaCHS", hint="This can happen if there"
@@ -58,7 +56,7 @@ class Model(object):
 	to populate the model.
 	"""
 
-	# non-well-known models can be fed in through fromFile; they well
+	# non-well-known models can be fed in through fromFile; they will
 	# be entered here and can then be obtained through fromPrefix
 	# as long as the don't clash with KNOWN_MODELS.
 
@@ -67,12 +65,11 @@ class Model(object):
 	def __init__(self, prefix, dmlTree):
 		self.prefix = prefix
 		self.title = self.version = None
-		self.version = self.url = None
+		self.version = self.uri = None
 		self.description = None
 		self.dmlTree = dmlTree
 		self.__idIndex = None
-		if self.dmlTree:
-			self._getModelMeta()
+		self._getModelMeta()
 
 	@classmethod
 	def fromPrefix(cls, prefix):
@@ -86,10 +83,7 @@ class Model(object):
 		inF = openModelFile(prefix)
 		try:
 			try:
-				res = cls(prefix, ElementTree.parse(inF))
-				# as long as we can't get the URL from the XML, patch it in here
-				res.url = KNOWN_MODELS[prefix][1]
-				return res
+				return cls(prefix, etree.parse(inF))
 			except Exception as ex:
 				raise base.ui.logOldExc(
 					base.StructureError("Failure to parse VO-DML for prefix %s: %s"%(
@@ -114,10 +108,10 @@ class Model(object):
 			inF = openModelFile(src)
 
 		try:
-			tree = ElementTree.parse(inF)
+			tree = etree.parse(inF)
 			prefix = tree.find("name").text
 			res = cls(prefix, tree)
-			res.url = srcURL
+			res.uri = srcURL
 
 			if prefix not in KNOWN_MODELS:
 				cls._modelsReadFromFile[prefix] = res
@@ -125,27 +119,6 @@ class Model(object):
 		finally:
 			inF.close()
 
-	@property
-	def idIndex(self):
-		"""returns a dictionary mapping vodmlids to elementtree objects.
-		"""
-		if self.__idIndex is None:
-			self.__idIndex = self._createIndex()
-		return self.__idIndex
-	
-	def _createIndex(self):
-		"""returns a dictionary mapping vodml-ids to elementtree objects.
-
-		Use the idIndex property rather than this function, as the former will 
-		cache the dicts.
-		"""
-		res = {}
-		for element in self.dmlTree.getroot().iter():
-			id = element.find("vodml-id")
-			if id is not None:
-				res[id.text] = element
-		return res
-
 	def _getModelMeta(self):
 		"""sets some metadata on the model from the parsed VO-DML.
 
@@ -158,81 +131,113 @@ class Model(object):
 			self.title = self.dmlTree.find("title").text
 			self.version = self.dmlTree.find("version").text
 			self.description = self.dmlTree.find("description").text
+			self.uri = self.dmlTree.find("uri").text
 		except AttributeError:
 			# probably the VO-DML file is bad; just fall through to
 			# non-validatable model.
 			pass
 
-	def _resolveNonLocalVODMLId(self, id):
-		"""returns an etree Element pointed to by the VO-DML id 
-
-		This is a helper for getByVODMLId and works by successively
-		trying shorter pieces of id.
-
-		This returns None on a failure rather than raising an exception
-		(because it's really a helper for getByVODMLId).
-		"""
-		parts = id.split(".")
-		for splitPoint in range(len(parts)-1, 0, -1):
-			newId = ".".join(parts[:splitPoint])
-			if newId in self.idIndex:
-				# this should be an attribute definition.  Now follow
-				# the chain of attribute names to the end
-				att = self.idIndex[newId]
-				thisType = resolveVODMLId(
-					att.find("datatype").find("vodml-ref").text)
-
-				for attName in parts[splitPoint:]:
-					att = getAttributeDefinition(thisType, attName)
-					thisType = resolveVODMLId(
-						att.find("datatype").find("vodml-ref").text)
-				return att
-		# fall through on failure
-
+	@functools.lru_cache(200)
 	def getByVODMLId(self, vodmlId):
 		"""returns the element with vodmlId.
 
 		This raises a NotFoundError for elements that are not present.
 
-		This can be used with or without the prefix.  The prefix is not
-		validated, though.
+		This can be used with or without a prefix.  The prefix is just
+		discarded, though.
+
+		Do not pass in unparsed ids; they are used in xpaths.
 		"""
-		if ":" in vodmlId:
-			vodmlId = vodmlId.split(":", 1)[1]
+		vodmlId = vodmlId.split(":")[-1]
 
-		# We may have to follow ids through several documents based on types.
-		# First try to directly find the element.
-		if vodmlId in self.idIndex:
-			return self.idIndex[vodmlId]
-
-		res = self._resolveNonLocalVODMLId(vodmlId)
+		res = self.dmlTree.xpath(f"//*[vodml-id='{vodmlId}']")
 		if res:
-			return res
+			return res[0]
 		else:
 			raise base.NotFoundError(vodmlId, "data model element",
 				self.prefix+" data model")
 
-	def getAttributeMeta(self, vodmlId):
-		"""returns a metadata dictionary for a VO-DML element with vodmlId.
+	def _resolveVODMLId(self, vodmlId):
+		"""returns an etree element for vodmlId, which may include a prefix.
 
-		This includes datatype add description.  If vodmlId points to
-		the value of a quantity, the associate unit and ucd attributes
-		are returned as well.
+		(in which case we'd probably be looking in a different DM).
 
-		If the vodmlId cannot be found, a NotFoundError is raised.
+		This will raise a NotFoundError when the vodmlId points nowhere.
 		"""
-		raise NotImplementedError("We've not yet figured out how this is"
-			" supposed to work.")
+		if ":" in vodmlId:
+			return resolveVODMLId(vodmlId)
+		else:
+			return self.getByVODMLElement(vodmlId)
+	
+	def _makeAttrDict(self, attrNode):
+		"""returns a dictionary of attribute metadata for an etree attrNode.
+		"""
+		res = {}
+		for child in attrNode:
+			if child.tag=="vodml-id":
+				res["vodml-id"] = child.text.strip()
+			elif child.tag=="description":
+				res["description"] = child.text.strip()
+			elif child.tag=="datatype":
+				res["datatype"] = child[0].text
+		return res
+
+	@functools.lru_cache(100)
+	def getType(self, name):
+		"""returns an etree for a data or object type with the *name* name
+		within this DM.
+
+		Any prefix on name will be discarded without further ado.  Don't
+		pass in anythin unparsed here; we are using xpaths.
+		"""
+		name = name.split(":")[-1]
+		type = self.dmlTree.xpath(f"dataType[name='{name}']")
+		if type:
+			return type[0]
+
+		type = self.dmlTree.xpath(f"objectType[name='{name}']")
+		if type:
+			return type[0]
+
+		raise base.NotFoundError(
+			name, "VO-DML type", self.prefix+" data model")
+
+	@functools.lru_cache(1000)
+	def getAttributeMeta(self, typeName, attrName):
+		"""returns a metadata dictionary for a the attribute attrName of the
+		type referenced by type name.
+
+		attrName is the unqualified attribute name.  typeName may be qualified,
+		but prefixes are ignored.
+
+		Do not pass in unparsed strings to typeName and attrName; they are
+		blindly used in xpath expressions.
+
+		The metadata returned includes datatype, description, and vodmlId.
+
+		If either the data model, or type, or the attribute cannot be
+		found a NotFoundError is raised.
+		"""
+		type = self.getType(typeName)
+
+		while True:
+			attr = type.xpath(f"(attribute|composition)[name='{attrName}']")
+			if attr:
+				return self._makeAttrDict(attr[0])
+			extends = type.xpath("extends/vodml-ref")
+			if extends:
+				type = self._resolveVODMLId(extends[0].text)
+			else:
+				raise base.NotFoundError(
+					attrName, "attribute", f"VO-DML type {typeName}")
 
 	def getVOT(self, ctx, instance):
 		"""returns xmlstan for a VOTable declaration of this DM.
 		"""
-		return V.MODEL[
-			V.NAME(version=self.version)[self.prefix],
-			V.URL[self.url]]
+		return V.MODEL(name=self.prefix, url=self.uri)
 
 
-@functools.lru_cache()
+@functools.lru_cache(30)
 def getModelForPrefix(prefix):
 	"""returns a vodml.Model instance for as well-known VODML prefix.
 
@@ -246,25 +251,24 @@ def getModelForPrefix(prefix):
 	try:
 		return Model.fromPrefix(prefix)
 	except base.NotFoundError:
-		res = Model(prefix, ElementTree.fromstring(
+		res = Model(prefix, etree.fromstring(
 			"""<junk><title>DaCHS standin model</title>
 				<description>This is used by DaCHS during the old west
 				days of VO DM development.  Any annotation using this will
 				not be interoperable.</description>
 				<version>invalid</version></junk>"""))
-		res.url = "urn:dachsjunk:not-model:"+prefix
+		res.uri = "urn:dachsjunk:not-model:"+prefix
 		return res
 
-def getAttributeDefinition(typeDef, attName):
-	"""returns the attribute definition for attName in typeDef as an etree.
 
-	This raises a NotFoundError if the attribute is not found.
+def getAttributeDefinition(qualifiedType, attrName):
+	"""returns attribute metadata for a type.
+
+	qualifiedType is a type name with a prefix, attrName is the attribute's
+	name (not vodml-id).
 	"""
-	for attribute in typeDef.findall("attribute"):
-		if attribute.find("name").text==attName:
-			return attribute
-	raise base.NotFoundError(attName, "Attribute", 
-		"VO-DML type "+typeDef.find("name").text)
+	prefix, typeName = qualifiedType.split(":")
+	return getModelForPrefix(prefix).getAttributeMeta(typeName, attrName)
 
 
 def resolveVODMLId(vodmlId):
