@@ -442,7 +442,7 @@ class TemplatedPage(resource.Resource, CommonRenderers):
 
     def handleError(self, failure, request):
         failure.printTraceback()
-        request.write("Uncaught error:\n<![CDATA[\n{}\n]]>\n"
+        request.write("Uncaught error:\n<pre><![CDATA[\n{}\n]]></pre>\n"
             .format(failure.getErrorMessage())
             .encode("utf-8"))
         try:
