@@ -2,11 +2,12 @@
 Helpers for testing code using gavo.formal
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import io
 import math
 
@@ -33,6 +34,14 @@ def bytify(s):
     return s
 
 
+def debytify(b):
+    """returns b utf-8 decoded if it is bytes, unmodified otherwise.
+    """
+    if isinstance(b, bytes):
+        return b.decode("utf-8")
+    return b
+
+
 def assertHasStrings(content, strings, inverse=False):
     """asserts that all strings in the list strings are in content.
 
@@ -62,45 +71,6 @@ def raiseException(failure):
     failure.raiseException()
 
 
-class FakeFile:
-    def __init__(self, filename, fileobj):
-        self.filename, self.file = filename, fileobj
-        if isinstance(self.file, bytes):
-            self.file = io.BytesIO(self.file)
-
-
-class FakeFieldStorage(object):
-    filename = None
-    def __init__(self, args):
-        self.args = {}
-        for k, vals in args.items():
-            res = self.args[k.decode("utf-8")] = []
-            for v in vals:
-                if isinstance(v, bytes):
-                    res.append(v.decode("utf-8"))
-                else:
-                    res.append(FakeFile(*v))
-
-    def __iter__(self):
-        return iter(self.args)
-    
-    def getfirst(self, key):
-        return self.args[key][0]
-    
-    def getlist(self, key):
-        return self.args[key]
-
-    def __getitem__(self, key):
-        return self.args[key][0]
-
-    def keys(self):
-        return self.args.keys()
-
-    @property
-    def file(self):
-        return io.StringIO(self.args[0])
-
-
 def bytify_seq(s):
     """returns a list from s with any elements bytified.
 
@@ -116,6 +86,17 @@ def bytify_seq(s):
     return [bytify(item) for item in s]
 
 
+class FakeFile:
+  """a fake file upload.
+
+  Construct this with a file name and a payload, both of which can be
+  bytes or str (which will be utf-8-encoded).
+  """
+  def __init__(self, file_name, payload):
+    self.file_name = bytify(file_name)
+    self.file_object = io.BytesIO(bytify(payload))
+
+
 class FakeRequest(DummyRequest):
     """A request for test purposes.
 
@@ -124,6 +105,12 @@ class FakeRequest(DummyRequest):
 
     Also, stock twisted DummyRequest produces an endless loop with push
     producers (which is what we have), so we fix that, too.
+
+    You can pass in args as a str -> str mapping; the strings will be
+    encoded as utf-8 so request.args is bytes -> [bytes].  For
+    convenience, we'll turn single values to lists.
+
+    For uploads, you can pass (single) args with FakeFile-valued arguments.
     """
 
     method = b"GET"
@@ -145,9 +132,14 @@ class FakeRequest(DummyRequest):
 
         DummyRequest.__init__(self, uri)
 
+        self.files, self.args = {}, {}
         if args:
-            self.args = dict((bytify(k), bytify_seq(v)) 
-                for k, v in args.items())
+            for k,v in args.items():
+                if isinstance(v, FakeFile):
+                    self.files[debytify(k)] = [v]
+
+                else:
+                    self.args[bytify(k)] = bytify_seq(v)
         else:
             self.args = {}
 
@@ -158,7 +150,6 @@ class FakeRequest(DummyRequest):
         self.deferred = defer.Deferred()
         self.accumulator = b""
         self.prepath = []
-        self.fields = FakeFieldStorage(self.args)
         self.finished = False
         self.secure = False
         self.channel = 1 # must be non-None for custom hangup detection
@@ -172,7 +163,7 @@ class FakeRequest(DummyRequest):
         if not self.startedWriting:
             if self.lastModified is not None:
                 self.responseHeaders.setRawHeaders(
-                    b"last-modified", 
+                    b"last-modified",
                     [http.datetimeToString(self.lastModified)])
             self.startedWriting = True
 
@@ -214,7 +205,7 @@ class FakeRequest(DummyRequest):
 
     def setHost(self, host, port):
         self.host = host
-    
+
     def getHost(self):
         return self.host
 
@@ -274,10 +265,14 @@ class FakeRequest(DummyRequest):
         self.go = 0
         self.channel = None
         del self.producer
-    
+
     def getResponseHeader(self, headerName):
         return self.responseHeaders.getRawHeaders(headerName, [None])[0]
 
+    def addUpload(self, name, content):
+        self.files.setdefault(name, []).append(
+            FakeFile(name, content))
+
 
 def _doRender(page, request):
     result = page.render(request)
@@ -297,20 +292,13 @@ def _doRender(page, request):
 
 
 def _buildRequest(
-        method, 
-        path, 
-        rawArgs, 
-        moreHeaders=None, 
+        method,
+        path,
+        args,
+        moreHeaders=None,
         requestClass=None):
     if requestClass is None:
         requestClass = FakeRequest
-
-    args = {}
-    for k, v in rawArgs.items():
-        if isinstance(v, list):
-            args[k] = v
-        else:
-            args[k] = [v]
     req = requestClass(path, args=args)
 
     req.headers = {}
@@ -322,11 +310,11 @@ def _buildRequest(
     return req
 
 
-def runQuery(page, 
-        method, 
-        path, 
-        args, 
-        moreHeaders=None, 
+def runQuery(page,
+        method,
+        path,
+        args,
+        moreHeaders=None,
         requestMogrifier=None,
         requestClass=None,
         produceErrorDocument=None):
@@ -363,7 +351,7 @@ class RenderTest(TrialTest):
     errorHandler = None # override with a runQuery produceErrorDocument
     runQuery = staticmethod(runQuery)
 
-    def assertStringsIn(self, result, strings, inverse=False, 
+    def assertStringsIn(self, result, strings, inverse=False,
             customTest=None):
         # this wraps testhelpers.assertHasStrings to work better with
         # twisted results; in particular, we need to return the result.
@@ -377,12 +365,12 @@ class RenderTest(TrialTest):
                 f.write(content)
             raise
         return result
-    
-    def assertResultHasStrings(self, method, path, args, strings, 
+
+    def assertResultHasStrings(self, method, path, args, strings,
             rm=None, inverse=False, customTest=None):
         return self.runQuery(
-            self.renderer, method, 
-            path, args, 
+            self.renderer, method,
+            path, args,
             requestMogrifier=rm,
             produceErrorDocument=self.errorHandler
             ).addCallback(self.assertStringsIn, strings, inverse=inverse,
@@ -390,11 +378,11 @@ class RenderTest(TrialTest):
 
     def assertGETHasStrings(self, path, args, strings, rm=None,
             customTest=None):
-        return self.assertResultHasStrings("GET", 
+        return self.assertResultHasStrings("GET",
             path, args, strings, rm=rm, customTest=customTest)
 
     def assertGETLacksStrings(self, path, args, strings, rm=None):
-        return self.assertResultHasStrings("GET", 
+        return self.assertResultHasStrings("GET",
             path, args, strings, rm=rm, inverse=True)
 
     def assertPOSTHasStrings(self, path, args, strings, rm=None):
@@ -407,7 +395,7 @@ class RenderTest(TrialTest):
             return res
         return self.runQuery(
             self.renderer, "GET",
-            path, args, 
+            path, args,
             requestMogrifier=rm,
             produceErrorDocument=self.errorHandler
             ).addCallback(check)
@@ -421,14 +409,14 @@ class RenderTest(TrialTest):
             if alsoCheck is not None:
                 alsoCheck(flr)
 
-        return self.runQuery(self.renderer, "GET", 
+        return self.runQuery(self.renderer, "GET",
             path, args,
             produceErrorDocument=self.errorHandler
             ).addCallback(cb
             ).addErrback(eb)
 
     def assertGETIsValid(self, path, args={}):
-        return self.runQuery(self.renderer, "GET", 
+        return self.runQuery(self.renderer, "GET",
             path, args,
             produceErrorDocument=self.errorHandler
             ).addCallback(self.assertResponseIsValid)
