@@ -414,19 +414,27 @@ class MPRequest(server.Request):
 
         if self.method == b"POST" and ctype:
             chunk_size = 1048576
+            strippedCtype, ctypeParams = multipart.parse_options_header(ctype)
+            if strippedCtype.lower()==b'application/x-www-form-urlencoded':
+                unquote = urlparse.unquote_plus
+            else:
+                unquote = urlparse.unquote
 
             def addArg(field):
-                # The string/byte situation on urlparse.unquote* is confusingly
-                # bizarre.  If I want to be compatible with baseline twisted
-                # args, I don't think I get around the odd quote/dequote thing
-                # I'm doing here; and yes, it's latin1 on the incoming string,
-                # as any non-ascii in there probably isn't utf-8.
+                # The encoding situation is ugly on the incoming side.
+                # It's likely utf-8, but I don't want to fail hard
+                # if it's not.  And I re-encode for compatibility
+                # with twisted.web's args.
                 if field.value is None:
                     # ignore empty parameters coming in (it's what cgi did, too)
                     return
+                try:
+                    value = field.value.decode("utf-8")
+                except UnicodeDecodeError:
+                    value = field.value.decode("iso-8859-1")
+
                 self.args.setdefault(field.field_name, []).append(
-                    urlparse.unquote_plus(
-                        field.value.decode("latin-1")).encode("utf-8"))
+                    unquote(value).encode("utf-8"))
 
             def addFile(file_obj):
                 # some versions of multipart pass in the files already
@@ -441,7 +449,6 @@ class MPRequest(server.Request):
             total_read = 0
 
             try:
-                strippedCtype, ctypeParams = multipart.parse_options_header(ctype)
                 parser = multipart.FormParser(
                     strippedCtype.decode("latin-1"), addArg, addFile,
                     boundary=ctypeParams.get(b"boundary"))
