@@ -371,7 +371,7 @@ class SelectChoice(ChoiceBase):
                 optValue = iformal.IKey(item).key()
                 optLabel = iformal.ILabel(item).label()
                 optValue = converter.fromType(optValue)
-                option = T.option(value=str(optValue))[
+                option = T.option(value=stringifyOptionValue(optValue))[
                     iformal.ILabel(optLabel).label()]
                 if optValue == value:
                     option = option(selected='selected')
@@ -525,7 +525,8 @@ class RadioChoice(ChoiceBase):
 
         def renderOption(request, itemKey, itemLabel, num, selected):
             cssid = render_cssid(key, num)
-            tag = T.input(name=key, type='radio', id=cssid, value=str(itemKey))
+            tag = T.input(name=key, type='radio', id=cssid,
+                value=stringifyOptionValue(itemKey))
             if selected:
                 tag = tag(checked='checked')
             if disabled:
@@ -908,6 +909,17 @@ class MMYYDatePartsInput(object):
         return self.original.validate(value)
 
 
+def stringifyOptionValue(s):
+    """returns s safe for inclusion into selection boxes.
+
+    That's a str with + replaced by %2b; I introduced this during a time
+    of confusion before I understood that plus-quoting was for urlencoded
+    values only.  But doing this helps in case there are broken http
+    components in the way, so I'll keep replacing where I easily can.
+    """
+    return str(s).replace("+", "%2b")
+
+
 @implementer( iformal.IWidget )
 class CheckboxMultiChoice(template.Element):
     """
@@ -943,9 +955,7 @@ class CheckboxMultiChoice(template.Element):
                optValue = converter.fromType(optValue)
                optid = render_cssid(key, n)
                checkbox = T.input(type='checkbox', name=key,
-                    # ouch: at least some browsers don't quote the + in
-                    # value attributes properly.  Hack around it:
-                    value=str(optValue).replace("+", "%2b"),
+                    value=stringifyOptionValue(optValue),
                     id=optid+["-box"], class_="multichoice",
                     onchange="Forms.Util.updateLabel(this)")
                if disabled:
@@ -1060,7 +1070,7 @@ class Hidden(object):
             value = ""
 
         return T.input(type=self.inputType, name=key, id=render_cssid(key),
-            value=str(value))
+            value=stringifyOptionValue(value))
 
     def renderImmutable(self, request, key, args, errors):
         return self.render(request, key, args, errors)
