@@ -12,6 +12,11 @@ format that essentially looks like this::
 	}
 
 No streaming at all is forseen for this format at this point.
+
+Note that we don't do any custom type serialisers here; all the
+stringification (or whatever) necessary happens within our own
+mappers (utils.serializers or wherever something is added to the
+defaultMFRegistry).
 """
 
 #c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
