@@ -16,6 +16,7 @@ You should access this module through formats.votable.
 
 
 import contextlib
+import datetime
 import functools
 import io
 import itertools
@@ -321,7 +322,7 @@ def _iterUniqueInfosFromMeta(
 
 	for mc in metaCarriers:
 		for metaValue in mc.iterMeta(metaKey, propagate=True):
-			value = metaValue.getContent("text")
+			value = metaValue.getContent("text", macroPackage=mc)
 			if value in seenItems:
 				continue
 			seenItems.add(value)
@@ -336,7 +337,17 @@ def _iterUniqueInfosFromMeta(
 
 
 def _iterResourceMeta(ctx, dataSet):
-	"""adds resource metadata to the Resource parent.
+	"""adds Data Origin metadata to the RESOURCE parent.
+
+	At this point, we don't know the creating service any more, and depending
+	on the sort of thing we are doing here, that's where we should take
+	the metadata from.  We therefore inspect the contributingMetaCarriers
+	attribute on the data item coming in and integrate the metadata
+	from everything that's in there (usually a service, but in the TAP
+	case perhaps also joined tables).
+
+	If this kind of things becomes more common, we should think of a
+	less hacky way to smuggle extra meta sources in here.
 	"""
 	yield V.DESCRIPTION[base.getMetaText(dataSet, "description",
 		macroPackage=dataSet.dd.rd, propagate=False)]
@@ -344,29 +355,69 @@ def _iterResourceMeta(ctx, dataSet):
 			_iterInfoInfos(dataSet), _iterWarningInfos(dataSet)):
 		yield el
 
-	sources = list(dataSet.tables.values())
-
-	yield from _iterUniqueInfosFromMeta(ctx, sources,
-		"source",
-		"citation",
-		lambda value: "meta.bib.bibcode"
-			if utils.couldBeABibcode(value)
-			else "meta.bib",
-		"This resource contains data associated with the publication {value}")
+	yield V.INFO(name="server_software", value=base.SERVER_SOFTWARE)[
+		"Software that produced this VOTable"]
+	yield V.INFO(name="server", value=base.getConfig("web", "serverURL"))[
+		"Base URI of the server"]
+	
+	sources = dataSet.contributingMetaCarriers
 
 	yield from _iterUniqueInfosFromMeta(ctx, sources,
 		"howtociteLink",
 		"citation",
-		"meta.bib",
-		"For advice on how to cite the resource(s) that contributed"
-		" to this result, see {value}")
+		"",
+		"Advice on citing this resource")
+	yield from _iterUniqueInfosFromMeta(ctx, sources,
+		"source",
+		"publication_id",
+		lambda value: "meta.bib.bibcode"
+			if utils.couldBeABibcode(value)
+			else "meta.bib",
+		"A bibliographic source citable for (parts of) this data")
 
+	yield from _iterUniqueInfosFromMeta(ctx, sources,
+		"published_identifier",
+		"ivoid",
+		"meta.ref.ivoid",
+		"Originating VO resource")
+	yield from _iterUniqueInfosFromMeta(ctx, sources,
+		"publisher",
+		"publisher",
+		None,
+		"Data centre that has delivered the data")
+	yield from _iterUniqueInfosFromMeta(ctx, sources,
+		"rights",
+		"rights",
+		None,
+		"Legal conditions applicable to (parts of) the data contained")
+	yield from _iterUniqueInfosFromMeta(ctx, sources,
+		"rights.rightsURI",
+		"rights_uri",
+		None,
+		"URI of legal conditions applicable to (parts of) the data contained")
+
+	yield V.INFO(name="request_date", ucd="time.creation",
+		value=utils.formatISODT(datetime.datetime.utcnow()))
+	yield from _iterUniqueInfosFromMeta(ctx, sources,
+		"contact.email",
+		"contact",
+		"meta.email",
+		"Contact option")
 	yield from _iterUniqueInfosFromMeta(ctx, sources,
 		"referenceURL",
-		"referenceURL",
+		"reference_url",
 		"meta.ref.url",
-		"More information on a resource that contributed to this data is"
-		" found at {value}")
+		"More information on the data Source")
+	yield from _iterUniqueInfosFromMeta(ctx, sources,
+		"version",
+		"resource_version",
+		"meta.version",
+		"Version of a contributing resource.")
+	yield from _iterUniqueInfosFromMeta(ctx, sources,
+		"creator.name",
+		"creator",
+		"meta.bib.author",
+		"Name of a person or entity that produced a contributing resource")
 
 
 def _iterToplevelMeta(ctx, dataSet):
@@ -732,12 +783,11 @@ def _guessTimeOrigin(frame, timeCol):
 		# that's the only sane option
 		return frame.get("time0")
 
-	if timeCol:
-		timeCol = timeCol.weakref()
-	if not timeCol:
-		# A timesys without location is rather bogus anyway.
-		return None
+	if not isinstance(timeCol, dm.ParamLikeAnnotation):
+		# it's a literal and we have no further metadata.
+		return
 
+	timeCol = timeCol.weakref()
 	# let's guess time origin based on units and such.
 	if timeCol.unit=="d":
 		# assuming JD or MJD.  Since people didn't give a time0, try to guess.
