@@ -53,51 +53,15 @@ class CBooster(object):
 				"platform"))
 		self._ensureBinary()
 
-	def _copySources(self, wd):
+	def _copySources(self, srcDir):
 		def getResource(src, dest):
 			inF = pkg_resources.resource_stream('gavo', src)
-			outF = open(os.path.join(wd, dest), "wb")
-			outF.write(inF.read())
-			outF.close()
+			with open(dest, "wb") as f:
+				f.write(inF.read())
 			inF.close()
 		getResource("resources/src/boosterskel.c", "boosterskel.c")
 		getResource("resources/src/boosterskel.h", "boosterskel.h")
-		shutil.copyfile(self.srcName, os.path.join(wd, "func.c"))
-
-		dd = self.dataDesc
-		if len(dd.makes)!=1:
-			raise base.ReportableError(
-				"Direct grammars can only work on single-make DDs")
-		query_n_pars = len(dd.makes[0].table.columns)
-
-		f = open(os.path.join(wd, "Makefile"), "w")
-
-		if dd.grammar.type=="fits":
-			f.write("LDFLAGS += -lcfitsio\n")
-
-		if dd.grammar.type.startswith("hdf5"):
-			flags = subprocess.check_output(
-				["pkg-config", "--cflags", "hdf5"]).strip().decode("ascii")
-			if flags:
-				f.write(f"CFLAGS += {flags}\n")
-			flags = subprocess.check_output(
-				["pkg-config", "--libs", "hdf5"]).strip().decode("ascii")
-			if flags:
-				f.write(f"LDFLAGS += {flags}\n")
-
-		f.write("LDFLAGS += -lm\n"
-			"CFLAGS += -Wall -DINPUT_LINE_MAX=%d -DQUERY_N_PARS=%s\n"%(
-				self.recordSize, query_n_pars))
-		if self.autoNull:
-			f.write("CFLAGS += -DAUTO_NULL='%s'\n"%self.autoNull.replace(
-				"\\", "\\\\"))
-		if self.ignoreBadRecords:
-			f.write("CFLAGS += -DIGNORE_BAD_RECORDS\n")
-		f.write("CFLAGS += -g\n")
-
-		f.write("booster: boosterskel.c func.c\n"
-			"\t$(CC) $(CFLAGS) %s -o booster $^ $(LDFLAGS)\n"%self.customFlags)
-		f.close()
+		shutil.copyfile(os.path.join(srcDir, self.srcName), "func.c")
 	
 	def _build(self):
 		callArgs = {}
@@ -106,10 +70,37 @@ class CBooster(object):
 			callArgs["stdout"] = utils.devnull()
 		if subprocess.call("make", **callArgs):
 			raise base.ReportableError("Booster build failed, messages above.")
-	
-	def _retrieveBinary(self, od):
-		shutil.copyfile("booster", self.binaryName)
-		os.chmod(self.binaryName, 0o775)
+
+	def _getMakefile(self, dd):
+		res = []
+		if dd.grammar.type=="fits":
+			res.append("LDFLAGS += -lcfitsio")
+
+		if dd.grammar.type.startswith("hdf5"):
+			flags = subprocess.check_output(
+				["pkg-config", "--cflags", "hdf5"]).strip().decode("ascii")
+			if flags:
+				res.append(f"CFLAGS += {flags}")
+			flags = subprocess.check_output(
+				["pkg-config", "--libs", "hdf5"]).strip().decode("ascii")
+			if flags:
+				res.append(f"LDFLAGS += {flags}")
+
+		query_n_pars = len(dd.makes[0].table.columns)
+		res.append("LDFLAGS += -lm\n"
+			"CFLAGS += -Wall -DINPUT_LINE_MAX=%d -DQUERY_N_PARS=%s"%(
+				self.recordSize, query_n_pars))
+		if self.autoNull:
+			res.append("CFLAGS += -DAUTO_NULL='%s'"%self.autoNull.replace(
+				"\\", "\\\\"))
+		if self.ignoreBadRecords:
+			res.append("CFLAGS += -DIGNORE_BAD_RECORDS")
+		res.append("CFLAGS += -g\n")
+
+		res.append("booster: boosterskel.c func.c\n"
+			"\t$(CC) $(CFLAGS) %s -o booster $^ $(LDFLAGS)"%self.customFlags)
+
+		return "\n".join(res)
 
 	def _ensureBinary(self):
 		"""makes sure the booster binary exists and is up-to-date.
@@ -121,13 +112,26 @@ class CBooster(object):
 				return
 		except os.error:
 			pass
-		if os.path.exists(self.srcName):
-			utils.runInSandbox(self._copySources, self._build, self._retrieveBinary)
-		else:
+
+		if not os.path.exists(self.srcName):
 			raise base.ReportableError("Booster source does not exist."
 				"  You will not be able to import the enclosing data.",
 				hint="Use gavo mkboost to create a skeleton for the booster.")
 
+		with utils.sandbox() as origpath:
+			dd = self.dataDesc
+			if len(dd.makes)!=1:
+				raise base.ReportableError(
+					"Direct grammars can only work on single-make DDs")
+
+			self._copySources(origpath)
+			with open("Makefile", "w") as f:
+				f.write(self._getMakefile(dd))
+				
+			self._build()
+			shutil.copyfile("booster", self.binaryName)
+			os.chmod(self.binaryName, 0o775)
+
 	def getOutput(self, argName):
 		"""returns a pipe you can read the booster's output from.
 
