@@ -9,7 +9,7 @@ TODO: Most of the hdf interface functions return 0 on success, != on
 failure; we'd like a macro to catch these.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -24,15 +24,16 @@ from gavo import utils
 from gavo.grammars import directgrammar
 
 
-class HDF5vaexCodeGenerator(directgrammar._NumpyMetaCodeGenerator):
-	"""A code generator for boosters importing HDF5 files in VAEX convention.
-
-	These have one array per column in a "columns" group; the actual data
-	is in a "data" group.
+class BaseHDF5CodeGenerator(directgrammar._NumpyMetaCodeGenerator):
+	"""An abstract base for generating boosters for various sorts of
+	HDF5 files.
 
 	Our strategy when parsing from them is to read CHUNK_SIZE items
 	at a time into the corresponding arrays, and then iterating
 	over these chunks, building the records.
+
+	The difference between the various flavours is where the column
+	metadata is taken from and where the data is then pulled from.
 	"""
 	_h5typemap = {
 		"byte": "H5T_NATIVE_CHAR",
@@ -43,14 +44,15 @@ class HDF5vaexCodeGenerator(directgrammar._NumpyMetaCodeGenerator):
 		"int16": "H5T_NATIVE_SHORT",
     "ushort": "H5T_NATIVE_USHORT",
     "uint16": "H5T_NATIVE_USHORT",
-		"cint": "H5T_NATIVE_INT", 
+		"cint": "H5T_NATIVE_INT",
 		"int32": "H5T_NATIVE_INT",
-		"uint": "H5T_NATIVE_UINT", 
-		"uintc": "H5T_NATIVE_UINT", 
-		"int64": "H5T_NATIVE_LLONG", 
-		"uint64": "H5T_NATIVE_ULLONG", 
+		"uint": "H5T_NATIVE_UINT",
+		"uintc": "H5T_NATIVE_UINT",
+		"int64": "H5T_NATIVE_LLONG",
+		"uint64": "H5T_NATIVE_ULLONG",
 		"ulonglong": "H5T_NATIVE_ULLONG",
 		"single": "H5T_NATIVE_FLOAT",
+		"float16": "H5T_NATIVE_FLOAT",
 		"float32": "H5T_NATIVE_FLOAT",
 		"double": "H5T_NATIVE_DOUBLE",
 		"float64": "H5T_NATIVE_DOUBLE",
@@ -63,23 +65,16 @@ class HDF5vaexCodeGenerator(directgrammar._NumpyMetaCodeGenerator):
 			raise base.StructureError("Cannot make HDF5 vaex booster without"
 				" a sources element on the embedding data.")
 		try:
-			sampleSource = next(self.grammar.parent.sources.iterSources())
-			hdf = h5py.File(sampleSource)
+			self.sampleSource = next(self.grammar.parent.sources.iterSources())
+			hdf = h5py.File(self.sampleSource)
 		except StopIteration:
 			raise base.StructureError("Building an HDF5 booster requires"
 				" at least one matching source.")
 
-		try:
-			self.dsName = self.grammar.getProperty("dataset", "table")
-			cols = hdf[self.dsName]["columns"]
-		except KeyError:
-			raise base.StructureError(f"Cannot access dataset {self.dsName} in"
-				f" {sampleSource}.  Override the grammar's dataset property"
-				" to point it to the right dataset.")
+		cols = self.getColumns(hdf)
 
 		self.inputColumns = []
-		for name in cols:
-			numpyType = cols[name]["data"].dtype
+		for name, numpyType in cols:
 			if numpyType.ndim!=0:
 				raise NotImplementedError("Cannot handle arrays in vaex"
 					" HDF5 yet.  Please contact the authors.")
@@ -98,7 +93,6 @@ class HDF5vaexCodeGenerator(directgrammar._NumpyMetaCodeGenerator):
 		self.byName = utils.CaseSemisensitiveDict(
 			(nameMap.get(t[0], t[0]), t)
 			for t in self.inputColumns)
-
 		
 	def _getStructDefinition(self):
 		"""returns a definition for a C structure for holding one record.
@@ -138,18 +132,6 @@ class HDF5vaexCodeGenerator(directgrammar._NumpyMetaCodeGenerator):
 		
 		return "\n".join(res)
 
-	def _getDatasetsCode(self):
-		"""returns code to create the datasets for the columns; the
-		result is kept in a datasets array.
-		"""
-		res = []
-		for index, (name, _, cType) in enumerate(self.inputColumns):
-			col_ds_name = "/".join([self.dsName, "columns", name, "data"])
-			res.extend([
-				f"  datasets[{index}] = H5Dopen2("
-					f'input_id, "{col_ds_name}", H5P_DEFAULT);',
-				f"  if (datasets[{index}]<0) abort();"])
-		return "\n".join(res)
 	
 	def _getTupleBuilder(self):
 		"""returns code to build the InRec struct passed to getTuple.
@@ -170,7 +152,7 @@ class HDF5vaexCodeGenerator(directgrammar._NumpyMetaCodeGenerator):
 			f"#define N_COLS {n_cols}",
 			# how many rows to process in one go?
 			"#define CHUNK_SIZE {}".format(
-				self.grammar.getProperty("chunkSize", 5000)),  
+				self.grammar.getProperty("chunkSize", 5000)),
 			self._getStructDefinition(),
 			"#define HDFGUARD(x) if ((x)<0) abort();",]
 
@@ -238,4 +220,73 @@ class HDF5vaexCodeGenerator(directgrammar._NumpyMetaCodeGenerator):
 				f"{typeMacro}({nameForItem}, (({castTo})(data->{origName})));")
 
 		return res
-			
+
+	def getColumns(self):
+		"""has to return a sequence of (name, type) pairs for the columns
+		of the relation encoded in the HDF5 file.
+		"""
+		raise NotImplementedError("HDF5 getColumns")
+
+	def _getDatasetsCode(self):
+		"""has to return C code to fill datasets[0...input_columns] with
+		the values the C code should to process.
+		"""
+		raise NotImplementedError("HDF5 _getDatasetsCode")
+
+
+class HDF5vaexCodeGenerator(BaseHDF5CodeGenerator):
+	"""A code generator for boosters importing HDF5 files in VAEX convention.
+
+	These have one array per column in a "columns" group; the actual data
+	is in a "data" group.
+
+	Our strategy when parsing from them is to read CHUNK_SIZE items
+	at a time into the corresponding arrays, and then iterating
+	over these chunks, building the records.
+	"""
+
+	def getColumns(self, hdf):
+		# SIDE EFFECT: self.dsName is the dataset name to read from after
+		# this function.
+		try:
+			self.dsName = self.grammar.getProperty("dataset", "table")
+			cols = hdf[self.dsName]["columns"]
+		except KeyError:
+			raise base.StructureError(f"Cannot access dataset {self.dsName} in"
+				f" {self.sampleSource}.  Override the grammar's dataset property"
+				" to point it to the right dataset.")
+		return [(name, col["data"].dtype) for name, col in cols.items()]
+
+	def _getDatasetsCode(self):
+		"""returns code to create the datasets for the columns; the
+		result is kept in a datasets array.
+		"""
+		res = []
+		for index, (name, _, cType) in enumerate(self.inputColumns):
+			col_ds_name = "/".join([self.dsName, "columns", name, "data"])
+			res.extend([
+				f"  datasets[{index}] = H5Dopen2("
+					f'input_id, "{col_ds_name}", H5P_DEFAULT);',
+				f"  if (datasets[{index}]<0) abort();"])
+		return "\n".join(res)
+
+
+class HDF5rootarrCodeGenerator(BaseHDF5CodeGenerator):
+	"""A code generator for boosters importing HDF5 files having
+	their columns in arrays in the root dataset.
+	"""
+
+	def getColumns(self, hdf):
+		return [(name, col.dtype) for name, col in hdf.items()]
+
+	def _getDatasetsCode(self):
+		"""returns code to create the datasets for the columns; the
+		result is kept in a datasets array.
+		"""
+		res = []
+		for index, (name, _, cType) in enumerate(self.inputColumns):
+			res.extend([
+				f"  datasets[{index}] = H5Dopen2("
+					f'input_id, "{name}", H5P_DEFAULT);',
+				f"  if (datasets[{index}]<0) abort();"])
+		return "\n".join(res)
