@@ -11,26 +11,31 @@ A grammar taking rows from a FITS table.
 from gavo import base
 from gavo.grammars import common
 
-from gavo.utils import pyfits
+from astropy import table
 
 
 class FITSTableIterator(common.RowIterator):
 	"""The row iterator for FITSTableGrammars.
 	"""
 	def _iterRows(self):
-		hdus = pyfits.open(self.sourceToken)
-		fitsTable = hdus[self.grammar.hdu].data
-		names = [n for n in fitsTable.dtype.names]
+		data = table.Table.read(self.sourceToken)
+		names = [n for n in data.dtype.names]
 
 		if self.grammar.lowerKeys:
 			names = [n.lower() for n in names]
 
-		for row in fitsTable:
-			if self.grammar.nanIsNULL:
-				row = [v if v==v else None for v in row]
+		for raw_row, mask in zip(data, data.mask):
+			row = {}
+			for i in range(len(raw_row)):
+				val = raw_row[i]
+				if mask[i]:
+					row[names[i]] = None
+				elif val!=val and self.grammar.nanIsNULL:
+					row[names[i]] = None
+				else:
+					row[names[i]] = val
 
-			res = dict(zip(names, row))
-			yield res
+			yield row
 
 
 class FITSTableGrammar(common.Grammar):
