@@ -15,7 +15,7 @@ can be used by processors.  If these use custom templates, they should
 register them (or regret it later).  See registerTemplate's docstring.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -27,7 +27,7 @@ import re
 from gavo import base
 from gavo.utils import pyfits
 
-DEFAULT_IGNORED_HEADERS = ["simple", "bitpix", "naxis", "imageh", 
+DEFAULT_IGNORED_HEADERS = ["simple", "bitpix", "naxis", "imageh",
 	"imagew", "naxis1", "naxis2", "datamin", "datamax", "date"]
 
 
@@ -62,7 +62,7 @@ def addHistoryCard(header, entry, recognizer):
 		header.add_history(entry)
 
 
-def copyFields(header, cardList, 
+def copyFields(header, cardList,
 		ignoredHeaders=frozenset(DEFAULT_IGNORED_HEADERS)):
 	"""copies over all cards from cardList into header, excluding headers
 	named in ignoredHeaders.
@@ -96,7 +96,7 @@ def getHeaderAsDict(hdr):
 def _makeHeaderSequence(keyTpl, commentTpl):
 	try:
 		return [
-			(keyTpl%ind, commentTpl%numeral) 
+			(keyTpl%ind, commentTpl%numeral)
 			for ind, numeral in [
 				(1, "1st"),
 				(2, "2nd"),
@@ -120,7 +120,7 @@ MINIMAL_IMAGE_TEMPLATE = [
 
 WCS_TEMPLATE = [
 		pyfits.Card(value="-------------------- Spatial WCS"),
-	('EQUNIOX', "Equinox of RA and Dec"),
+	('EQUINOX', "Equinox of RA and Dec"),
 	('RADESYS', "Reference System used for RA and Dec"),
 	('WCSAXES', "Number of FITS axes covered by WCS"),
 	('CTYPE1', "Projection on axis 1"),
@@ -178,6 +178,7 @@ WCS_TEMPLATE = [
 	('BP_1_1', "Axis 2 inverse polynomial, coefficient"),
 	('BP_2_0', "Axis 2 inverse polynomial, coefficient"),]
 
+
 # Internal representation of Tuvikene et al FITS headers for plates
 # See https://www.plate-archive.org/applause/wiki/fits-header-format-dr2/
 WFPDB_TEMPLATE = MINIMAL_IMAGE_TEMPLATE+[
@@ -211,7 +212,8 @@ WFPDB_TEMPLATE = MINIMAL_IMAGE_TEMPLATE+[
 		("SITELONG", " [deg] East longitude of observatory"),
 		("SITELAT", " [deg] Latitude of observatory"),
 		("SITEELEV", " [m] Elevation of the observatory"),
-		("OTA-NAME", "Telescope name"),
+		("TELESCOP", "Telescope name"),
+		("OTA-NAME", "Designation of the optical tube assembly"),
 		("OTA-DIAM", " [m] Clear aperture of the telescope"),
 		("OTA-APER", " [m] Clear aperture of the telescope"),
 		("FOCLEN", " [m] Focal length of the telescope"),
@@ -312,7 +314,7 @@ WFPDB_TEMPLATE = MINIMAL_IMAGE_TEMPLATE+[
 		("SVC-URI", "Where to get these files"),
 		
 		] + WCS_TEMPLATE + [
-		pyfits.Card(value="-------------------- History/Legal"),
+		pyfits.Card(value="-------------------- Legal"),
 		("LICENCE", "URI for conditions of use"),
 
 		pyfits.Card(value="-------------------- Other header cards"),]
@@ -320,6 +322,7 @@ WFPDB_TEMPLATE = MINIMAL_IMAGE_TEMPLATE+[
 
 _TEMPLATE_NAMES = [
 	("minimal", MINIMAL_IMAGE_TEMPLATE),
+	("wcs", WCS_TEMPLATE),
 	("wfpdb", WFPDB_TEMPLATE),]
 
 
@@ -355,7 +358,7 @@ def getNameForTemplate(template):
 	for name, namedTemplate in _TEMPLATE_NAMES:
 		if template is namedTemplate:
 			return name
-	raise base.NotFoundError("template "+str(id(template)), 
+	raise base.NotFoundError("template "+str(id(template)),
 		"FITS template",
 		"registered templates", hint="If you used a custom template,"
 		" have you called fitstricks.registerTemplate(name, template)"
@@ -391,10 +394,15 @@ def _applyTemplate(hdr, template, values):
 			hdr.append(tp, end=True)
 		else:
 			key, comment = tp
-			argKey = key.replace("-", "_")
-			if values.get(argKey) is not None:
+			
+			argKey = key
+			val = values.get(argKey)
+			if val is None:
+				argKey = argKey.replace("-", "_")
+				val = values.get(argKey)
+
+			if val is not None:
 				try:
-					val = values[argKey]
 					hdr.append(pyfits.Card(key, val, comment), end=True)
 				except Exception as ex:
 					if hasattr(ex, "args") and isinstance(ex.args[0], str):
@@ -437,12 +445,12 @@ def makeHeaderFromTemplate(template, originalHeader=None, **values):
 
 	template usually is the name of a template previously registered with
 	registerTemplate, or one of DaCHS predefined template names (currently,
-	minimal and wfpdb).  In a pinch, you can also pass in an immediate 
+	minimal and wfpdb).  In a pinch, you can also pass in an immediate
 	headers.
 
 	originalHeader can be a pre-existing header; the history and comment
 	cards are copied over from it, and if any of its other cards have not
-	yet been added to the header, they will be added in the order that they 
+	yet been added to the header, they will be added in the order that they
 	apprear there.
 
 	values for which no template item is given are added in random order
@@ -459,7 +467,7 @@ def makeHeaderFromTemplate(template, originalHeader=None, **values):
 		try:
 			templateName = getNameForTemplate(template)
 		except base.NotFoundError:
-			base.notifyWarning("Using anonymous FITS template.")
+			base.ui.notifyWarning("Using anonymous FITS template.")
 			templateName = "anonymous"
 
 	_applyTemplate(hdr, template, values)
@@ -482,7 +490,7 @@ def updateTemplatedHeader(hdr, templateName=None, **kwargs):
 	"""return hdr updated with kwargs.
 
 	hdr is assumed to have been created with makeHeaderFromTemplate
-	and contain the template name in a history entry.  
+	and contain the template name in a history entry.
 
 	You can pass in templateName to keep DaCHS from trying to get things
 	from the header.
