@@ -16,7 +16,6 @@ import http.server
 import contextlib
 import gc
 import io
-import importlib
 import os
 import pickle
 import re
@@ -56,8 +55,8 @@ except ImportError:
 #
 # All the custom setup can be suppressed by setting a GAVO_OOTTEST
 # env var before importing this.  That's for "out of tree test"
-# and is used by the relational registry "unit" tests (and possibly
-# others later).
+# and is used by several unittest-like things in various resources
+# (e.g., rr).
 
 def ensureResources():
 	# overridden below if necessary
@@ -76,9 +75,19 @@ else:
 		warnings.warn("testhelpers imported from non-test directory.  This"
 			" is almost certainly not what you want (or set GAVO_OOTTEST).")
 
+	# keep import base from touching the DB in case the test environment
+	# is not present yet; we're calling initPsycopg manually later.
+	os.environ["GAVO_INIT_RUNNING"] = "1"
 	from gavo import base #noflake: import above is conditional
+	del os.environ["GAVO_INIT_RUNNING"]
 
-	if not os.path.exists(base.getConfig("rootDir")):
+	if os.path.exists(base.getConfig("rootDir")):
+		# we're assuming the test environment is complete then
+		# (i.e., including the test database)
+		from gavo.base import sqlsupport
+		sqlsupport.initPsycopg()
+
+	else:
 		from gavo.user import initdachs
 		dbname = "dachstest"
 		dsn = initdachs.DSN(dbname)
@@ -95,8 +104,8 @@ else:
 			f.write('http://sofo-hd.de sofo !redirect\n'
 				'data/cores/scs/scs.xml fromvanitynames\n')
 
-		from gavo.base import config
-		config.makeFallbackMeta(reload=True)
+		from gavo.base import config, meta
+		meta.makeFallbackMeta(reload=True)
 		os.symlink(os.path.join(TEST_BASE, "test_data"),
 			os.path.join(base.getConfig("inputsDir"), "data"))
 		os.rmdir(os.path.join(base.getConfig("inputsDir"), "__system"))
@@ -104,44 +113,37 @@ else:
 			os.path.join(base.getConfig("inputsDir"), "__system"))
 		os.mkdir(os.path.join(base.getConfig("inputsDir"), "test"))
 
+		subprocess.check_call(["createdb", "--template=template0",
+			"--encoding=UTF-8", "--locale=C", dbname])
+		initdachs.initDB(dsn)
+		from gavo.base import sqlsupport
+		sqlsupport.initPsycopg()
+
 		def ensureResources(): #noflake: deliberate override
 			# delay everything that needs DB connectivity until
 			# after the import; otherwise, we may create a conn pool
 			# and that, as used in DaCHS, may create a thread.  That means
 			# risking a deadlock while python is importing.
 			try:
-				subprocess.check_call(["createdb", "--template=template0",
-					"--encoding=UTF-8", "--locale=C", dbname])
-
-				initdachs.initDB(dsn)
-				from gavo.base import sqlsupport
-				# update the type bindings
-				importlib.reload(sqlsupport)
-				# update the UDFs
-				importlib.reload(ufunctions)
-
 				from gavo.registry import publication
 				from gavo import rsc
 				from gavo import rscdesc #noflake: caches registration
 				from gavo.protocols import creds
-				from gavo.protocols import tap
 
 				publication.updateServiceList([base.caches.getRD("//rds")])
 				publication.updateServiceList([base.caches.getRD("//services")])
 				publication.updateServiceList([base.caches.getRD("//tap")])
 
 				# Import some resources necessary in trial tests
+				from gavo.user import importing
+				importing.process(rsc.getParseOptions(), ["//obscore"])
+				importing.process(rsc.getParseOptions(), ["//biblinks"])
 				with base.getWritableAdminConn() as conn:
-					rsc.makeData(base.resolveCrossId("//obscore#makeSources"),
-						connection=conn)
-					rsc.makeData(base.resolveCrossId("//obscore#create"),
-						connection=conn)
-					tap.publishToTAP(base.resolveCrossId("//obscore"),
-						conn)
 					rsc.makeData(base.resolveCrossId("//uws#enable_useruws"),
 						connection=conn)
 					creds.addUser(conn, "testing", "testing", "powerless test user")
 					creds.addUser(conn, "other", "other", "another test user")
+
 			except:
 				traceback.print_exc()
 				sys.stderr.write("Creation of test environment failed.  Remove %s\n"
@@ -155,7 +157,7 @@ from gavo.base import osinter
 osinter.sendMail = lambda *args, **kwargs: None
 
 from gavo import utils
-from gavo.base import config
+from gavo.base import config  #noflake: previous import conditional
 from gavo.base import sqlsupport
 from gavo.adql import ufunctions
 
@@ -290,6 +292,10 @@ from gavo.helpers.testtricks import (  #noflake: exported names
 class ForkingSubprocess(subprocess.Popen):
 	"""A subprocess that doesn't exec but fork.
 	"""
+	def __init__(self, *args, **kwargs):
+		self.input = kwargs.pop("input", None)
+		super().__init__(*args, **kwargs)
+
 	def _execute_child(self, args, executable, preexec_fn, close_fds,
 							 pass_fds, cwd, env,
 							 startupinfo, creationflags, shell,
@@ -311,13 +317,26 @@ class ForkingSubprocess(subprocess.Popen):
 # ignoring everything that 2.7 hasn't known under the bold assumption that our
 # simple testing forks that doesn't matter (which ought to be true for
 # start_new_session at least:-).
+#
+# Since we don't exec, I'm catching and communicating subprocess exceptions
+# while the function is running.  This means that the parent will
+# hang in a read while the subprocess is running. In turn, this means
+# that you can't subprocess.communicate because the parent isn't running.
+# You can pass in small amounts of input, though (but things will hang
+# if it'd fill up a buffer and the child crashes before it can read.
 
 		sys.argv = args
 		if executable is None:
 				executable = args[0]
 
 		def _close_in_parent(fd):
+			try:
 				os.close(fd)
+			except os.error:
+				# fd can be a bad file descriptor under circumstances I have
+				# not investigated.  If it is, don't bail out; it's better
+				# if we at least close whatever is left.
+				pass
 
 		# For transferring possible exec failure from child to parent.
 		# Data format: "exception name:hex errno:description"
@@ -391,6 +410,10 @@ class ForkingSubprocess(subprocess.Popen):
 												preexec_fn()
 
 										exitcode = 0
+										try:
+											executable()
+										except SystemExit as ex:
+											exitcode = ex.code
 
 								except:
 										exc_type, exc_value, tb = sys.exc_info()
@@ -404,11 +427,6 @@ class ForkingSubprocess(subprocess.Popen):
 										os._exit(255)
 
 								os.close(errpipe_write)
-								try:
-									executable()
-								except SystemExit as ex:
-									exitcode = ex.code
-
 								if "COVERAGE_FILE" in os.environ:
 									cov.stop()
 									cov.save()
@@ -422,6 +440,18 @@ class ForkingSubprocess(subprocess.Popen):
 						if gc_was_enabled:
 								gc.enable()
 
+						if self.input:
+							# this is lame as it will hang if the child crashes
+							# early and we fill the buffer.  But I'd like to avoid
+							# re-implementing subprocess.communicate here...
+							os.write(p2cwrite, self.input)
+							# I need to close p2cwrite so children waiting for eof
+							# run...
+							_close_in_parent(p2cwrite)
+							p2cwrite = -1
+							# keep subprocess.communicate from trying to close again.
+							self.stdin = None
+
 						# Now wait for the child to come up (at which point it will
 						# close its error pipe)
 						errpipe_data = bytearray()
@@ -444,7 +474,7 @@ class ForkingSubprocess(subprocess.Popen):
 						_close_in_parent(errwrite)
 
 				# be sure the error pipe FD is eventually no matter what
-				os.close(errpipe_read)
+				_close_in_parent(errpipe_read)
 
 
 class VerboseTest(testresources.ResourcedTestCase):
@@ -537,10 +567,12 @@ class VerboseTest(testresources.ResourcedTestCase):
 		if isinstance(toExec, str):
 			p = subprocess.Popen([toExec]+argList, executable=toExec,
 				stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
+			out, err = p.communicate(input=utils.bytify(input))
 		else:
 			p = ForkingSubprocess(["test harness"]+argList, executable=toExec,
-				stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
-		out, err = p.communicate(input=utils.bytify(input))
+				stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE,
+				input=utils.bytify(input))
+			out, err = p.communicate()
 		retcode = p.wait()
 
 		try:
@@ -678,7 +710,7 @@ def computeWCSKeys(pos, size, cutCrap=False):
 			"bandpassId": None,
 			"bandpassUnit": None,
 			"bandpassRefval": None,
-			"bandpassLo": pos[0],
+			"bandpassLo": pos[0]+0.0001,
 			"bandpassHi": pos[0]+size[0],
 			"pixflags": None,
 			"accref": "image/%s/%s"%(pos, size),
@@ -927,30 +959,51 @@ def main(testClass, methodPrefix=None):
 		base.DEBUG = True
 		from gavo.user import logui
 		logui.LoggingUI(base.ui)
-	
+
 	if "GAVO_OOTTEST" not in os.environ:
 		# run any pending upgrades (that's a test for them, too... of sorts)
 		from gavo.user import upgrade
 		upgrade.upgrade()
 
+	className, methodPrefix = None, None
 	try:
 		# two args: first one is class name, locate it in caller's globals
 		# and ignore anything before any dot for cut'n'paste convenience
-		if len(sys.argv)>2:
-			className = sys.argv[-2].split(".")[-1]
-			testClass = getattr(sys.modules["__main__"], className)
+		if len(sys.argv)==3:
+			className = sys.argv[1].split(".")[-1]
+			methodPrefix = sys.argv[2]
 		
-		# one arg: test method prefix on testClass
-		if len(sys.argv)>1:
-			suite = unittest.makeSuite(testClass, methodPrefix or sys.argv[-1],
-				suiteClass=testresources.OptimisingTestSuite)
-		else:  # Zero args, emulate unittest.run behaviour
+		# one arg: dissect dots and decide what to do then
+		elif len(sys.argv)==2:
+			parts = sys.argv[1].split(".")
+			if len(parts)==3:
+				className, methodPrefix = parts[1:]
+			elif len(parts)==2:
+				className, methodPrefix = parts
+			elif len(parts)==1:
+				className = parts[0]
+			else:
+				sys.exit(f"{className}?")
+
+		elif len(sys.argv)==1:
+			pass
+
+		else:
+			sys.exit("usage: test-id or class-name test-prefix")
+
+		if className is None:
 			suite = testresources.TestLoader().loadTestsFromModule(
 				sys.modules["__main__"])
+		else:
+			testClass = getattr(sys.modules["__main__"], className)
+			suite = unittest.makeSuite(testClass, methodPrefix or "test",
+				suiteClass=testresources.OptimisingTestSuite)
 
 		runner = unittest.TextTestRunner(
 			verbosity=int(os.environ.get("TEST_VERBOSITY", 1)))
+
 		result = runner.run(suite)
+			
 		if result.errors or result.failures:
 			sys.exit(1)
 		else:
