@@ -7,10 +7,8 @@ Code to bind the adql library to the data center software.
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
-
 import sys
 
-
 from gavo import adql
 from gavo import base
 from gavo import rsc
@@ -274,45 +272,30 @@ def _addTableMeta(translated, query, tree, table):
 	"""adds various info items from query and its parsed tree to a
 	result table.
 	"""
+	# preserve existing INFOs copied from the source table
 	table.makeOriginal("info")
-	table.addMeta("info", "", infoName="server",
-		infoValue=base.getConfig("web", "serverURL"))
-	table.addMeta("info", "", infoName="sql_query", infoValue=translated)
-	table.addMeta("info", "", infoName="query", infoValue=query)
+
+	table.addMeta("info", "ADQL query translated to local SQL (for debugging)",
+		infoName="sql_query", infoValue=translated)
+	table.addMeta("info", "Original ADQL query",
+		infoName="query", infoValue=query)
 
 	mth = base.caches.getMTH(None)
 	sourceTables = tree.getContributingNames()
+
 	# for 1-table queries, we've already copied the entire table metadata.
-	# don't re-copy it.
-	tableMetaCopied = len(sourceTables)==1
+	# don't re-copy it.  Otherwise tell votablewrite to obtain further
+	# metadata from our contributing tables
+	if len(sourceTables)!=1:
+		metaSources = []
 
-	tablesSeen = set()
-	for tableName in sourceTables:
-		if tableName in tablesSeen:
-			continue
-		tablesSeen.add(tableName)
+		# Secret handshake with taprunner._makeDataFor; see there.
+		table.tableDef.contributingTables = metaSources
 
-		try:
-			sourceTD = mth.getTableDefForTable(tableName)
-			table.addMeta("info",
-				base.getMetaText(sourceTD.rd, "description", ""),
-				infoName="src_res",
-				infoValue="Contains traces from resource %s"%(sourceTD.rd.sourceId))
-			for m in sourceTD.iterMeta("rights", propagate=True):
-				table.addMeta("info",
-					m.getContent("text"),
-					infoName="copyright",
-					infoValue="%s copyright or license"%(sourceTD.rd.sourceId))
-			table.addMeta("info",
-				base.getMetaText(sourceTD, "description", "", propagate=False),
-				infoName="src_table",
-				infoValue="Contains traces from table %s"%(
-					sourceTD.getQName()))
-
-			if not tableMetaCopied:
-				for copiedKey in ["howtociteLink", "source", "referenceURL"]:
-					for m in sourceTD.iterMeta(copiedKey, propagate=True):
-						table.addMeta(copiedKey, m)
+		for tableName in set(tableName for tableName in sourceTables):
+			try:
+				sourceTD = mth.getTableDefForTable(tableName)
+				metaSources.append(sourceTD)
 
 				for m in sourceTD.iterMeta("_associatedDatalinkService"):
 					idColumn = sourceTD.getColumnByName(
@@ -333,9 +316,15 @@ def _addTableMeta(translated, query, tree, table):
 					except base.NotFoundError:
 						# User hasn't selected the column with the id.  No problem.
 						pass
-		except base.Error:
-			# don't fail just because of funny metadata or tables not found
-			pass
+
+			except base.NotFoundError:
+				# that's probably upload tables; we don't parse their
+				# metadata so far anyway.  Perhaps we should do so one day.
+				pass
+
+			except base.Error as msg:
+				# don't fail just because of funny metadata or tables not found
+				base.ui.notifyWarning(f"While adding TAP metadata: {msg}")
 
 
 def _updateMatchLimits(tree, maxrec, hardLimit):
@@ -393,6 +382,8 @@ def morphADQL(query, metaProfile=None, tdsForUploads=[],
 	additional table.tableDef.overflowLimit attribute on the returned table,
 	see _updateMatchLimits above; this will always be an integer.
 	"""
+	base.ui.notifyInfo("Incoming ADQL query: %s"%query)
+
 	ctx, t = adql.parseAnnotating(query,
 		DaCHSFieldInfoGetter(metaProfile, tdsForUploads))
 
@@ -520,14 +511,13 @@ class ADQLCore(svcs.Core, base.RestrictionMixin):
 	def run(self, service, inputTable, queryMeta):
 		inRow = inputTable.getParamDict()
 		queryString = inRow["query"]
-		base.ui.notifyInfo("Incoming ADQL query: %s"%queryString)
 		try:
 			res = query(queryString,
 				timeout=queryMeta["timeout"], hardLimit=100000,
 				externalLimit=queryMeta["dbLimit"])
 # XXX Warning: We're returning the db connection to the connection
 # pool here while we still have a named cursor on it.  This is
-# risky because someone might fuzz with our connection later.
+# risky because someone might futz with our connection later.
 # However, postponing the return of the connection isn't nice
 # either because then the renderer would have to manage the core's
 # connections, which is ugly, too.
