@@ -154,7 +154,7 @@ def mangleUploads(request):
 	request.strargs["upload"] = newUploads
 			
 
-def getDALIError(errInfo, queryStatus="ERROR"):
+def getDALIError(errInfo, queryStatus="ERROR", request=None):
 	"""returns  a DALI-compliant error VOTable from an error info.
 
 	errInfo can either be an exception, a UWS error info (a
@@ -175,6 +175,9 @@ def getDALIError(errInfo, queryStatus="ERROR"):
 	res = V.RESOURCE(type="results") [
 		V.INFO(name="QUERY_STATUS", value=queryStatus)[
 			errInfo["msg"]]]
+	if request:
+		res[V.INFO(name="request", value=utils.debytify(request.uri))]
+
 	if errInfo["hint"]:
 		res[V.INFO(name="HINT", value="HINT")[
 			errInfo["hint"]]]
@@ -201,7 +204,7 @@ def serveDALIError(request, errInfo, httpStatus=200, queryStatus="ERROR"):
 
 	try:
 		request.setHeader("content-type", "text/xml")
-		errVOT = getDALIError(errInfo, queryStatus)
+		errVOT = getDALIError(errInfo, queryStatus, request)
 		request.setResponseCode(httpStatus)
 		request.write(
 			utils.xmlrender(errVOT,
