@@ -69,6 +69,14 @@ REMOTE_URL_PATTERN = re.compile("(https?|ftp)://")
 MS = base.makeStruct
 
 
+@functools.cache
+def productTableColNames():
+	"""returns the names of the columns of the product table in
+	DaCHS' capitalisation.
+	"""
+	return [c.name for c in base.resolveCrossId(PRODUCTS_TDID)]
+
+
 def makePreviewFromFITS(product):
 	"""returns image/jpeg bytes for a preview of a product spitting out a
 	2D FITS.
@@ -825,6 +833,7 @@ class RAccref(object):
 	if that exists.  If it doesn't, accessing the property will raise
 	an NotFoundError.
 	"""
+
 	_buildKeys = dict((
 		("dm", str),    # data model, VOTable generation
 		("ra", float),  # cutouts
@@ -899,8 +908,13 @@ class RAccref(object):
 		try:
 			return self._productsRowCache
 		except AttributeError:
-			res = base.resolveCrossId(PRODUCTS_TDID).doSimpleQuery(
-				fragments="accref=%(accref)s", params={"accref": self.accref})
+
+			with base.getTableConn() as conn:
+				res = [dict(zip(productTableColNames(), row))
+					for row in conn.query(
+						"SELECT * FROM dc.products WHERE accref=%(accref)s",
+						{"accref": self.accref})]
+
 			if not res:
 				raise base.NotFoundError(self.accref, "accref", "product table",
 					hint="Product URLs may disappear, though in general they should"
