@@ -2,7 +2,7 @@
 IVOA cone search: Helper functions, a core, and misc.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -32,8 +32,8 @@ def getRadialCondition(td, ra, dec, sr,
 	spoints with pos.eq;meta.main.  This, of course, will not work
 	with SCS itself (out of the box).
 	"""
-	if (isinstance(ra, str) and ";" in ra 
-		) or (isinstance(dec, str) and ";" in dec 
+	if (isinstance(ra, str) and ";" in ra
+		) or (isinstance(dec, str) and ";" in dec
 		) or (isinstance(sr, str) and ";" in sr):
 		raise base.ReportableError(
 			"getRadialCondition's last resort alarm triggered.")
@@ -85,9 +85,9 @@ def findNClosest(alpha, delta, tableDef, n, fields, searchRadius=5):
 	the query center in degrees.
 	"""
 	with base.getTableConn() as conn:
-		raField = tableDef.getColumnByUCDs("pos.eq.ra;meta.main", 
+		raField = tableDef.getColumnByUCDs("pos.eq.ra;meta.main",
 			"POS_EQ_RA_MAIN").name
-		decField = tableDef.getColumnByUCDs("pos.eq.dec;meta.main", 
+		decField = tableDef.getColumnByUCDs("pos.eq.dec;meta.main",
 			"POS_EQ_RA_MAIN").name
 		res = list(conn.query("SELECT %s,"
 				" (spoint(radians(%s), radians(%s)) <->"
@@ -139,7 +139,7 @@ class SCSCore(svcs.DBCore):
 	"""A core performing cone searches.
 
 	This will, if it finds input parameters it can make out a position from,
-	add a _r column giving the distance between the match center and 
+	add a _r column giving the distance between the match center and
 	the columns that a cone search will match against.
 
 	If any of the conditions for adding _r aren't met, this will silently
@@ -164,13 +164,15 @@ class SCSCore(svcs.DBCore):
 			self.idColumn = self.outputTable.getColumnByUCDs(
 				"meta.id;meta.main", "ID_MAIN")
 		except base.StructureError:
-			base.ui.notifyWarning("SCS core at %s: Output table has no"
+			base.ui.notifyWarning("SCS core at %s: Output table has no unique"
 				" meta.id;meta.main column.  This service will be invalid."%
 				self.getSourcePosition())
 
 		self.distCol = base.resolveCrossId("//scs#distCol")
-		self.outputTable = self.outputTable.change(
-			columns=[self.distCol]+self.outputTable.columns)
+		# let me indulge in this ugly hack -- outputTable belongs to
+		# us anyway, and saving a copy is totally not worth it
+		self.outputTable.columns[0:0] = [self.distCol]
+		self.outputTable.columns.redoIndex()
 
 		if not self.hasProperty("defaultSortKey"):
 			self.setProperty("defaultSortKey", self.distCol.name)
@@ -187,7 +189,7 @@ class SCSCore(svcs.DBCore):
 			try:
 				return parseHumanSpoint(pars["hscs_pos"], "hscs_pos")
 			except (ValueError, base.ValidationError):
-				# We do not want to fail for this fairly unimportant thing.  
+				# We do not want to fail for this fairly unimportant thing.
 				# If the core actually needs the position, it should fail itself.
 				return None
 		else:
@@ -228,8 +230,8 @@ class SCSCore(svcs.DBCore):
 			self.getQueryCols(service, queryMeta))
 
 		return base.makeStruct(outputdef.OutputTableDef,
-			parent_=self.queriedTable.parent, 
+			parent_=self.queriedTable.parent,
 			id="result",
-			onDisk=False, 
+			onDisk=False,
 			columns=outCols,
 			params=self.queriedTable.params)
