@@ -119,6 +119,16 @@ def makeSDMDataForSSARow(ssaRow, spectrumData,
 			if val:
 				param.set([val.x/utils.DEG, val.y/utils.DEG])
 
+	# we want to include service metadata for data origin, but getting to
+	# the service object from in here is really hard (since services may
+	# share cores).  So... let's steal.  Nothing much breaks if this doesn't
+	# work out.
+	try:
+		resData.contributingMetaCarriers.append(utils.stealVar("service"))
+	except ValueError:
+		# no service local in stack
+		pass
+	
 	return resData
 
 
@@ -363,7 +373,6 @@ def formatSDMData(sdmData, format, queryMeta=svcs.emptyQueryMeta):
 
 	(you'll usually use this via //soda#sdm_format)
 	"""
-
 	destMime =  str(format or base.votableType)
 	if queryMeta["tdEnc"] and destMime==base.votableType:
 		destMime = "application/x-votable+xml;serialization=tabledata"
