@@ -141,7 +141,7 @@ base.caches.makeCache("getSesame", lambda key="ignored": Sesame())
 
 from gavo import adql
 
-@adql.userFunction("gavo_simbadpoint",
+@adql.userFunction("ivo_simbadpoint",
 	"(identifier TEXT) -> POINT",
 	"""
 	gavo_simbadpoint queries simbad for an identifier and returns the
@@ -151,7 +151,8 @@ from gavo import adql
 	to fire off millions of simbad queries anyway; use simbad's own
 	TAP service for this kind of application.
 	""",
-	"point", ucd="pos.eq;src")
+	"point", ucd="pos.eq;src",
+	additionalNames=["gavo_simbadpoint"])
 def _simbadpoint(args):
 	from gavo.adql import nodes
 
