@@ -11,6 +11,7 @@ TAP: schema maintenance, job/parameter definition incl. upload and UWS actions.
 import datetime
 import functools
 import os
+import signal
 
 from twisted.internet import threads
 
@@ -179,7 +180,7 @@ class TAPTransitions(uws.ProcessBasedUWSTransitions):
 		uws.SimpleUWSTransitions.__init__(self, "TAP")
 
 	def getCommandLine(self, wjob):
-		return "gavo", ["gavo", "--ui", "stingy", "taprun", "--", str(wjob.jobId)]
+		return "dachs", ["dachs", "--ui", "stingy", "taprun", "--", str(wjob.jobId)]
 
 	def queueJob(self, newState, wjob, ignored):
 		"""puts a job on the queue.
@@ -318,6 +319,15 @@ class TAPJob(uws.UWSJobWithWD):
 				{'dt': self.destructionTime}, conn)[0]["count"]
 		return datetime.datetime.utcnow()+nBefore*EST_TIME_PER_JOB
 
+	def prepareForDestruction(self):
+		if self.phase==uws.EXECUTING:
+			# since we're about to kill all state anyway, we're just TERM-ing
+			# the child rather than ask it politely.
+			# Let's just hope self.phase isn't lying; if it is, we'll possibly
+			# kill something else...
+			os.kill(self.pid, signal.SIGTERM)
+		super().prepareForDestruction()
+
 
 #################### The TAP worker system
 
