@@ -116,6 +116,14 @@ def _makeDataFor(resultTable):
 	additional metadata.
 	"""
 	resData = rsc.wrapTable(resultTable)
+	
+	resData.contributingMetaCarriers.append(base.resolveCrossId("//tap#run"))
+	# secret handshake with adqlglue: Add all tables of which we know that
+	# contributed to the query result to the meta sources for Data Origin
+	if hasattr(resultTable.tableDef, "contributingTables"):
+		resData.contributingMetaCarriers.extend(
+			resultTable.tableDef.contributingTables)
+
 	resData.addMeta("info", "Query successful",
 		infoName="QUERY_STATUS", infoValue="OK")
 	resData.setMeta("_type", "results")
