@@ -693,6 +693,8 @@ class BaseUWSJob(object, metaclass=UWSJobType):
 	  id.  The default implementation does this, but if you want
 	  something in the file system, you probably don't want to
 	  use that.
+	- a method prepareForDestruction() to do extra cleanup before
+	  a job is destroyed.
 
 	The job parameters -- those eventually passed to the core
 	when the job runs -- are held in a dictionary parameters.  You
@@ -1436,7 +1438,7 @@ class ProcessBasedUWSTransitions(SimpleUWSTransitions):
 		cmd, args = self.getCommandLine(wjob)
 		pt = reactor.spawnProcess(_UWSBackendProtocol(wjob.jobId, wjob.uws),
 			cmd, args=args,
-				env=os.environ)
+			env=os.environ)
 		wjob.change(pid=pt.pid, phase=EXECUTING)
 
 	def _startJobNonTwisted(self, wjob):
@@ -1446,7 +1448,9 @@ class ProcessBasedUWSTransitions(SimpleUWSTransitions):
 		cmd, args = self.getCommandLine(wjob)
 		pid = os.fork()
 		if pid==0:
-			_replaceFDs("/dev/zero", "/dev/null")
+			# in case of taprunner trouble, comment out the following;
+			# you will then at least see the output of taprunner.
+			#_replaceFDs("/dev/zero","/dev/null")
 			os.execlp(cmd, *args)
 		elif pid>0:
 			wjob.change(pid=pid, phase=EXECUTING)
