@@ -126,7 +126,8 @@ _vrResourceBuilder = meta.ModelBasedBuilder([
 			(None, SF(lambda term=term:
 				VOR.relationship[VOR.relationshipType[term]]), [
 			(term[0].lower()+term[1:], SF(VOR.relatedResource), (), {
-					"ivoId": "ivoId"})])
+					"ivoId": "ivoId",
+					"altIdentifier": "altIdentifier"})])
 			for term in [
 				"Cites",
 				"IsSupplementTo",
