@@ -143,6 +143,9 @@ class VOSICapInterface(VOSIInterface):
 class VOSITMInterface(VOSIInterface):
 	renderer = "tableMetadata"
 
+	def _makeInterface(self, publication):
+		return super()._makeInterface(publication)(version="1.1")
+
 
 class DALIInterface(InterfaceMaker):
 	interfaceClass = DaFut.daliInterface
@@ -747,6 +750,13 @@ class EditionCapabilityMaker(CapabilityMaker):
 				base.getMetaText(publication, "locTitle", None)]]
 
 
+class BiblinksCapabilityMaker(CapabilityMaker):
+	renderer = "biblinks.json"
+
+	class capabilityClass(VOR.capability):
+		_a_standardID = "ivo://ivoa.net/std/BibVO#biblink-harvest-0.1"
+
+
 _getCapabilityMaker = utils.buildClassResolver(CapabilityMaker,
 	list(globals().values()), instances=True,
 	key=lambda obj: obj.renderer)
