@@ -10,7 +10,8 @@ Instantiated resources (tables, etc), plus data mangling.
 
 # Not checked by pyflakes: API file with gratuitous imports
 
-from gavo.rsc.common import DBTableError, FLUSH
+from gavo.utils import DBTableError
+from gavo.rsc.common import FLUSH
 from gavo.rsc.dbtable import DBTable
 from gavo.rsc.dumping import createDump, restoreDump
 from gavo.rsc.qtable import QueryTable
