@@ -13,18 +13,6 @@ import copy
 from gavo import base
 
 
-class DBTableError(base.Error):
-	"""is raised when a manipulation of an on-disk table fails.
-
-	It always has a qName attribute containing the qualified name of
-	the table causing the trouble.
-	"""
-	def __init__(self, msg, qName, hint=None):
-		base.Error.__init__(self, msg, hint=hint)
-		self.qName = qName
-		self.args = [msg, qName]
-
-
 class FLUSH(object):
 	"""A sentinel that grammars can yield to flush out records to the
 	Database.
