@@ -206,6 +206,10 @@ class Data(base.MetaMixin, common.ParamMixin):
 	It is what ``makeData`` returns.  In typical one-table situations,
 	you just want to call the ``getPrimaryTable()`` method to obtain the
 	table built.
+
+	These also have an attribute contributingMetaCarriers, a list of
+	base.MetaCarrier-s used by votablewrite to create Data Origin INFO-s.  By
+	default, that's the first table.  You can add to that attribute
 	"""
 	def __init__(self,
 			dd,
@@ -216,6 +220,7 @@ class Data(base.MetaMixin, common.ParamMixin):
 		self.dd, self.parseOptions = dd, parseOptions
 		self.makes = overrideMakes or self.dd.makes
 		self.tables = tables
+		self.contributingMetaCarriers = []
 		self.setMetaParent(self.dd)
 		self._initParams(self.dd)
 
@@ -278,8 +283,11 @@ class Data(base.MetaMixin, common.ParamMixin):
 				" has thought about it; it would probably be harmless).")
 
 		controlledTables = {tableDef.id: tables.TableForDef(tableDef)}
-		return cls(dd, controlledTables, parseOptions,
+		res = cls(dd, controlledTables, parseOptions,
 			overrideMakes=[dd.makes[0].change(table=tableDef)])
+		res.contributingMetaCarriers.append(tableDef)
+
+		return res
 
 	def dbCatalogChanged(self):
 		"""returns true if a database table has been newly created by
@@ -686,6 +694,7 @@ def wrapTable(table, rdSource=None, resTypeDefault="results"):
 	newDD.setMetaParent(table.tableDef.rd)
 
 	res = Data(newDD, tables={table.tableDef.id: table})
+	res.contributingMetaCarriers.append(table)
 	res.setMeta("_type", resTypeDefault)
 
 	for infoMeta in table.iterMeta("info"):
