@@ -14,7 +14,6 @@ from gavo import base
 from gavo import rscdef
 from gavo import utils
 from gavo.base import sqlsupport
-from gavo.rsc import common
 from gavo.rsc import table
 
 
@@ -169,7 +168,7 @@ class DBMethodsMixin(sqlsupport.QuerierMixin):
 					self.tableName, ", ".join(self.tableDef.primary)))
 			except sqlsupport.DBError as msg:
 				raise base.ui.logOldExc(
-					common.DBTableError("Primary key %s could not be added (%s)"%(
+					base.DBTableError("Primary key %s could not be added (%s)"%(
 						self.tableDef.primary, repr(str(msg))), self.tableName,
 						hint="The howDoI documentation text may contain help on"
 						" how to find the offending elements."))
@@ -668,13 +667,6 @@ class DBTable(DBMethodsMixin, table.BaseTable, MetaTableMixin):
 		self.create()
 		return self
 
-	def expand(self, s):
-		"""macro-expands the string s within self's tableDef.
-		"""
-		if "\\" in s:
-			s = self.tableDef.expand(s)
-		return s
-
 	def query(self, query, data={}):
 		"""runs query within this table's connection.
 
