@@ -7,8 +7,8 @@ Description of columns (and I/O fields).
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+import functools
 
-from gavo import adql
 from gavo import base
 from gavo import dm
 from gavo import utils
@@ -70,7 +70,18 @@ class ParamNameAttribute(base.UnicodeAttribute):
 			raise base.StructureError(
 				f"'{value}' is not a valid column identifier")
 		return value
-	
+
+
+@functools.lru_cache(None)
+def _getReservedWords():
+	"""returns column names that need quoting.
+
+	That's really adql.ALL_RESERVED_WORDS at this point, but I want
+	to defer importing adql to when I actually need it.
+	"""
+	from gavo import adql
+	return adql.ALL_RESERVED_WORDS
+
 
 class ColumnNameAttribute(ParamNameAttribute):
 	"""An attribute containing a name suitable for SQL table names.
@@ -92,7 +103,7 @@ class ColumnNameAttribute(ParamNameAttribute):
 		if value.startswith("quoted/"):
 			return utils.QuotedName(value[7:])
 		value = ParamNameAttribute.parse(self, value)
-		if value.upper() in adql.ALL_RESERVED_WORDS:
+		if value.upper() in _getReservedWords():
 			base.ui.notifyWarning("Column name '%s' coincides with a"
 				" ADQL/SQL reserved name.  This may lead to serious trouble"
 				" later.  Please consider changing the name, or prepend quoted/"
