@@ -172,10 +172,18 @@ class IVOMetaMixin(base.StructCallbacks):
 		# copied.  Code there knows what to do when we return None
 		if self.rd is None:
 			return None
-		
+			
 		return "ivo://%s/%s/%s"%(base.getConfig("ivoa", "authority"),
 				urllib.parse.quote(self.rd.sourceId), self.id)
 
+	def _meta_published_identifier(self):
+		"""returns identifier if we believe we are dealing with a published
+		resource.
+		"""
+		if (getattr(self, "publications", None)
+				or getattr(self, "registration", None)):
+			return self._meta_identifier()
+
 	def _meta_status(self):
 		return "active"
 
