@@ -327,6 +327,11 @@ class DataURL(base.Structure):
 		" with a POST request (conflicts with additional parameters).",
 		copyable=True)
 
+	_postMediaType = base.UnicodeAttribute("httpPostMediaType",
+		default="application/octet-stream",
+		description="The media type of postPayload",
+		copyable=True)
+
 	_parset = base.EnumeratedUnicodeAttribute("parSet",
 		description="Preselect a default parameter set; form gives what"
 			" our framework adds to form queries.", default=base.NotGiven,
@@ -407,6 +412,7 @@ class DataURL(base.Structure):
 
 		if self.httpMethod=="POST":
 			if self.postPayload:
+				headers["content-type"] = self.httpPostMediaType
 				with open(self.postPayload, "rb") as f:
 					payload = f.read()
 
@@ -696,17 +702,28 @@ class RegTest(procdef.ProcApp, unittest.TestCase):
 				key, self.headers))
 
 	@utils.document
-	def getFirstVOTableRow(self):
+	def getFirstVOTableRow(self, rejectExtras=True):
 		"""interprets data as a VOTable and returns the first row as a dictionary
 
-		In test use, make sure the VOTable returned is sorted, or you will get
-		randomly failing tests.  Ideally, you'll constrain the results to just
-		one match; database-querying cores (which is where order is an
-		issue) also honor _DBOPTIONS_ORDER).
+		It will normally ensure that only one row is returned.  To make it
+		silently discard extra rows, make sure the result is sorted, or you will
+		get randomly failing tests.  Database-querying cores (which is where order
+		is an issue) also honor _DBOPTIONS_ORDER).
 		"""
 		data, metadata = votable.loads(self.data)
-		for row in metadata.iterDicts(data):
-			return row
+		rows = metadata.iterDicts(data)
+		result = next(rows)
+
+		if rejectExtras:
+			try:
+				secondRow = next(rows)
+			except StopIteration:
+				pass
+			else:
+				raise AssertionError(
+					f"getFirstVOTableRow swallows a row: {secondRow}")
+
+		return result
 		
 	@utils.document
 	def getVOTableRows(self):
