@@ -18,6 +18,9 @@ import functools
 import math
 from math import sin, cos, tan, atan, acos
 
+from astropy import coordinates
+from astropy import units as u
+
 from gavo import utils
 from gavo.stc import common
 from gavo.stc import conform
@@ -50,9 +53,9 @@ def getHeading(long1, lat1, long2, lat2):
 		#	starting on north pole
 		return math.pi
 
-	zeta = utils.spherDist(
-		utils.spherToCart(long1, lat1),
-		utils.spherToCart(long2, lat2))
+	zeta = coordinates.angular_separation(
+		long1*u.rad, lat1*u.rad,
+		long2*u.rad, lat2*u.rad).to(u.rad).value
 	cosPhi = (sin(lat2)-sin(lat1)*cos(zeta))/(sin(zeta)*cos(lat1))
 	if abs(cosPhi-1)<1e-10:
 		return 0
