@@ -23,6 +23,7 @@ from gavo.stc import sphermath
 from gavo.stc import times
 from gavo.stc import units
 from gavo.utils import DEG, ARCSEC
+from gavo.utils import mathtricks
 
 
 ############### Basic definitions for transforms
@@ -216,8 +217,8 @@ def getPrecMatrix(fromEquinox, toEquinox, precTheory):
 	"""
 	zeta, z, theta = precTheory(fromEquinox, toEquinox)
 	return numpy.dot(
-		numpy.dot(sphermath.getRotZ(-z), sphermath.getRotY(theta)),
-		sphermath.getRotZ(-zeta))
+		numpy.dot(mathtricks.getRotZ(-z), mathtricks.getRotY(theta)),
+			mathtricks.getRotZ(-zeta))
 
 
 _nullMatrix = numpy.zeros((3,3))
@@ -438,7 +439,7 @@ def _getEclipticMatrix(epoch):
 	"""
 	t = times.getSeconds(epoch-times.dtJ2000)/common.secsPerJCy
 	obliquity = (84381.448+(-46.8150+(-0.00059+0.001813*t)*t)*t)*ARCSEC
-	return sphermath.getRotX(obliquity)
+	return mathtricks.getRotX(obliquity)
 
 def _getFromEclipticMatrix(fromNode, toNode, sixTrans):
 	return threeToSix(numpy.transpose(_getEclipticMatrix(fromNode[1])))
