@@ -15,6 +15,7 @@ import numpy
 
 from gavo.stc import common
 from gavo.stc import units
+from gavo.utils import mathtricks
 
 from gavo.utils.mathtricks import cartToSpher, spherToCart #noflake
 
@@ -31,34 +32,15 @@ _svVTimeUnit = ("d", "d", "d")
 _lightAUd = 86400.0/499.004782
 
 
-def getRotX(angle):
-	"""returns a 3-rotation matrix for rotating angle radians around x.
-	"""
-	c, s = math.cos(angle), math.sin(angle)
-	return numpy.array([[1, 0, 0], [0, c, s], [0, -s, c]])
-
-
-def getRotY(angle):
-	"""returns a 3-rotation matrix for rotating angle radians around y.
-	"""
-	c, s = math.cos(angle), math.sin(angle)
-	return numpy.array([[c, 0, -s], [0, 1, 0], [s, 0, c]])
-
-
-def getRotZ(angle):
-	"""returns a 3-rotation matrix for rotating angle radians around u.
-	"""
-	c, s = math.cos(angle), math.sin(angle)
-	return numpy.array([[c, s, 0], [-s, c, 0], [0, 0, 1]])
-
-
 def getMatrixFromEulerAngles(z1, x, z2):
 	"""returns a 3-rotation matrix for the z-x-z Euler angles.
 
 	There are some functions to obtain such angles below.
 	"""
 	return numpy.dot(
-		numpy.dot(getRotZ(z2), getRotX(x)), getRotZ(z1))
+		numpy.dot(mathtricks.getRotZ(z2),
+			mathtricks.getRotX(x)),
+			mathtricks.getRotZ(z1))
 
 
 def getEulerAnglesFromMatrix(matrix):
