@@ -179,7 +179,8 @@ class MultiSelectChoice(SelectChoice):
 			optValue = iformal.IKey(item).key()
 			optLabel = iformal.ILabel(item).label()
 			optValue = converter.fromType(optValue)
-			option = T.option(value=str(optValue))[str(optLabel)]
+			option = T.option(value=widget.stringifyOptionValue(optValue))[
+				str(optLabel)]
 			if optValue in value:
 				option = option(selected='selected')
 			tag[option]
