@@ -8,7 +8,7 @@ They are collected in contextGrammars, entities creating inputArgs
 items.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -59,13 +59,13 @@ class InputKey(column.ParamBase):
 	  presumably right for almost everyone, having to delete stuff
 	  you don't want over and over is really annoying).
 	* adaptToRenderer -- a true boolean literal here causes the param
-	  to be adapted for the renderer (e.g., float could become vizierexpr-float).
-	  You'll usually not want this, because the expressions are 
+	  to be adapted for the renderer (e.g., float could become vexpr-float).
+	  You'll usually not want this, because the expressions are
 	  generally evaluated by the database, and the condDescs do the
 	  adaptation themselves.  This is mainly for rare situations like
 	  file uploads in custom cores.
 	* notForRenderer -- a renderer name for which this inputKey is suppressed
-	* onlyForRenderer -- a renderer name for which this inputKey will be 
+	* onlyForRenderer -- a renderer name for which this inputKey will be
 	  preserved; it will be dropped for all others.
 	"""
 	name_ = "inputKey"
@@ -103,7 +103,7 @@ class InputKey(column.ParamBase):
 		" Exception (that will be turned into a ValidationError).  The"
 		" input string is available as input.  Note that need to be able"
 		" to handle incoming None-s gracefully (e.g.,"
-		" ``input and input.lower()``).", 
+		" ``input and input.lower()``).",
 		copyable=True)
 	
 	# Don't validate meta for these -- while they are children
@@ -119,7 +119,7 @@ class InputKey(column.ParamBase):
 		if self.restrictedMode and (
 				self.widgetFactory
 				or self.preparse):
-			raise base.RestrictedElement(self.name_, 
+			raise base.RestrictedElement(self.name_,
 				hint="The preparse and widgetFactory attribute are not allowed"
 					" in restricted mode.")
 
@@ -272,7 +272,7 @@ class InputTD(base.Structure, base.StandardMacroMixin):
 	adapting them to the current renderer.  Their main use is for the derivation
 	of contextGrammars.
 
-	They can carry metadata, though, which is sometimes convenient when 
+	They can carry metadata, though, which is sometimes convenient when
 	transporting information from the parameter parsers to the core.
 	
 	For the typical dbCores (and friends), these are essentially never
@@ -286,8 +286,8 @@ class InputTD(base.Structure, base.StandardMacroMixin):
 	name_ = "inputTable"
 
 	_inputKeys = rscdef.ColumnListAttribute("inputKeys",
-		childFactory=InputKey, 
-		description='Input parameters for this table.', 
+		childFactory=InputKey,
+		description='Input parameters for this table.',
 		copyable=True)
 
 	_groups = base.StructListAttribute("groups",
@@ -372,7 +372,7 @@ class CoreArgs(base.MetaMixin):
 
 	@classmethod
 	def fromRawArgs(cls, inputTD, rawArgs, contextGrammar=None):
-		"""returns a CoreArgs instance built from an inputDD and 
+		"""returns a CoreArgs instance built from an inputDD and
 		ContextGrammar-parseable rawArgs.
 
 		contextGrammar can be overridden, e.g., to cache or to add
@@ -389,7 +389,7 @@ class ContextRowIterator(grammars.RowIterator):
 	"""is a row iterator over "contexts", i.e. single dictionary-like objects.
 
 	The source token expected here can be a request.strargs style dictionary
-	with lists of strings as arguments, or a parsed dictionary from 
+	with lists of strings as arguments, or a parsed dictionary from
 	gavo.formal. Non-list literals in the latter are packed into a list to ensure
 	consistent behaviour.
 	"""
@@ -410,11 +410,19 @@ class ContextRowIterator(grammars.RowIterator):
 		"""
 		inKeys = set(n.lower() for n in self.sourceToken)
 		expectedKeys = set(k.name.lower() for k in self.grammar.iterInputKeys())
+
 		if inKeys-expectedKeys:
+			badKeys = inKeys-expectedKeys
+
+			if len(badKeys)==1:
+				fieldName = next(iter(badKeys))
+			else:
+				fieldName = "(various)"
+
 			raise base.ValidationError("The following parameter(s) are"
 				" not accepted by this service: %s"%",".join(
-					sorted(inKeys-expectedKeys)),
-				"(various)")
+					sorted(badKeys)),
+				fieldName)
 
 	def _iterRows(self):
 		# we don't really return any rows, but this is where our result
@@ -422,7 +430,7 @@ class ContextRowIterator(grammars.RowIterator):
 		if self.grammar.rejectExtras:
 			self._ensureNoExtras()
 	
-		self.coreArgs = {} 
+		self.coreArgs = {}
 
 		for ik in self.grammar.iterInputKeys():
 			self.locator = "param %s"%ik.name
@@ -474,8 +482,8 @@ class ContextGrammar(grammars.Grammar):
 	"""
 	name_ = "contextGrammar"
 
-	_inputTD = base.ReferenceAttribute("inputTD", 
-		default=base.NotGiven, 
+	_inputTD = base.ReferenceAttribute("inputTD",
+		default=base.NotGiven,
 		description="The input table from which to take the input keys",
 		copyable=True)
 
