@@ -10,7 +10,7 @@ step ::= somethingMagicallyDefined
 range ::= [literal] rangeSep literal | literal rangeSep
 steppedRange ::= range [rangeSep step]
 qualification ::= qualSep somethingMagicallyDefined
-listItem ::= steppedRange | literal 
+listItem ::= steppedRange | literal
 rangeList ::= listItem {valSep listItem} [qualification]
 
 This defines a regular language, and we're going to slaughter it using
@@ -22,7 +22,7 @@ dates), parsing is done by the specific PQLPar types (fromLiteral).  See
 the PQLPar docstring for further info.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -73,18 +73,18 @@ class PQLRange(object):
 	def __init__(self, value=None, start=None, stop=None, step=None):
 		self.start, self.stop, self.step = start, stop, step
 		self.value = value
-		if (self.step is not None 
+		if (self.step is not None
 				and (self.start is None or self.stop is None)):
 			raise ValueError("Open intervals cannot have steps")
-		if (self.value is None 
+		if (self.value is None
 				and (self.start is None and self.stop is None)):
 			raise ValueError("Doubly open intervals are not allowed")
 
 	def __eq__(self, other):
 		return (isinstance(other, PQLRange)
-			and self.value==other.value 
-			and self.start==other.start 
-			and self.stop==other.stop 
+			and self.value==other.value
+			and self.start==other.start
+			and self.stop==other.stop
 			and self.step==other.step)
 
 	def __repr__(self):
@@ -126,8 +126,8 @@ class PQLRange(object):
 			else:
 				start, stop, step = vals
 			return cls(
-				start=_parsePQLValue(start, vp=valParser), 
-				stop=_parsePQLValue(stop, 1, vp=valParser), 
+				start=_parsePQLValue(start, vp=valParser),
+				stop=_parsePQLValue(stop, 1, vp=valParser),
 				step=_parsePQLValue(step, 1, vp=stepParser))
 		except ValueError as ex:
 			raise base.LiteralParseError("range within %s"%destName, literal,
@@ -140,7 +140,7 @@ class PQLRange(object):
 		if self.value is not None:
 			return set([self.value])
 		elif (self.step is not None \
-				and self.start is not None 
+				and self.start is not None
 				and self.stop is not None):
 			if (self.stop-self.start)/abs(self.step)+1e-10>2000:
 				raise ValueError("Too many steps; will not check discretely")
@@ -163,26 +163,26 @@ class PQLRange(object):
 
 		# Single Value
 		if self.value is not None:
-			return "%s = %%(%s)s"%(cmpExpr, 
+			return "%s = %%(%s)s"%(cmpExpr,
 				base.getSQLKey(colName, self.value, sqlPars))
 		
 		# Discrete Set
 		try:
-			return "%s IN %%(%s)s"%(cmpExpr, base.getSQLKey(colName, 
+			return "%s IN %%(%s)s"%(cmpExpr, base.getSQLKey(colName,
 				self.getValuesAsSet(), sqlPars))
 		except ValueError: # Not a discrete set
 			pass
 
 		# At least one half-open or non-stepped range
 		if self.start is None and self.stop is not None:
-			return "%s <= %%(%s)s"%(cmpExpr, 
+			return "%s <= %%(%s)s"%(cmpExpr,
 				base.getSQLKey(colName, self.stop, sqlPars))
 		elif self.start is not None and self.stop is None:
-			return "%s >= %%(%s)s"%(cmpExpr, 
+			return "%s >= %%(%s)s"%(cmpExpr,
 				base.getSQLKey(colName, self.start, sqlPars))
 		else:
 			assert self.start is not None and self.stop is not None
-			return "%s BETWEEN %%(%s)s AND %%(%s)s "%(cmpExpr, 
+			return "%s BETWEEN %%(%s)s AND %%(%s)s "%(cmpExpr,
 				base.getSQLKey(colName, self.start, sqlPars),
 				base.getSQLKey(colName, self.stop, sqlPars))
 
@@ -246,9 +246,9 @@ class PQLNoRange(PQLRange):
 
 	def __eq__(self, other):
 		return (isinstance(other, PQLRange)
-			and self.value==other.value 
-			and self.start==other.start 
-			and self.stop==other.stop 
+			and self.value==other.value
+			and self.start==other.start
+			and self.stop==other.stop
 			and self.step==other.step)
 
 	def __str__(self):
@@ -272,7 +272,7 @@ class PQLPar(object):
 	and an attribute ranges, a list of PQLRange objects.
 	
 	As a client, you will usually construct PQLPar objects using the
-	fromLiteral class method; it takes a PQL literal and a name to be 
+	fromLiteral class method; it takes a PQL literal and a name to be
 	used for LiteralParseErrors it may raise.
 
 	The plain PQLPar parses string ranges and does not allow steps.
@@ -285,8 +285,8 @@ class PQLPar(object):
 
 	PQLPars usually support a covers(value) method that you can
 	pass a value having the required type; it will return whether or
-	not value would be picked up by the condition formulated in PQL.  
-	Some advanced PQLPars do not support this method and will 
+	not value would be picked up by the condition formulated in PQL.
+	Some advanced PQLPars do not support this method and will
 	raise a ValueError if called.
 
 	Since "PQL" is totally crazy, not even the range parser is constant.
@@ -347,7 +347,7 @@ class PQLPar(object):
 		for rangeMat in LIST_RE.finditer(listLiteral):
 			try:
 				ranges.append(
-					cls.rangeClass.fromLiteral(rangeMat.group(1), destName, 
+					cls.rangeClass.fromLiteral(rangeMat.group(1), destName,
 						cls.valParser, cls.stepParser))
 			except base.LiteralParseError as ex:
 				ex.pos = rangeMat.start()
@@ -394,14 +394,14 @@ class PQLPar(object):
 			return self.ranges[0].getSQL(colName, sqlPars, cmpExpr=cmpExpr)
 
 		try:
-			return "%s IN %%(%s)s"%(cmpExpr, base.getSQLKey(colName, 
+			return "%s IN %%(%s)s"%(cmpExpr, base.getSQLKey(colName,
 				self.getValuesAsSet(), sqlPars))
 		except ValueError:  # at least one open or non-stepped range
 			return "(%s)"%" OR ".join(
 				r.getSQL(colName, sqlPars, cmpExpr=cmpExpr) for r in self.ranges)
 
 	def covers(self, value):
-		"""returns true if value is within the ranges specified by the PQL 
+		"""returns true if value is within the ranges specified by the PQL
 		expression.
 
 		value must be type-true, i.e., you are responsible for converting it
@@ -444,9 +444,9 @@ class PQLDatePar(PQLPar):
 		"""returns an SQL condition expressing the PQL constraint for colName.
 
 		In addition to the usual parameters, we here accept an additional
-		argument convert with possible values None (meaning timestamp, 
-		which is the default) mjd, jd, and jy, which represents how the 
-		datetimes are represented in the database.  
+		argument convert with possible values None (meaning timestamp,
+		which is the default) mjd, jd, and jy, which represents how the
+		datetimes are represented in the database.
 		"""
 		converter = {
 			None: utils.identity,
@@ -469,7 +469,7 @@ class PQLDatePar(PQLPar):
 class PQLPositionPar(PQLPar):
 	"""a PQL position parameter, as for SSA.
 
-	Cones and intervals or real lists do not mix; we support STC-S 
+	Cones and intervals or real lists do not mix; we support STC-S
 	identifiers as qualifiers.
 
 	The literals here are basically two-float lists.
@@ -511,24 +511,24 @@ class PQLPositionPar(PQLPar):
 				lastCoo = None
 
 	def getConeSQL(self, colName, sqlPars, coneSize):
-		"""returns SQL to compute the constraint agains a pgsphere point column.
+		"""returns SQL to compute the constraint against a pgsphere point column.
 		"""
 		sizeName = base.getSQLKey("size", coneSize*DEG, sqlPars)
 		parts = []
 		for ra, dec in self._iterICRSPositions():
 			parts.append("%s <@ scircle(%%(%s)s, %%(%s)s)"%(colName,
-				base.getSQLKey("pos", pgsphere.SPoint.fromDegrees(ra, dec), 
+				base.getSQLKey("pos", pgsphere.SPoint.fromDegrees(ra, dec),
 					sqlPars), sizeName))
 		return "(%s)"%" OR ".join(parts)
 
 	def getQ3CSQL(self, raCol, decCol, sqlPars, coneSize):
-		"""returns SQL to compute the constraint agains a pgsphere point column.
+		"""returns SQL to compute the constraint against a pgsphere point column.
 		"""
 		sizeName = base.getSQLKey("size", coneSize, sqlPars)
 		parts = []
 		for ra, dec in self._iterICRSPositions():
 			parts.append("q3c_radial_query({}, {}, %({})s, %({})s, %({})s)".format(
-				raCol, decCol, 
+				raCol, decCol,
 				base.getSQLKey("ra", ra, sqlPars),
 				base.getSQLKey("dec", dec, sqlPars),
 				sizeName))
@@ -556,7 +556,7 @@ class PQLFloatPar(PQLPar):
 				lowerColName, upperColName, sqlPars)
 		else:
 			return "(%s)"%" OR ".join(
-				r.getSQLForInterval(lowerColName, upperColName, sqlPars) 
+				r.getSQLForInterval(lowerColName, upperColName, sqlPars)
 					for r in self.ranges)
 
 
@@ -710,7 +710,7 @@ def _makeFactory(parType):
 		except ValueError:
 			raise base.ui.logOldExc(utils.ValidationError(
 				"Invalid input for type %s"
-				" (valid PQL literals are described in the help)"%field.type, 
+				" (valid PQL literals are described in the help)"%field.type,
 				field.name))
 	return factory
 
