@@ -1110,6 +1110,8 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		meta.
 		"""
 		if self.getMeta("creationDate"):
+			# depend on creationDate because to filter out totally whacko
+			# RDs that will never make it to the Registry.
 			return base.META_CLASSES_FOR_KEYS["_related"](
-				self.getURL("howtocite", False),
+				self.getURL("howtocite", absolute=True),
 				title="Advice on citing this resource")
