@@ -439,14 +439,17 @@ class FancyQueryCore(TableBasedCore, base.RestrictionMixin):
 	Unless you select \*, you *must* define the outputTable here;
 	Weird things will happen if you don't.
 
-	The queriedTable attribute is ignored.
+	The queriedTable attribute is mandatory but only used for name resolution
+	(e.g., in outputTable).  Instead, define your FROM in the query attribute
+	(and use a %s where the condDescs-generated WHERE clause should end up)
 	"""
 	name_ = "fancyQueryCore"
 
 	_query = base.UnicodeAttribute("query", description="The query to"
 		" execute.  It must contain exactly one %s where the generated"
 		" where clause is to be inserted.  Do not write WHERE yourself."
-		" All other percents must be escaped by doubling them.",
+		" All other percents must be escaped by doubling them.  This"
+		" string is macro-expanded in the core's service.",
 		default=base.Undefined,
 		copyable=True)
 	_timeout = base.FloatAttribute("timeout", default=5.,
@@ -464,7 +467,7 @@ class FancyQueryCore(TableBasedCore, base.RestrictionMixin):
 				res = rsc.TableForDef(
 					self.outputTable,
 					rows=list(conn.queryToDicts(
-						self.query%fragment,
+						service.expand(self.query%fragment),
 						pars,
 						timeout=self.timeout,
 						caseFixer=self.outputTable.caseFixer)))
