@@ -178,7 +178,7 @@ def suggestucds(querier, args):
 	apiURL = "http://dc.zah.uni-heidelberg.de/ucds/ui/ui/api"
 
 	def getMatches(description):
-		res = utils.urlopenRemote(apiURL, {"description": description})
+		res = utils.urlopenRemote(apiURL, data={"description": description})
 		data, metadata = votable.load(res)
 		if metadata:
 			return list(metadata.iterDicts(data))
