@@ -144,13 +144,21 @@ def main():
 	# this prominent place for overriding a user decision...
 	os.umask(0o02)
 
-	try:
-		if len(sys.argv)>1 and sys.argv[1]=="init":
-			# Special case: initial setup, no api working yet
+	# Special case: initial setup, no api working yet, and if things fail,
+	# I cannot rely on normal DaCHS error reporting, which needs significant
+	# parts of DaCHS.
+	if len(sys.argv)>1 and sys.argv[1]=="init":
+		try:
 			del sys.argv[1]
 			from gavo.user import initdachs
 			sys.exit(initdachs.main())
+		except Exception as exc:
+			sys.stderr.write(f"Ouch -- Exception escaped from initdachs: {exc}\n")
+			sys.exit(1)
 
+	# Normal case: try executing what the user asked for and report
+	# errors centrally.
+	try:
 		opts, module, funcName = _parseCLArgs()
 		from gavo import base
 		from gavo import utils
