@@ -11,6 +11,7 @@ docs.
 
 import locale
 import inspect
+import os
 import re
 import sys
 import textwrap
@@ -265,9 +266,8 @@ class MacroDoc(object):
 		"""
 		# macros have args in {}, of course there's no self, and null-arg
 		# macros have not {}...
-		args, varargs, varkw, defaults = inspect.getargspec(self.macFunc)
-		args = inspect.formatargspec(args[1:], varargs, varkw, defaults
-			).replace("(", "{").replace(")", "}").replace("{}", ""
+		sig = inspect.signature(self.macFunc)
+		args = str(sig).replace("(", "{").replace(")", "}").replace("{}", ""
 			).replace(", ", "}{")
 		content.addRaw("::\n\n  \\%s%s\n\n"%(self.name, args))
 		content.addRaw(utils.fixIndentation(
@@ -420,8 +420,7 @@ def _getModuleFunctionDocs(module):
 		if hasattr(ob, "buildDocsForThis"):
 			if ob.__doc__ is None:  # silently ignore if no docstring
 				continue
-			res.append(
-				"*%s%s*"%(name, inspect.formatargspec(*inspect.getargspec(ob))))
+			res.append("``"+name+str(inspect.signature(ob))+"``")
 			res.append(utils.fixIndentation(ob.__doc__, "  ", 1))
 			res.append("")
 	return "\n".join(res)
@@ -552,8 +551,7 @@ def getAPIDocs(docStructure):
 		content.makeSpace()
 		if whatsit=="Function ":
 			content.addNormalizedPara(
-				"Signature: ``%s%s``"%(
-					name, inspect.formatargspec(*inspect.getargspec(obj))))
+				"Signature: ``{}{}``".format(name, str(inspect.signature(obj))))
 		content.makeSpace()
 
 		if whatsit=="Constant ":
@@ -573,9 +571,13 @@ def makeReferenceDocs():
 	**WARNING**: refdoc.rstx can execute arbitrary code right now.  We
 	probably want to change this to having macros here.
 	"""
+	if not os.path.exists("refdoc.rstx"):
+		raise base.ReportableError("This needs to be run in the docs"
+			" directory of a checkout or source tarball (the location"
+			" of refdoc.rstx).")
+
 	res, docStructure = [], DocumentStructure()
-	f = pkg_resources.resource_stream("gavo",
-		"resources/templates/refdoc.rstx")
+	f = open("refdoc.rstx", "rb")
 	
 	parseState = "content"
 	code = []
