@@ -50,7 +50,7 @@ def setupServer(rootPage, runIntervalJobs=True):
 	which would be unwelcome on debug instances.
 	"""
 	manageResourceLimits()
-	config.setMeta("upSince", utils.formatISODT(datetime.datetime.utcnow()))
+	base.setGlobalMeta("upSince", utils.formatISODT(datetime.datetime.utcnow()))
 	base.ui.notifyWebServerUp()
 
 	if not runIntervalJobs:
@@ -220,19 +220,31 @@ def daemonize(logFile, callable):
 		os._exit(0)
 
 
-def _configureTwistedLog():
-	logFile = getLogFile("web.log")
-	logPath = logFile.name
-	logFile.close()
+def _configureTwistedLog(theLog=None):
+	"""sets up the destination of the twisted web.log.
 
-	theLog = logfile.LogFile(
-		os.path.basename(logPath),
-		os.path.dirname(logPath))
-	log.startLogging(theLog, setStdout=False)
-	def rotator():
-		theLog.shouldRotate()
-		reactor.callLater(86400, rotator)
-	rotator()
+	theLog can be a file to write to rather than the default web.log.  No
+	rotation will take place on that.
+
+	Note that dcErrors and dcInfos are set up in user.logui.
+	"""
+	if theLog is None:
+		# create the file with the right permissions before passing it on
+		logFile = getLogFile("web.log")
+		logPath = logFile.name
+		logFile.close()
+
+		theLog = logfile.LogFile(
+			os.path.basename(logPath),
+			os.path.dirname(logPath))
+
+		log.startLogging(theLog, setStdout=False)
+		def rotator():
+			theLog.shouldRotate()
+			reactor.callLater(86400, rotator)
+		rotator()
+	else:
+		log.startLogging(theLog, setStdout=False)
 
 
 def getLogFile(baseName):
@@ -543,8 +555,7 @@ class ExitPage(twresource.Resource):
 @exposedFunction(help="run a server and remain in the foreground, dumping"
 	" all kinds of stuff to the terminal")
 def debug(args):
-	log.startLogging(sys.stderr)
-	root.site.webLog = _configureTwistedLog()
+	root.site.webLog = _configureTwistedLog(sys.stderr)
 	base.DEBUG = True
 	root.root.child_exit = ExitPage()
 	reactor.listenTCP(
@@ -623,10 +634,12 @@ def updateCertificate(args):
 		"--account-key", "./account.key", "--csr", "dachs.csr",
 		"--acme-dir", acmeChallengeDir])
 	
-	# finally, glue together the signature with intermediary certificates
+	# finally, glue together the signature with intermediate certificates
 	# and the private key to get the PEM for twisted
 	f = utils.urlopenRemote(
-		"https://letsencrypt.org/certs/lets-encrypt-x3-cross-signed.pem")
+		# This link needs to be updated now and then.  If anyone knows a
+		# stable URI for "*the* itermediate certs for LE": tell me.
+		"https://letsencrypt.org/certs/lets-encrypt-r3.pem")
 	intermediate = f.read()
 
 	with open("server.key", "rb") as f:
