@@ -3,7 +3,7 @@ Stuff dealing with the upgrade of the database schema.
 
 From software version 0.8.2 on, there is a dc.metastore table with a key
 schemaversion.  Each change in the central schema increases the value
-(interpreted as an integer) by one, and this module will contain a 
+(interpreted as an integer) by one, and this module will contain a
 corresponding upgrader.
 
 An upgrader inherits form the Upgrader class.  See there for more details.
@@ -13,7 +13,7 @@ upgrade does everything required to bring the what's in the database in sync
 with the code (or so I hope).
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -32,7 +32,7 @@ from gavo import rscdesc  #noflake: for cache registration
 from gavo import utils
 
 
-CURRENT_SCHEMAVERSION = 33
+CURRENT_SCHEMAVERSION = 34
 
 
 class _COMMIT(object):
@@ -62,7 +62,7 @@ def _updateTAP_SCHEMA(connection):
 	"""
 	from gavo.protocols import tap
 
-	toDo = [r[0] for r in 
+	toDo = [r[0] for r in
 		connection.query(
 			"SELECT DISTINCT sourceRD FROM tap_schema.tables")]
 
@@ -166,7 +166,7 @@ def dumpExtensionUpdater(conn):
 
 
 class Upgrader(object):
-	"""A specification to upgrade from some schema version to another schema 
+	"""A specification to upgrade from some schema version to another schema
 	version.
 
 	Upgraders live as uninstanciated classes.  Their version attribute gives the
@@ -178,20 +178,20 @@ class Upgrader(object):
 	always need to see if the manipulation already has been performed.
 	Use relationExists and/or getColumnNamesFor as appropriate.
 
-	An upgrader can have attributes named u_<seqno>_<something>.  
+	An upgrader can have attributes named u_<seqno>_<something>.
 	
-	These can be 
+	These can be
 	
 	* strings, which are then directly executed in the database (but you
 	  should only do that for idempotent operations)
-	* classmethods, which will be executed in sequence (version, seqno) 
+	* classmethods, which will be executed in sequence (version, seqno)
 	  after *all* schemachange methods are executed.
 	
-	The methods will be called with a connection argument.  You 
+	The methods will be called with a connection argument.  You
 	must not commit this connection.  You must not swallow exceptions
 	that have left the connection unready (i.e., require a rollback).
 
-	Note that if you run rsc.makeData, you MUST pass both 
+	Note that if you run rsc.makeData, you MUST pass both
 	connection=connection and runCommit=False in order to avoid messing
 	people's lives up.
 
@@ -200,10 +200,10 @@ class Upgrader(object):
 	The update of the schemaversion is done automatically, you don't
 	need to worry about it.
 
-	Note that whenever you change the schema in a system RD, the 
+	Note that whenever you change the schema in a system RD, the
 	corresponding upgrader needs to be a schemachange; otherwise,
 	it's very likely that any use of the system RD will fail, which
-	usually makes an installation not upgradeable without manual 
+	usually makes an installation not upgradeable without manual
 	intervention.
 	"""
 	version = None
@@ -255,7 +255,7 @@ class To0Upgrader(Upgrader):
 				res = base.getRD(sourceRD).getById(resId)
 				authors = "; ".join(m.getContent("text") #noflake: used through locals
 					for m in res.iterMeta("creator.name", propagate=True))
-			except: 
+			except:
 				# don't worry if fetching authors fails; people will notice...
 				pass
 			else:
@@ -370,7 +370,7 @@ class To6Upgrader(Upgrader):
 	def u_020_remetaObscore(cls, connection):
 		"""update obscore metadata to fix the erroneous id"""
 		rsc.makeData(base.caches.getRD("//obscore").getById("create"),
-			connection=connection, runCommit=False, 
+			connection=connection, runCommit=False,
 			parseOptions=rsc.getParseOptions(metaOnly=True))
 
 
@@ -470,7 +470,7 @@ class To11Upgrader(Upgrader):
 							).exists():
 						continue
 
-					# table needs re-importing, see if you can find a corresponding 
+					# table needs re-importing, see if you can find a corresponding
 					# data element
 					for dd in rd.dds:
 						for make in dd.makes:
@@ -536,7 +536,7 @@ class To15Upgrader(Upgrader):
 		"""adding creationTime to UWS tables for UWS 1.1 support"""
 		q = base.UnmanagedQuerier(connection)
 		for tableId, tableName in [
-				("//datalink#datalinkjobs", "dc.datalinkjobs"), 
+				("//datalink#datalinkjobs", "dc.datalinkjobs"),
 				("//uws#userjobs", "uws.userjobs"),
 				("//tap#tapjobs", "tap_schema.tapjobs")]:
 			if q.getTableType(tableName) is not None:
@@ -565,7 +565,7 @@ class To16Upgrader(Upgrader):
 			connection.execute("ALTER TABLE TAP_SCHEMA.tables"
 				" ADD COLUMN table_index SMALLINT")
 
-	@classmethod 
+	@classmethod
 	def s_20_upgrade_tap_schema_columns(cls, connection):
 		"""Adding 1.1 columns to TAP_SCHEMA.columns.
 		"""
@@ -595,7 +595,7 @@ class To16Upgrader(Upgrader):
 						"SELECT DISTINCT tablename FROM ivoa._obscoresources"
 						" WHERE sqlfragment LIKE '%%''spectrum'' AS text) AS dataprod%%'"):
 					table = rsc.TableForDef(
-							base.getTableDefForTable(connection, row[0]), 
+							base.getTableDefForTable(connection, row[0]),
 						connection=connection)
 					showProgress(" "+row[0])
 					scripting.PythonScriptRunner(script).run(table)
@@ -689,7 +689,7 @@ class To21Upgrader(Upgrader):
 	def s_10_update_obscoresources(cls, connection):
 		"""adding sourcerd column to obscore sources table"""
 		if (relationExists("ivoa._obscoresources", connection)
-				and not "sourcerd" in 
+				and not "sourcerd" in
 					getColumnNamesFor("//obscore#_obscoresources", connection)):
 			connection.execute("ALTER TABLE ivoa._obscoresources"
 				" ADD COLUMN sourcerd TEXT")
@@ -793,7 +793,7 @@ class To27Upgrader(Upgrader):
 		if not relationExists("dc.simple_col_stats", connection):
 			# this will first make tableName alone a primary key on dc.tablemeta
 			tablemeta = rsc.TableForDef(
-				base.resolveCrossId("//dc_tables#tablemeta"), 
+				base.resolveCrossId("//dc_tables#tablemeta"),
 				connection=connection)
 			tablemeta.updateMeta()
 			tablemeta._dropPrimaryKey()
@@ -878,7 +878,7 @@ class To30Upgrader(Upgrader):
 	@classmethod
 	def u_20_create_discrete_text_values(cls, connection):
 		"""Creating dc.discrete_string_values"""
-		# do this 
+		# do this
 		rsc.TableForDef(
 			base.resolveCrossId("//dc_tables#discrete_string_values"),
 			connection=connection,
@@ -930,12 +930,21 @@ class To33Upgrader(Upgrader):
 		_updateTAP_SCHEMA(connection)
 
 
-def iterStatements(startVersion, endVersion=CURRENT_SCHEMAVERSION, 
+class To34Upgrader(Upgrader):
+	version = 33
+	@classmethod
+	def u_000_update_funcs(cls, connection):
+		"""update GAVO server-side functions"""
+		rsc.makeData(base.caches.getRD("//adql").getById("make_udfs"),
+			connection=connection, runCommit=False)
+
+
+def iterStatements(startVersion, endVersion=CURRENT_SCHEMAVERSION,
 		upgraders=None):
 	"""yields all upgraders from startVersion to endVersion in sequence.
 	"""
 	toRun = []
-	for upgrader in utils.iterDerivedClasses(Upgrader, 
+	for upgrader in utils.iterDerivedClasses(Upgrader,
 			upgraders or list(globals().values())):
 		if startVersion<=upgrader.version<endVersion:
 			toRun.append(upgrader)
@@ -1022,7 +1031,7 @@ def parseCommandLine():
 	parser.add_argument("-t", "--update-tap-schema",
 		help="Force the re-generation of the TAP schema.  This is basic"
 		" housekeeping that's perhaps smart to do on the occasion of"
-		" an upgrade.", 
+		" an upgrade.",
 		dest="updateTAPSchema", action="store_true")
 	return parser.parse_args()
 
