@@ -26,8 +26,8 @@ from gavo.utils.autonode import AutoNode
 
 from gavo.utils.codetricks import (silence, ensureExpression, compileFunction,
 	loadPythonModule, DeferredImport,
-	memoized, identity, runInSandbox, document,
-	buildClassResolver, CachedGetter, CachedResource, intToFunnyWord,
+	memoized, identity, document,
+	buildClassResolver, CachedGetter, intToFunnyWord,
 	IdManagerMixin,
 	addDefaults, iterDerivedClasses, iterDerivedObjects, iterConsecutivePairs,
 	importModule, loadInternalObject, printFrames, memoizeOn, forgetMemoized,
@@ -60,7 +60,7 @@ from gavo.utils.misctricks import (Undefined, QuotedName, getfirst,
 	couldBeABibcode,
 	parseKVLine, makeKVLine,
 	StreamBuffer, CaseSemisensitiveDict,
-	NotInstalledModuleStub, grouped,
+	grouped,
 	getCleanBytes, getDirtyBytes,
 	RateLimiter)
 
