@@ -7,17 +7,19 @@ Some fundamental algorithms not found in the standard library.
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
-
-from typing import (Any, Callable, Dict, FrozenSet, Iterator,
-	Hashable, List, Sequence, Set, Tuple, TypeVar, Union)
 import itertools
 
+from gavo.utils.dachstypes import (Any, Callable, Dict, FrozenSet,
+	Generic, Iterator, Hashable, List, Sequence, Set, Tuple, TypeVar, Union)
 
+
+# A node within an IndexedGraph
 IgNode = TypeVar("IgNode")
+# A Generic for "local" use
 T = TypeVar("T")
 
 
-class IndexedGraph:
+class IndexedGraph(Generic[IgNode]):
 	"""A graph that is indexed by both incoming and outgoing edges.
 
 	The constructor argument edges is an iterable of (from, to)-tuples,
