@@ -15,8 +15,11 @@ syntax trees in STC or ADQL.
 import itertools
 
 
+from gavo.utils.dachstypes import Any, Dict, List, Tuple
+
+
 class AutoNodeType(type):
-	"""A metaclass for AutoNodes..
+	"""A metaclass for AutoNodes.
 
 	The idea here is to define children in a class definition and make sure they
 	are actually present.
@@ -37,26 +40,26 @@ class AutoNodeType(type):
 	The defaults must return a repr that constructs them, since we create
 	a source fragment.
 	"""
-	def __init__(cls, name, bases, dict):
+	def __init__(cls, name: str, bases: Tuple[type, ...], dict: Dict):
 		cls._collectAttributes()
 		cls._buildConstructor()
 	
-	def _collectAttributes(cls):
-		cls._nodeAttrs = []
+	def _collectAttributes(cls) -> None:
+		cls._nodeAttrs: List[Tuple[str, Any]] = []
 		for name in dir(cls):
 			if name.startswith("_a_"):
 				cls._nodeAttrs.append((name[3:], getattr(cls, name)))
 	
-	def _buildConstructor(cls):
+	def _buildConstructor(cls) -> None:
 		argList, codeLines = ["self"], []
 		for argName, argDefault in cls._nodeAttrs:
 			argList.append("%s=%s"%(argName, repr(argDefault)))
 			codeLines.append("  self.%s = %s"%(argName, argName))
 		codeLines.append("  self._setupNode()\n")
 		codeLines.insert(0, "def constructor(%s):"%(", ".join(argList)))
-		ns = {}
+		ns: Dict[str, Any] = {}
 		exec("\n".join(codeLines), ns)
-		cls.__init__ = ns["constructor"]
+		cls.__init__ = ns["constructor"]  # type: ignore
 
 
 class AutoNode(object, metaclass=AutoNodeType):
@@ -97,7 +100,7 @@ class AutoNode(object, metaclass=AutoNodeType):
 		return self.__class__(**consArgs)
 
 	@classmethod
-	def cloneFrom(cls, other, **kwargs):
+	def cloneFrom(cls, other, **kwargs) -> 'AutoNode':
 		"""returns a shallow clone of other.
 
 		other should be of the same class or a superclass.
@@ -106,16 +109,16 @@ class AutoNode(object, metaclass=AutoNodeType):
 		consArgs.update(kwargs)
 		return cls(**consArgs)
 
-	def iterAttributes(self, skipEmpty=False):
+	def iterAttributes(self, skipEmpty: bool=False):
 		"""yields pairs of attributeName, attributeValue for this node.
 		"""
-		for name, _ in self._nodeAttrs:
+		for name, _ in self._nodeAttrs:  # type: ignore
 			val = getattr(self, name)
 			if skipEmpty and not val:
 				continue
 			yield name, val
 	
-	def iterChildren(self, skipEmpty=False):
+	def iterChildren(self, skipEmpty: bool=False):
 		for name, val in self.iterAttributes(skipEmpty):
 			if isinstance(val, (list, tuple)):
 				for c in val:
