@@ -8,6 +8,7 @@ external code.
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+from __future__ import annotations
 
 import ast
 import atexit
@@ -30,8 +31,14 @@ from gavo.utils import algotricks
 from gavo.utils import misctricks
 from gavo.utils import excs
 
+from gavo.utils.dachstypes import (Any, Callable, Dict, Filename,
+	Generator, Hashable, List, ModuleType, Optional,
+	Sequence, Set, StrToStrMap, TextIO, Tuple, TypeVar, TYPE_CHECKING)
 
-def document(origFun):
+if TYPE_CHECKING:
+	T = TypeVar("T")
+
+def document(origFun: Any):
 	"""is a decorator that adds a "buildDocsForThis" attribute to its argument.
 
 	This attribute is evaluated by documentation generators.
@@ -40,7 +47,7 @@ def document(origFun):
 	return origFun
 
 
-class CachedGetter(object):
+class CachedGetter:
 	"""A cache for a callable.
 
 	This is basically memoization, except that these are supposed
@@ -62,12 +69,14 @@ class CachedGetter(object):
 	cache before the cache is being returned.  If it returns false,
 	the resource is re-made (no concurrency control is enforced here).
 	"""
-	def __init__(self, getter, *args, **kwargs):
+	def __init__(self, getter: Callable, *args, **kwargs):
 		if getter is None:
 			getter = self.impl
-		self.cache, self.getter = None, getter
+		self.cache: Any = None
+		self.getter = getter
 		
-		self.isAlive = kwargs.pop("isAlive", None)
+		self.isAlive: Optional[Callable[[Any], bool]
+			] = kwargs.pop("isAlive", None)
 		self.args, self.kwargs = args, kwargs
 		self.lock = threading.Lock()
 	
@@ -94,22 +103,7 @@ class CachedGetter(object):
 		return self.cache
 
 
-class CachedResource(object):
-	"""is like CachedGetter but with a built-in getter.
-
-	Here, you define your class and have a class method impl returning
-	what you want.
-	"""
-	cache = None
-
-	@classmethod
-	def __new__(cls, arg):
-		if cls.cache is None:
-			cls.cache = cls.impl()
-		return cls.cache
-
-
-class DeferredImport(object):
+class DeferredImport:
 	"""A trivial deferred module loader.
 
 	Use this to delay the actual import of a module until it's actually
@@ -130,18 +124,19 @@ class DeferredImport(object):
 	"""
 	loadedModule = None
 
-	def __init__(self, moduleName, loadingCode):
-		self.parentGlobals = inspect.currentframe().f_back.f_globals
+	def __init__(self, moduleName: str, loadingCode:str):
+		# I now f_back is non-None here, so silence mypy
+		self.parentGlobals = inspect.currentframe().f_back.f_globals # type: ignore
 		self.moduleName = moduleName
 		self.loader = compile(loadingCode, "<modloader-%s>"%id(self), 'exec')
 		self.parentGlobals[moduleName] = self
 	
-	def __getattr__(self, name):
+	def __getattr__(self, name: str):
 		exec(self.loader, self.parentGlobals)
 		return getattr(self.parentGlobals[self.moduleName], name)
 
 
-class IdManagerMixin(object):
+class IdManagerMixin:
 	"""
 	A mixin for objects requiring unique IDs.
 	
@@ -171,30 +166,45 @@ class IdManagerMixin(object):
 	__cleanupPat = re.compile("[^A-Za-z0-9_]+")
 # Return a proxy instead of raising a KeyError here?  We probably do not
 # really want to generate xml with forward references, but who knows?
-	def __getIdMaps(self):
+	def __getIdMaps(self) -> Tuple[Dict, Dict]:
 		try:
 			return self.__objectToId, self.__idsToObject
 		except AttributeError:
-			self.__objectToId, self.__idsToObject = {}, {}
+			self.__objectToId: Dict[Hashable, str] = {}
+			self.__idsToObject: Dict[str, Hashable] = {}
 			return self.__objectToId, self.__idsToObject
 
-	def _fixSuggestion(self, suggestion, invMap):
+	def _fixSuggestion(self, suggestion: str, invMap: StrToStrMap
+			) -> str:
+		"""returns suggestion with some suffix to make in unique if it not
+		already is.
+		"""
 		for i in itertools.count(start=2):
 			newId = f"{suggestion}-{i:02d}"
 			if newId not in invMap:
 				return newId
+		return suggestion  # notreached
 
 	def cloneFrom(self, other):
 		"""takes the id management dictionaries from other.
 		"""
 		self.__objectToId, self.__idsToObject = other.__getIdMaps()
 
-	def makeIdFor(self, ob, suggestion=None):
+	def makeIdFor(self, ob: Hashable, suggestion: Optional[str]=None
+			) -> str:
+		"""returns a new id for ob (which must be Hashable).
+
+		suggestion can be a string giving what id ought to be; the function
+		will change it as necessary to make it a usable and unique id.
+
+		If the object already has an id, the method will raise a ValueError
+		(since DaCHS 2.8.1; returned None before).
+		"""
 		map, invMap = self.__getIdMaps()
 		if suggestion:
 			suggestion = self.__cleanupPat.sub("", suggestion)
 		if id(ob) in map:
-			return None
+			raise ValueError(f"{ob} already has the id {id(ob)}")
 
 		if suggestion is not None:
 			if suggestion in invMap:
@@ -212,7 +222,11 @@ class IdManagerMixin(object):
 			invMap[newId] = ob
 		return newId
 	
-	def getIdFor(self, ob):
+	def getIdFor(self, ob: Hashable) -> str:
+		"""returns the id for ob if it is known.
+
+		If ob does not yet have an id, the method raises a NotFoundError.
+		"""
 		try:
 			return self.__getIdMaps()[0][id(ob)]
 		except KeyError:
@@ -221,13 +235,23 @@ class IdManagerMixin(object):
 				" id of an object not managed by the id manager.  This usually"
 				" is a software bug.")
 
-	def getOrMakeIdFor(self, ob, suggestion=None):
+	def getOrMakeIdFor(self,
+			ob: Hashable, suggestion: Optional[str]=None) -> str:
+		"""returns the id of the (hashable) ob or creates one if it
+		does not already have one.
+
+		This is the function you would normally use with the id manager.
+		"""
 		try:
 			return self.getIdFor(ob)
 		except excs.NotFoundError:
 			return self.makeIdFor(ob, suggestion)
 
-	def getForId(self, id):
+	def getForId(self, id: str) -> Hashable:
+		"""returns the object associated with id.
+
+		This will raise a KeyError for unknown ids.
+		"""
 		try:
 			return self.__getIdMaps()[1][id]
 		except KeyError:
@@ -242,7 +266,7 @@ class NullObject(object):
 
 	This mainly here for use in scaffolding.
 	"""
-	def __getattr__(self, name):
+	def __getattr__(self, name: str):
 		return self
 	
 	def __call__(self, *args, **kwargs):
@@ -268,6 +292,7 @@ class _CmpType(type):
 	def __eq__(cls, other):
 		return False
 
+
 class _Comparer(object, metaclass=_CmpType):
 	def __init__(self, *args, **kwargs):
 		raise excs.Error(
@@ -334,16 +359,19 @@ class AllEncompassingSet(set):
 	def __bool__(self):
 		return True
 
-	def __and__(self, other):
+	# I don't want to mimic the complex signature of the superclass
+	# here, so let's shut mypy up.
+	def __and__(self, other: Set) -> Set:  # type: ignore
 		return other
 	
-	intersection = __and__
+	def intersection(self, other: Set) -> Set: # type: ignore
+		return other
 
-	def __contains__(self, el):
+	def __contains__(self, el: Any):
 		return True
 
 
-def iterDerivedClasses(baseClass, objects):
+def iterDerivedClasses(baseClass: type, objects: List[Any]):
 	"""iterates over all subclasses of baseClass in the sequence objects.
 	"""
 	for cand in objects:
@@ -354,7 +382,7 @@ def iterDerivedClasses(baseClass, objects):
 			pass
 
 
-def iterDerivedObjects(baseClass, objects):
+def iterDerivedObjects(baseClass: type, objects: List[Any]):
 	"""iterates over all instances of baseClass in the sequence objects.
 	"""
 	for cand in objects:
@@ -362,8 +390,17 @@ def iterDerivedObjects(baseClass, objects):
 			yield cand
 
 
-def buildClassResolver(baseClass, objects, instances=False,
-		key=lambda obj: getattr(obj, "name", None), default=None):
+# The API of buildClassResolver is a bit painful because it
+# can actually instantiate the classes. That's why there's an
+# Any in the signature; this would be a good place for overload
+# (or to separate the two functions).
+def buildClassResolver(
+		baseClass: type,
+		objects: List[Any],
+		instances: bool=False,
+		key: Callable[[Hashable], Optional[str]]
+			=lambda obj: getattr(obj, "name", None), default=None
+		) -> Callable[[str], Any]:
 	"""returns a function resolving classes deriving from baseClass
 	in the sequence objects by their names.
 
@@ -380,15 +417,16 @@ def buildClassResolver(baseClass, objects, instances=False,
 	will not be entered into the registry.
 	"""
 	if instances:
-		registry = algotricks.DeferringDict()
+		registry: Dict[str, Any] = algotricks.DeferringDict()
 	else:
 		registry = {}
+
 	for cls in iterDerivedClasses(baseClass, objects):
 		clsKey = key(cls)
 		if clsKey is not None:
 			registry[clsKey] = cls
 
-	def resolve(name, registry=registry):
+	def resolve(name:str, registry: Dict[str, Any]=registry):
 		try:
 			return registry[name]
 		except KeyError:
@@ -396,14 +434,15 @@ def buildClassResolver(baseClass, objects, instances=False,
 				return default
 			raise
 
-	resolve.registry = registry
+	# see mypy bug #2087
+	resolve.registry = registry  # type: ignore
 	return resolve
 
 
 _SILENCE_LOCK = threading.RLock()
 
 @contextlib.contextmanager
-def silence(errToo=False):
+def silence(errToo: bool=False) -> Generator:
 	"""a context manager to temporarily redirect stdout to /dev/null.
 
 	This is used to shut up some versions of pyparsing and pyfits that
@@ -430,7 +469,7 @@ def silence(errToo=False):
 
 
 @contextlib.contextmanager
-def in_dir(destDir):
+def in_dir(destDir: Filename) -> Generator:
 	"""executes the controlled block within destDir and then returns
 	to the previous directory.
 
@@ -445,7 +484,10 @@ def in_dir(destDir):
 
 
 @contextlib.contextmanager
-def sandbox(tmpdir=None, debug=False, extractfunc=None):
+def sandbox(
+		tmpdir: Optional[Filename]=None,
+		debug: Optional[bool]=False,
+		extractfunc: Optional[Callable[[Filename], None]]=None):
 	"""sets up and tears down a sandbox directory within tmpdir.
 
 	This is is a context manager.  The object returned is the original
@@ -481,44 +523,8 @@ def sandbox(tmpdir=None, debug=False, extractfunc=None):
 			shutil.rmtree(wd)
 
 
-def runInSandbox(setUp, func, tearDown, *args, **kwargs):
-	"""runs func in a temporary ("sandbox") directory.
-
-	func is called with args and kwargs.  setUp and tearDown are
-	two functions also called with args and kwargs; in addition, they
-	are passed the path of the tempdir (setUp) or the path of the
-	original directory (teardown) in the first argument.
-	
-	setUp is called after the directory has been created,
-	but the process is still in the current WD.
-	
-	tearDown is called before the temp dir is deleted and in this directory.
-	Its return value is the return value of runInSandbox, which is the
-	preferred way of getting data out of the sandbox.
-
-	If any of the handlers raise exceptions, the following handlers will not
-	be called.  The sandbox will be torn down, though.
-
-	This is only present for legacy code.  Use the sandbox context manager
-	now.
-	"""
-	owd = os.getcwd()
-	# within DaCHS, this should be within tempDir, but we don't bother to
-	# get access to DaCHS' config.  So there.
-	wd = tempfile.mkdtemp("sandbox")
-	try:
-		if setUp:
-			setUp(wd, *args, **kwargs)
-		os.chdir(wd)
-		func(*args, **kwargs)
-		result = tearDown(owd, *args, **kwargs)
-	finally:
-		os.chdir(owd)
-		shutil.rmtree(wd)
-	return result
-
-
-def _instrumentForDebugging(func, funcName, src):
+def _instrumentForDebugging(func: Callable, funcName: str, src: str
+		) -> Callable:
 	"""returns a callable for a callable func with instrumentation for
 	debugging.
 
@@ -527,7 +533,7 @@ def _instrumentForDebugging(func, funcName, src):
 	Specifically, when there's an exception when calling func, this will
 	be logged as a warning.
 	"""
-	debugLocals = {}
+	debugLocals: Dict[str, Any] = {}
 	embSrc = "\n".join([
 		"from gavo.utils import excs",
 		"def compileFunctionDebugWrapper(*args, **kwargs):",
@@ -545,11 +551,13 @@ def _instrumentForDebugging(func, funcName, src):
 	return debugLocals["compileFunctionDebugWrapper"]
 
 
-def compileFunction(src,
-		funcName,
-		useGlobals=None,
-		debug=False,
-		uniqueName=None):
+# What with function attributes, wild lambdas and the linecache,
+# it's nigh impossible to satisfy mypy on this:
+def compileFunction(src: str,
+		funcName: str,
+		useGlobals: Optional[Dict[str, Any]]=None,
+		debug: bool=False,
+		uniqueName: Optional[str]=None):
 	"""runs src through exec and returns the funcName from the resulting
 	namespace.
 
@@ -576,36 +584,37 @@ def compileFunction(src,
 	abc
 	"""
 	if not hasattr(compileFunction, "autoNumber"):
-		compileFunction.autoNumber = 0
+		compileFunction.autoNumber = 0 # type: ignore
 
 	doCleanup = False
 	if uniqueName is None:
-		uniqueName = "<generated code %s>"%compileFunction.autoNumber
-		compileFunction.autoNumber += 1
+		uniqueName = "<generated code %s>"%compileFunction.autoNumber # type: ignore
+		compileFunction.autoNumber += 1 # type: ignore
 		doCleanup = True
 	src = src+"\n"
 
-	locals = {}
+	localVars: Dict[str, Any] = {}
 	if useGlobals is None:
 		useGlobals = globals()
 
 	try:
 		code = compile(src, uniqueName, 'exec')
-		exec(code, useGlobals, locals)
+		exec(code, useGlobals, localVars)
 	except Exception as ex:
 		misctricks.sendUIEvent("Warning", "The code that failed to compile was:"
 			"\n%s"%src)
 		raise misctricks.logOldExc(excs.BadCode(src, "function", ex))
-	func = locals[funcName]
+	func = localVars[funcName]
 
 	# this makes our compiled lines available to the traceback writer.
 	# we might want to do sys.excepthook = traceback.print_exception
 	# somewhere so the post mortem dumper uses this, too.  Let's see
 	# if it's worth the added risk of breaking things.
-	linecache.cache[uniqueName] = len(src), None, src.split("\n"), uniqueName
+	linecache.cache[uniqueName] = (  # type: ignore
+		len(src), None, src.split("\n"), uniqueName)
 	if doCleanup:
 		func._cleanup = weakref.ref(func,
-			lambda _, key=uniqueName: linecache and linecache.cache.pop(key, None))
+			lambda _, key=uniqueName: linecache and linecache.cache.pop(key, None))  #type: ignore
 
 	if debug:
 		return _instrumentForDebugging(func, funcName, src)
@@ -613,7 +622,7 @@ def compileFunction(src,
 		return func
 
 
-def ensureExpression(expr, errName="unknown"):
+def ensureExpression(expr: str, errName: str="unknown") -> None:
 	"""raises a LiteralParserError if expr is not a parseable python expression.
 
 	>>> ensureExpression("4+4")
@@ -636,7 +645,7 @@ def ensureExpression(expr, errName="unknown"):
 		raise misctricks.logOldExc(excs.BadCode(expr, "expression", ex))
 
 
-def importModule(modName):
+def importModule(modName: str) -> ModuleType:
 	"""imports a module from the module path.
 
 	Use this to programmatically import "normal" modules, e.g., dc-internal
@@ -653,7 +662,7 @@ def importModule(modName):
 	if not re.match(r"([A-Za-z_]+)(\.[A-Za-z_]+)*", modName):
 		raise excs.Error("Invalid name in internal import: %s"%modName)
 	parts = modName.split(".")
-	vars = {}
+	vars: Dict[str, Any] = {}
 	if len(parts)==1:
 		exec("import %s"%modName, vars)
 	else:
@@ -662,7 +671,8 @@ def importModule(modName):
 
 
 @document
-def loadPythonModule(fqName, relativeTo=None):
+def loadPythonModule(fqName: Filename, relativeTo: Optional[Filename]=None
+		) -> Tuple[ModuleType, Any]:
 	"""imports fqName and returns the (module, spec).
 
 	Do not use this function to import DC-internal modules; this may
@@ -704,12 +714,12 @@ def loadPythonModule(fqName, relativeTo=None):
 			' of syntax errors or similar.  Try python -c "import mod" to get'
 			' a clue in that case.')
 
-	spec.loader.exec_module(module)
+	spec.loader.exec_module(module)  # type: ignore
 	
 	return module, spec
 
 
-def loadInternalObject(relativeName, objectName):
+def loadInternalObject(relativeName: str, objectName: str) -> Any:
 	"""gets a name from an internal module.
 
 	relativeName is the python module path (not including "gavo."),
@@ -722,7 +732,7 @@ def loadInternalObject(relativeName, objectName):
 	return getattr(module, objectName)
 
 
-def memoized(origFun):
+def memoized(origFun: Callable):
 	"""a trivial memoizing decorator.
 
 	This is a legacy wrapper for functools.lru_cache.  Don't use in new
@@ -731,7 +741,7 @@ def memoized(origFun):
 	return functools.lru_cache()(origFun)
 
 
-def iterConsecutivePairs(sequence):
+def iterConsecutivePairs(sequence: Sequence):
 	"""yields pairs of consecutive items from sequence.
 
 	If the last item cannot be paired, it is dropped.
@@ -748,8 +758,10 @@ def iterConsecutivePairs(sequence):
 		itertools.islice(iter2, None, None, 2)))
 
 
-def iterRanges(separators):
-	"""yields (left, right) pairs for a sequence of separators.
+def iterRanges(separators: Sequence[T]) -> Generator[Tuple[T, T], None, None]:
+	"""yields (left, right) pairs for a sequence of separating indexes.
+
+	This is when you want to partition a sequence based on cut points.
 
 	>>> list(iterRanges(range(6)))
 	[(0, 1), (1, 2), (2, 3), (3, 4), (4, 5)]
@@ -761,13 +773,13 @@ def iterRanges(separators):
 		left = right
 
 
-def identity(x):
+def identity(x: Any) -> Any:
 	return x
 
 
-def intToFunnyWord(anInt, translation=bytes.maketrans(
+def intToFunnyWord(anInt: int, translation=bytes.maketrans(
 		b"-0123456789abcdef",
-		b"zaeiousmnthwblpgd")):
+		b"zaeiousmnthwblpgd")) -> str:
 	"""returns a sometimes funny (but unique) byte string from an arbitrary
 	integer.
 
@@ -777,7 +789,7 @@ def intToFunnyWord(anInt, translation=bytes.maketrans(
 	return (b"%x"%anInt).translate(translation).decode("ascii")
 
 
-def addDefaults(dataDict, defaultDict):
+def addDefaults(dataDict: Dict, defaultDict: Dict) -> None:
 	"""adds key-value pairs from defaultDict to dataDict if the key is missing
 	in dataDict.
 	"""
@@ -787,25 +799,32 @@ def addDefaults(dataDict, defaultDict):
 
 
 @functools.lru_cache(None)
-def devnull():
+def devnull() -> TextIO:
 	"""returns a (string-) writable /dev/null.
 
 	This always returns the same object, and to placate resource warnings,
-	the file will be closed before exiting the programme; the close method
+	the file will be closed before exiting the program; the close method
 	of the returned thing is a no-op.
 	"""
 	devnull = open("/dev/null", "w")
 	closeIt = devnull.close
-	devnull.close = lambda: None
+	devnull.close = lambda: None   # type: ignore
 	atexit.register(closeIt)
 	return devnull
 
 
-def memoizeOn(onObject, generatingObject, generatingFunction, *args):
-	"""memoizes the result of generatingFunction on onObject.
+def memoizeOn(
+		onObject: object,
+		generatingObject: object,
+		generatingFunction: Callable,
+		*args) -> Any:
+	"""memoizes the result of generatingFunction on onObject and returns it.
 
 	This is for caching things that adapt to onObjects; see procdefs
 	and rowmakers for examples why this is useful.
+
+	args is passed to generatingFunction; generatingObject is only
+	needed for identification.
 	"""
 	cacheName = "_cache%s%s"%(generatingObject.__class__.__name__,
 		str(id(generatingObject)))
@@ -814,7 +833,7 @@ def memoizeOn(onObject, generatingObject, generatingFunction, *args):
 	return getattr(onObject, cacheName)
 
 
-def forgetMemoized(ob):
+def forgetMemoized(ob: object) -> None:
 	"""clears things memoizeOn-ed on ob or @utils.memoize-ed.
 
 	This is sometimes necessary to let the garbage collector free
@@ -835,35 +854,39 @@ def forgetMemoized(ob):
 			delattr(ob, n)
 
 
-def stealVar(varName):
+def stealVar(varName: str) -> Any:
 	"""returns the first local variable called varName in the frame stack
 	above my caller.
 
 	This is obviously abominable.  This is only used within the DC code where
 	the author deemed the specification ugly.  Ah.  Almost.
+
+	If no variable with varName is found anywhere in the stack, this
+	raises a ValueError.
 	"""
-	frame = inspect.currentframe().f_back.f_back
+# frame operations are essentially impossible to get through mypy right now.
+	frame = inspect.currentframe().f_back.f_back # type: ignore
 	while frame:
-		if varName in frame.f_locals:
-			return frame.f_locals[varName]
-		frame = frame.f_back
+		if varName in frame.f_locals: # type: ignore
+			return frame.f_locals[varName] # type: ignore
+		frame = frame.f_back # type: ignore
 	raise ValueError("No local %s in the stack"%varName)
 
 
-def printFrames():
+def printFrames() -> None:  # types: ignore
 	"""prints a compact list of frames.
 
 	This is an aid for printf debugging.
 	"""
-	frame = inspect.currentframe().f_back.f_back
-	if inspect.getframeinfo(frame)[2]=="getJobsTable":
+	frame = inspect.currentframe().f_back.f_back  #type: ignore
+	if inspect.getframeinfo(frame)[2]=="getJobsTable": # type: ignore
 		return
 	while frame:
 		print("[%s,%s], [%s]"%inspect.getframeinfo(frame)[:3])
 		frame = frame.f_back
 
 
-def getTracebackAsString():
+def getTracebackAsString() -> str:
 	import traceback
 	f = io.StringIO()
 	traceback.print_exc(file=f)
@@ -885,16 +908,20 @@ class EqualingRE(object):
 	>>> "ababa" == EqualingRE("(ab)+$")
 	False
 	"""
-	def __init__(self, pattern):
+	def __init__(self, pattern: str):
 		self.pat = re.compile(pattern)
 	
-	def __eq__(self, other):
-		return bool(self.pat.match(other))
+	def __eq__(self, other: Any) -> bool:
+		if isinstance(other, str):
+			return bool(self.pat.match(other))
+		elif isinstance(other, EqualingRE):
+			return id(self)==id(other)
+		return False
 
 	def __hash__(self):
 		return hash(self.pat)
 
-	def __ne__(self, other):
+	def __ne__(self, other: Any) -> bool:
 		return not self.__eq__(other)
 	
 	def __str__(self):
@@ -928,10 +955,10 @@ class NocaseString(str):
 	>>> NocaseString("axc")!=NocaseString("abc")
 	True
 	"""
-	def __eq__(self, other):
+	def __eq__(self, other: object):
 		if not hasattr(other, "lower"):
 			return False
-		return self.lower()==other.lower()
+		return self.lower()==other.lower()  # type: ignore
 
 
 # we define different list classes depending to help the sqlsupport
