@@ -10,42 +10,31 @@ it might be, so for now I'm making fitslock a no-op.
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
-
-# I'm wasting a lot of effort on handling gzipped FITS files, which is
-# something that's not terribly common in the end.  Maybe we should
-# cut the crap and let people with gzipped FITSes do their stuff manually?
-
-
+import builtins
 import datetime
-import gzip
 import itertools
-import os
 import re
 import struct
-import tempfile
 import threading
 import warnings
 from contextlib import contextmanager
 
+import numpy
+# temporary measure to shut up astropy's configuration parser.
+builtins._ASTROPY_SETUP_ = True  # type:ignore
+# the following is the original import of pyfits, and only it should be used
+from astropy.io import fits as pyfits # type:ignore
+from astropy.utils.exceptions import AstropyDeprecationWarning # type:ignore
+warnings.filterwarnings('ignore', category=AstropyDeprecationWarning)
+
 from gavo.utils import codetricks
 from gavo.utils import excs
-from gavo.utils import misctricks
 from gavo.utils import ostricks
 
-try:
-	import numpy
-	import builtins
-	# temporary measure to shut up astropy's configuration parser.
-	builtins._ASTROPY_SETUP_ = True
-	# the following is the original import of pyfits, and only it should be used
-	from astropy.io import fits as pyfits
-	from astropy.utils.exceptions import AstropyDeprecationWarning
-	warnings.filterwarnings('ignore', category=AstropyDeprecationWarning)
-except ImportError:
-	# pyfits is not installed; don't die, since the rest of gavo.utils
-	# will still work.
-	pyfits = misctricks.NotInstalledModuleStub( #noflake: exported name
-		"astropy and/or numpy")
+from gavo.utils.dachstypes import (Any, BinaryIO, Callable, Dict, Filename,
+	Generator, List, NDArray, Optional, StrOrBytes, StrToStrMap, Tuple)
+
+
 
 _FITS_TABLE_LOCK = threading.RLock()
 
@@ -77,7 +66,7 @@ try:
 except AttributeError: # pyfits too old to produce these: Good.
 	pass
 
-def padCard(input, length=CARD_SIZE):
+def padCard(input: StrOrBytes, length: int=CARD_SIZE) -> bytes:
 	"""pads input (a string or bytes) with blanks until len(result)%80=0
 
 	The length keyword argument lets you adjust the "card size".  Use
@@ -105,7 +94,7 @@ def padCard(input, length=CARD_SIZE):
 	return input+b' '*(length-l%length)
 
 
-def readHeaderBytes(f, maxHeaderBlocks=80):
+def readHeaderBytes(f: BinaryIO, maxHeaderBlocks: int=80) -> bytes:
 	"""returns the bytes belonging to a FITS header starting at the current
 	position within the file f.
 
@@ -131,7 +120,8 @@ def readHeaderBytes(f, maxHeaderBlocks=80):
 	return b"".join(parts)
 
 
-def readPrimaryHeaderQuick(f, maxHeaderBlocks=80):
+def readPrimaryHeaderQuick(f: BinaryIO, maxHeaderBlocks: int=80
+		) -> pyfits.Header:
 	"""returns a pyfits header for the primary hdu of the opened file f.
 
 	f must be opened in binary mode.
@@ -147,7 +137,7 @@ def readPrimaryHeaderQuick(f, maxHeaderBlocks=80):
 		readHeaderBytes(f, maxHeaderBlocks).decode("ascii", "ignore"))
 
 
-def parseCards(aString):
+def parseCards(aString: StrOrBytes) -> pyfits.Header:
 	"""returns a list of pyfits Cards parsed from aString.
 
 	This will raise a ValueError if aString's length is not divisible by 80.
@@ -173,7 +163,7 @@ def parseCards(aString):
 	return cards
 		
 
-def serializeHeader(hdr):
+def serializeHeader(hdr: pyfits.Header) -> bytes:
 	"""returns the FITS serialization of a FITS header hdr.
 
 	This is always bytes.
@@ -187,7 +177,11 @@ def serializeHeader(hdr):
 	return padCard(serForm, length=FITS_BLOCK_SIZE)
 
 
-def replacePrimaryHeader(inputFile, newHeader, targetFile, bufSize=100000):
+def replacePrimaryHeader(
+		inputFile: BinaryIO,
+		newHeader: pyfits.Header,
+		targetFile: BinaryIO,
+		bufSize: int=100000) -> None:
 	"""writes a FITS to targetFile having newHeader as the primary header,
 	where the rest of the data is taken from inputFile.
 
@@ -206,7 +200,8 @@ def replacePrimaryHeader(inputFile, newHeader, targetFile, bufSize=100000):
 		targetFile.write(buf)
 
 
-def replacePrimaryHeaderInPlace(fitsName, newHeader):
+def replacePrimaryHeaderInPlace(
+		fitsName: Filename, newHeader: pyfits.Header) -> None:
 	"""replaces the primary header of fitsName with newHeader.
 
 	Doing this, it tries to minimize the amount of writing necessary; if
@@ -263,7 +258,7 @@ STANDARD_CARD_SEQUENCE = [
 	("BSCALE", False),
 ]
 
-def _iterForcedPairs(seq):
+def _iterForcedPairs(seq: List) -> Generator:
 	"""helps _enforceHeaderConstraints.
 	"""
 	if seq is None:
@@ -275,7 +270,9 @@ def _iterForcedPairs(seq):
 			yield (item, False)
 
 
-def _enforceHeaderConstraints(cardList, cardSequence):
+def _enforceHeaderConstraints(
+		cardList: List[pyfits.Card],
+		cardSequence: Optional[List[Tuple[str, bool]]]):
 	"""returns a pyfits header containing the cards in cardList with FITS
 	sequence constraints satisfied.
 
@@ -289,11 +286,12 @@ def _enforceHeaderConstraints(cardList, cardSequence):
 	"""
 # I can't use pyfits.verify for this since cardList may not refer to
 # a data set that's actually in memory
-	cardsAdded, newCards = set(), []
+	cardsAdded = set()
+	newCards: List[pyfits.Card] = []
 	cardDict = dict((card.keyword, card) for card in cardList)
 
 	for kw, mandatory in itertools.chain(
-			STANDARD_CARD_SEQUENCE,  _iterForcedPairs(cardSequence)):
+			STANDARD_CARD_SEQUENCE,  _iterForcedPairs(cardSequence or [])):
 		if isinstance(kw, pyfits.Card):
 			newCards.append(kw)
 			continue
@@ -309,14 +307,16 @@ def _enforceHeaderConstraints(cardList, cardSequence):
 				raise FITSError("Mandatory card '%s' missing"%kw)
 
 	for card in cardList:  # use cardList rather than cardDict to maintain
-		                     # cardList order
+		                           # cardList order
 		if card.keyword not in cardsAdded or card.keyword=='':
 			newCards.append(card)
 	return pyfits.Header(newCards)
 
 
-def sortHeaders(header, commentFilter=None, historyFilter=None,
-		cardSequence=None):
+def sortHeaders(header: pyfits.Header,
+		commentFilter: Optional[Callable[[str], bool]]=None,
+		historyFilter: Optional[Callable[[str], bool]]=None,
+		cardSequence: Optional[List[Tuple[str, bool]]]=None) -> pyfits.Header:
 	"""returns a pyfits header with "real" cards first, then history, then
 	comment cards.
 
@@ -377,142 +377,35 @@ def sortHeaders(header, commentFilter=None, historyFilter=None,
 	return _enforceHeaderConstraints(newCards, cardSequence)
 
 
-def openGz(fitsName, tempDir=None):
-	"""returns the hdus for the gzipped fits fitsName.
+def openFits(fitsName: Filename) -> pyfits.HDUList:
+	"""returns the hdus for fName.
 
-	Scrap that as soon as we have gzipped fits support (i.e. newer pyfits)
-	in debian.
+	This is simulating legacy pyfits behaviour in that it will use
+	memmap I/O if it thinks that works.  You'll probably want to use
+	astropy directly these days if this gives you a headache.
 	"""
-	handle, pathname = tempfile.mkstemp(suffix="fits", dir=tempDir)
-	f = os.fdopen(handle, "w")
-	f.write(gzip.open(fitsName).read())
-	f.close()
-	hdus = pyfits.open(pathname)
-	hdus.readall()
-	os.unlink(pathname)
+	hdus = pyfits.open(fitsName, memmap=True)
+	# memmapping will bomb later if data needs to be scaled, so we
+	# guess now if that's going to happen
+	if (hdus[0].header.get("BZERO", 0)!=0
+			or hdus[0].header.get("BSCALE", 1)!=1
+			or "BLANK" in hdus[0].header):
+		hdus.close()
+		hdus = pyfits.open(fitsName, memmap=False)
 	return hdus
 
 
-def writeGz(hdus, fitsName, compressLevel=5, mode=0o664):
-	"""writes and gzips hdus into fitsName.  As a side effect, hdus will be
-	closed.
-
-	Apparently, not even recent versions of pyfits support writing of
-	zipped files (which is a bit tricky, admittedly).  So, we'll probably
-	have to live with this kludge for a while.
-	"""
-	handle, pathname = tempfile.mkstemp(suffix="fits")
-	with codetricks.silence():
-		hdus.writeto(pathname, overwrite=True)
-	os.close(handle)
-	rawFitsData = open(pathname, "rb").read()
-	os.unlink(pathname)
-	handle, pathname = tempfile.mkstemp(suffix="tmp",
-		dir=os.path.dirname(fitsName))
-	os.close(handle)
-	dest = gzip.open(pathname, "w", compressLevel)
-	dest.write(rawFitsData)
-	dest.close()
-	os.rename(pathname, fitsName)
-	os.chmod(fitsName, mode)
-
-
-def openFits(fitsName):
-		"""returns the hdus for fName.
-
-		This is simulating legacy pyfits behaviour in that it will use
-		memmap I/O if it thinks that works.  You'll probably want to use
-		astropy directly these days if this gives you a headache.
-		"""
-		hdus = pyfits.open(fitsName, memmap=True)
-		# memmapping will bomb later if data needs to be scaled, so we
-		# guess now if that's going to happen
-		if (hdus[0].header.get("BZERO", 0)!=0
-				or hdus[0].header.get("BSCALE", 1)!=1
-				or "BLANK" in hdus[0].header):
-			hdus = pyfits.open(fitsName, memmap=False)
-		return hdus
-
-
-class PlainHeaderManipulator:
-	"""A class that allows header manipulation of fits files
-	without having to touch the data.
-
-	This class exists because pyfits insists on scaling scaled image data
-	on reading it.  While we can scale back on writing, this is something
-	I'd rather not do.  So, I have this base class to facilate the
-	HeaderManipulator that can handle gzipped fits files as well.
-	"""
-	def __init__(self, fName):
-		self.hdus = pyfits.open(fName, "update")
-		self.add_comment = self.hdus[0].header.add_comment
-		self.add_history = self.hdus[0].header.add_history
-		self.add_blank = self.hdus[0].header.add_blank
-		self.update = self.hdus[0].header.set
-		self.set = self.hdus[0].header.set
-	
-	def updateFromList(self, kvcList):
-		for key, value, comment in kvcList:
-			self.hdus[0].header.set(key, value, comment=comment)
-
-	def close(self):
-		self.hdus.close()
-
-
-class GzHeaderManipulator(PlainHeaderManipulator):
-	"""is a class that allows header manipulation of fits files without
-	having to touch the data even for gzipped files.
-
-	See PlainHeaderManipulator.  We only provide a decoration here that
-	transparently gzips and ungzips compressed fits files.
-	"""
-	def __init__(self, fName, compressLevel=5):
-		self.origFile = fName
-		handle, self.uncompressedName = tempfile.mkstemp(
-			suffix="fits")
-		destFile = os.fdopen(handle, "w")
-		destFile.write(gzip.open(fName).read())
-		destFile.close()
-		self.compressLevel = compressLevel
-		PlainHeaderManipulator.__init__(self, self.uncompressedName)
-	
-	def close(self):
-		PlainHeaderManipulator.close(self)
-		destFile = gzip.open(self.origFile, "w", compresslevel=self.compressLevel)
-		with open(open(self.uncompressedName, "rb")) as f:
-			destFile.write(f.read())
-		destFile.close()
-		os.unlink(self.uncompressedName)
-
-
-def HeaderManipulator(fName):
-	"""returns a header manipulator for a FITS file.
-
-	(it's supposed to look like a class, hence the uppercase name)
-	It should automatically handle gzipped files.
-	"""
-	if fName.lower().endswith(".gz"):
-		return GzHeaderManipulator(fName)
-	else:
-		return PlainHeaderManipulator(fName)
-
-
-def getPrimarySize(fName):
-	"""returns x and y size a fits image.
-	"""
-	with open(fName, "rb") as f:
-		hdr = readPrimaryHeaderQuick(f)
-	return hdr["NAXIS1"], hdr["NAXIS2"]
-
-
-def getAxisLengths(hdr):
+def getAxisLengths(hdr: pyfits.Header) -> List[int]:
 	"""returns a sequence of the lengths of the axes of a FITS image
 	described by hdr.
 	"""
 	return [hdr["NAXIS%d"%i] for i in range(1, hdr["NAXIS"]+1)]
 
 
-def cutoutHeader(header, *cuts):
+_FitsCuts = Tuple[Optional[float], ...]
+
+def cutoutHeader(header: pyfits.Header, *cuts: _FitsCuts
+		) -> Tuple[List, pyfits.Header]:
 	"""returns a array slices and a new pyfits header reflecting cuts.
 
 	(see cutoutFits for what cuts is).
@@ -548,7 +441,7 @@ def cutoutHeader(header, *cuts):
 	return slices, newHeader
 
 
-def cutoutFITS(hdu, *cuts):
+def cutoutFITS(hdu: pyfits.ImageHDU, *cuts: _FitsCuts) -> pyfits.ImageHDU:
 	"""returns a cutout of hdu restricted to cuts.
 
 	hdu is a primary FITS hdu.  cuts is a list of cut specs, each of which is
@@ -585,7 +478,7 @@ def cutoutFITS(hdu, *cuts):
 	return newHDU
 
 
-def shrinkWCSHeader(oldHeader, factor):
+def shrinkWCSHeader(oldHeader: pyfits.Header, factor: float) -> pyfits.Header:
 	"""returns a FITS header suitable for a shrunken version of the image
 	described by oldHeader.
 
@@ -638,7 +531,7 @@ NUM_CODE = {
 		-32: '>f4',
 		-64: '>f8'}
 
-def _makeDecoder(hdr):
+def _makeDecoder(hdr: pyfits.Header) -> Callable[[BinaryIO], NDArray]:
 	"""returns a decoder for the rows of FITS primary image data.
 
 	The decoder is called with an open file and returns the next row.
@@ -650,18 +543,19 @@ def _makeDecoder(hdr):
 
 	bzero, bscale = hdr.get("BZERO", 0), hdr.get("BSCALE", 1)
 	if bzero!=0 or bscale!=1:
-		def read(f):
+		def read(f: BinaryIO) -> NDArray:
 			return numpy.asarray(
 				numpy.frombuffer(f.read(nBytes), numType, rowLength),
 					'float32')*bscale+bzero
 	else:
-		def read(f):  #noflake: conditional definition
+		def read(f: BinaryIO) -> NDArray:  #noflake: conditional definition
 			return numpy.frombuffer(f.read(nBytes), numType, rowLength)
 
 	return read
 
 
-def iterFITSRows(hdr, f):
+def iterFITSRows(hdr: pyfits.Header, f: BinaryIO
+		) -> Generator[NDArray, None, None]:
 	"""iterates over the rows of a FITS (primary) image.
 
 	You pass in a FITS header and a file positioned to the start of
@@ -680,7 +574,8 @@ def iterFITSRows(hdr, f):
 		yield decoder(f)
 
 
-def _iterSetupFast(inFile, hdr):
+def _iterSetupFast(inFile: BinaryIO, hdr: pyfits.Header
+		) -> Tuple[pyfits.Header, Generator[NDArray, None, None]]:
 	"""helps iterScaledRows for the case of a simple, real-file FITS.
 	"""
 	if hdr is None:
@@ -691,7 +586,7 @@ def _iterSetupFast(inFile, hdr):
 	return hdr, iterFITSRows(hdr, inFile)
 
 
-def fixImageExtind(hdus, extInd):
+def fixImageExtind(hdus: List, extInd: int) -> int:
 	"""returns the index of a compressed image HDU if it immediately
 	follows extInd.
 
@@ -709,7 +604,9 @@ def fixImageExtind(hdus, extInd):
 	return extInd
 
 
-def _iterSetupCompatible(inFile, hdr, extInd=0):
+def _iterSetupCompatible(
+		inFile: BinaryIO, hdr: pyfits.Header, extInd: int=0
+	) -> Tuple[pyfits.Header, Generator[NDArray, None, None]]:
 	"""helps iterScaledRows for when _iterSetupFast will not work.
 
 	Using extInd, you can select a different extension.  extInd=0
@@ -726,8 +623,13 @@ def _iterSetupCompatible(inFile, hdr, extInd=0):
 	return hdus[extInd].header, iterRows()
 
 
-def iterScaledRows(inFile, factor=None, destSize=None, hdr=None, slow=False,
-		extInd=0):
+def iterScaledRows(
+		inFile: BinaryIO,
+		factor: Optional[float]=None,
+		destSize: Optional[int]=None,
+		hdr: Optional[pyfits.Header]=None,
+		slow: bool=False,
+		extInd: int=0) -> Generator[NDArray, None, None]:
 	"""iterates over numpy arrays of pixel rows within the open FITS
 	stream inFile scaled by it integer in factor.
 
@@ -782,11 +684,15 @@ def iterScaledRows(inFile, factor=None, destSize=None, hdr=None, slow=False,
 		# horizontal scaling via reshaping to a matrix and then summing over
 		# its columns...
 		newRow = newRow[:destRowLength*factor]
-		yield sum(numpy.transpose(
+		# mypy 1.0.1 type inference bad
+		yield sum(numpy.transpose(  # type: ignore
 			(newRow/factor).reshape((destRowLength, factor))))
 
 
-def iterScaledBytes(inFileName, factor, extraCards={}):
+def iterScaledBytes(
+		inFileName: Filename,
+		factor: float,
+		extraCards: StrToStrMap={}) -> Generator[bytes, None, None]:
 	"""iterates over the bytes for a simple FITS file generated by scaling
 	down the 2D image inFileName by factor.
 
@@ -809,7 +715,7 @@ def iterScaledBytes(inFileName, factor, extraCards={}):
 				yield struct.pack("!%df"%len(row), *row)
 
 
-def headerFromDict(d):
+def headerFromDict(d: StrToStrMap) -> pyfits.Header:
 	"""returns a primary header sporting the key/value pairs given in the
 	dictionary d.
 
@@ -848,42 +754,48 @@ class WCSAxis(object):
 	To retrieve the metadata shoved in, use the name, crval, crpix, cdelt,
 	ctype, cunit, and axisLength attributes.
 	"""
-	def __init__(self, name, crval, crpix, cdelt,
-			ctype="UNKNOWN", cunit="", axisLength=1):
+	def __init__(self,
+			name: str,
+			crval: float,
+			crpix: float,
+			cdelt: float,
+			ctype: str="UNKNOWN",
+			cunit: Optional[str]="",
+			axisLength: int=1):
 		assert cdelt!=0
 		self.crval, self.crpix, self.cdelt = crval, crpix, cdelt
 		self.ctype, self.cunit, self.axisLength = ctype, cunit, axisLength
 		self.name = name
 
-	def pixToPhys(self, pixCoo):
+	def pixToPhys(self, pixCoo: float) -> float:
 		"""returns the physical value for a 1-based pixel coordinate.
 		"""
 		if pixCoo is None:
 			return None
 		return self.crval+(pixCoo-self.crpix)*self.cdelt
 
-	def pix0ToPhys(self, pix0Coo):
+	def pix0ToPhys(self, pix0Coo: float) -> float:
 		"""returns the physical value for a 0-based pixel coordinate.
 		"""
 		if pix0Coo is None:
 			return None
 		return self.pixToPhys(pix0Coo+1)
 	
-	def physToPix(self, physCoo):
+	def physToPix(self, physCoo: float) -> float:
 		"""returns a 1-based pixel coordinate for a physical value.
 		"""
 		if physCoo is None:
 			return None
 		return (physCoo-self.crval)/self.cdelt+self.crpix
 	
-	def physToPix0(self, physCoo):
+	def physToPix0(self, physCoo: float) -> float:
 		"""returns a 0-based pixel coordinate for a physical value.
 		"""
 		if physCoo is None:
 			return None
 		return self.physToPix(physCoo)-1
 
-	def getLimits(self):
+	def getLimits(self) -> Tuple[float, float]:
 		"""returns the minimal and maximal physical values this axis
 		takes within the image.
 		"""
@@ -891,7 +803,10 @@ class WCSAxis(object):
 		return min(limits), max(limits)
 
 	@classmethod
-	def fromHeader(cls, header, axisIndex, forceSeparable=False):
+	def fromHeader(cls,
+			header: pyfits.Header,
+			axisIndex: int,
+			forceSeparable: bool=False) -> 'WCSAxis':
 		"""returns a WCSAxis for the specified axis in header.
 
 		If the axis is mentioned in a transformation matrix (CD or PC),
@@ -926,7 +841,10 @@ class WCSAxis(object):
 
 # WCSAxis.fromHeader wrapper for rmkfuncs
 @codetricks.document
-def getWCSAxis(header, axisIndex, forceSeparable=False):
+def getWCSAxis(
+		header: pyfits.Header,
+		axisIndex: int,
+		forceSeparable: bool=False) -> WCSAxis:
 	"""returns a WCSAxis instance from an axis index and a FITS header.
 
 	If the axis is mentioned in a transformation matrix (CD or PC),
@@ -949,11 +867,39 @@ class ESODescriptorsError(excs.SourceParseError):
 	"""
 
 
-def _makeNamed(name, re):
+def _makeNamed(name: str, re: str) -> str:
+# A helper to construct REs for the ESO parser.
 	return "(?P<%s>%s)"%(name, re)
 
 
-class _ESODescriptorsParser(object):
+def _makeTypeScanner(
+		typeName: str,
+		literalRE: re.Pattern,
+		constructor: Callable[[str], Any]):
+	"""returns a scanner method for the _ESODescriptorsParser.
+
+	This is a helper method for building the class; typeName
+	must be the same as the name in _scan_name.  constructor is
+	a function that turns string literals into objects of the desired
+	type.
+	"""
+	def scanner(self) -> str:
+		mat = literalRE.match(self.data, self.curPos)
+		if not mat:
+			raise ValueError("Expected a %s here"%typeName)
+		self.curPos = mat.end()
+
+		self.curCol.append(constructor(mat.group()))
+		self.yetToRead -= 1
+		if self.yetToRead==0:
+			return "harvest"
+		else:
+			return typeName
+
+	return scanner
+
+
+class _ESODescriptorsParser:
 	"""an ad-hoc parser for ESO's descriptors.
 
 	These are sometimes in FITS files produced by MIDAS.  What I'm pretending
@@ -985,9 +931,10 @@ class _ESODescriptorsParser(object):
 	floatRE = re.compile(nextToken+floatPat)
 	integerRE = re.compile(nextToken+intPat)
 
-	def __init__(self, data):
+	def __init__(self, data: str):
 		self.data, state = data, "header"
-		self.result, self.curPos = {}, 0
+		self.result: Dict[str, List[float]] = {}
+		self.curPos:int = 0
 		while state!="end":
 			try:
 				state = getattr(self, "_scan_"+state)()
@@ -997,7 +944,7 @@ class _ESODescriptorsParser(object):
 					offending=repr(self.data[self.curPos:self.curPos+20]))
 
 	@classmethod
-	def fromFITSHeader(cls, hdr):
+	def fromFITSHeader(cls, hdr: pyfits.Header) -> '_ESODescriptorsParser':
 		"""returns a parser from the data in the pyfits hdr.
 		"""
 		descLines, collecting = [], False
@@ -1013,8 +960,8 @@ class _ESODescriptorsParser(object):
 
 		return cls("\n".join(descLines))
 
-	def _scan_header(self):
-		"""read the next descriptor header.
+	def _scan_header(self) -> str:
+		"""reads the next descriptor header and returns a type name for it.
 		"""
 		mat = self.headerRE.match(self.data, self.curPos)
 		if not mat:
@@ -1024,7 +971,7 @@ class _ESODescriptorsParser(object):
 				raise ValueError("Could not find next header")
 		
 		self.curPos = mat.end()
-		self.curCol = []
+		self.curCol: List[float] = []
 		self.curColName = mat.group("colName")[1:-1]
 		self.yetToRead = int(mat.group("endInd"))-int(mat.group("startInd"))+1
 
@@ -1044,36 +991,13 @@ class _ESODescriptorsParser(object):
 		del self.yetToRead
 		return "header"
 
-	def _makeTypeScanner(typeName, literalRE, constructor):
-		"""returns a scanner function (invisible to the outside.
-
-		This is a helper method for building the parser class; typeName
-		must be the same as the name in _scan_name.  constructor is
-		a function that turns string literals into objects of the desired
-		type.
-		"""
-		def scanner(self):
-			mat = literalRE.match(self.data, self.curPos)
-			if not mat:
-				raise ValueError("Expected a %s here"%typeName)
-			self.curPos = mat.end()
-
-			self.curCol.append(constructor(mat.group()))
-			self.yetToRead -= 1
-			if self.yetToRead==0:
-				return "harvest"
-			else:
-				return typeName
-
-		return scanner
-
 	_scan_float = _makeTypeScanner("float", floatRE, float)
 	_scan_integer = _makeTypeScanner("integer", integerRE, int)
 	
-	del _makeTypeScanner
+del _makeTypeScanner
 
 
-def parseESODescriptors(hdr):
+def parseESODescriptors(hdr: pyfits.Header) -> Dict[str, List[float]]:
 	"""returns parsed ESO descriptors from a pyfits header hdr.
 
 	ESO descriptors are data columns stuck into FITS history lines.
