@@ -17,8 +17,10 @@ import io
 from PIL import Image
 import numpy
 
+from gavo.utils.dachstypes import (Filename, NDArray)
 
-def _normalizeForImage(pixels, gamma):
+
+def _normalizeForImage(pixels: NDArray, gamma: float) -> NDArray:
 	"""helps jpegFromNumpyArray and friends.
 	"""
 	pixels = numpy.flipud(pixels)
@@ -27,7 +29,7 @@ def _normalizeForImage(pixels, gamma):
 		(pixels-pixMin)/(pixMax-pixMin), gamma)*255, 'uint8')
 
 
-def jpegFromNumpyArray(pixels, gamma=0.25):
+def jpegFromNumpyArray(pixels: NDArray, gamma: float=0.25) -> bytes:
 	"""returns a normalized JPEG for numpy pixels.
 
 	pixels is assumed to come from FITS arrays, which are flipped wrt to
@@ -42,7 +44,9 @@ def jpegFromNumpyArray(pixels, gamma=0.25):
 	return f.getvalue()
 
 
-def colorJpegFromNumpyArrays(rPix, gPix, bPix, gamma=0.25):
+def colorJpegFromNumpyArrays(
+		rPix: NDArray, gPix: NDArray, bPix: NDArray, gamma: float=0.25
+		) -> bytes:
 	"""as jpegFromNumpyArray, except a color jpeg is built from red, green,
 	and blue pixels.
 	"""
@@ -56,7 +60,7 @@ def colorJpegFromNumpyArrays(rPix, gPix, bPix, gamma=0.25):
 	return f.getvalue()
 
 
-def scaleNumpyArray(arr, destSize):
+def scaleNumpyArray(arr: NDArray, destSize: int) -> NDArray:
 	"""returns the numpy array arr scaled down to approximately destSize.
 	"""
 	origWidth, origHeight = arr.shape
@@ -81,8 +85,8 @@ def scaleNumpyArray(arr, destSize):
 	return img
 
 
-def getScaledPNG(srcFile, newWidth):
-	"""returns a PNG string that is a scaled version of an image in srcFile.
+def getScaledPNG(srcFile: Filename, newWidth: int) -> bytes:
+	"""returns PNG bytes that is a scaled version of an image in srcFile.
 
 	srcFile must correspond to something that PIL can read.  Since scaling
 	really sucks for non-RGB images, we unconditionally convert whatever we
@@ -90,7 +94,7 @@ def getScaledPNG(srcFile, newWidth):
 	"""
 	im = Image.open(srcFile).convert("RGB")
 	scale = newWidth/float(im.size[0])
-	scaled = im.resize((newWidth, int(im.size[1]*scale)), Image.ANTIALIAS)
+	scaled = im.resize((newWidth, int(im.size[1]*scale)), Image.LANCZOS)
 	f = io.BytesIO()
 	scaled.save(f, format="png")
 	return f.getvalue()
