@@ -7,7 +7,6 @@ Various helpers that didn't fit into any other xTricks.
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
-
 import base64
 import collections
 import functools
@@ -17,27 +16,26 @@ import re
 import struct
 import time
 import threading
-import urllib.request, urllib.error, urllib.parse
+import urllib.request as urlrequest
 import zlib
 
+from docutils import core as rstcore
+from docutils import nodes
+from docutils import utils as rstutils
+from docutils.parsers.rst import roles
+from docutils.parsers.rst import directives
+from docutils.parsers.rst.states import Inliner
+
 from gavo.utils import excs
 
-class NotInstalledModuleStub(object):
-	"""A stub that raises some more or less descriptive error on attribute
-	access.
-
-	This is used in some places no replace non-essential modules.
-	"""
-	def __init__(self, modName):
-		self.modName = modName
-
-	def __getattr__(self, name):
-		raise RuntimeError("%s not installed"%self.modName)
+from gavo.utils.dachstypes import (Any, BinaryIO, Callable, Dict,
+	Filename, Generator, Generic, List, Optional, Sequence, StrOrBytes,
+	StrToStrMap, Tuple, Type, TypeVar, Union, cast)
 
 
 BIBCODE_PATTERN = re.compile("[012]\d\d\d\w[^ ]{14}$")
 
-def couldBeABibcode(s):
+def couldBeABibcode(s: str) -> bool:
 	"""returns true if we think that the string s is a bibcode.
 
 	This is based on matching against BIBCODE_PATTERN.
@@ -45,138 +43,145 @@ def couldBeABibcode(s):
 	return bool(BIBCODE_PATTERN.match(s))
 
 
-try:
-	from docutils import core as rstcore
+_RSTRoleReturnType = Tuple[List[nodes.Node], List[str]]
+_RSTRoleFunction = Callable[[str, str, str, int, Inliner,
+	Optional[dict], Optional[List[str]]], _RSTRoleReturnType]
 
-	from docutils import nodes
-	from docutils import utils as rstutils
-	from docutils.parsers.rst import roles
-	from docutils.parsers.rst import directives
+class RSTExtensions:
+	"""a register for local RST extensions.
 
-	class RSTExtensions(object):
-		"""a register for local RST extensions.
+	This is for both directives and interpreted text roles.
 
-		This is for both directives and interpreted text roles.
+	We need these as additional markup in examples; these always
+	introduce local rst interpreted text roles, which always
+	add some class to the node in question (modifications are possible).
 
-		We need these as additional markup in examples; these always
-		introduce local rst interpreted text roles, which always
-		add some class to the node in question (modifications are possible).
+	These classes are then changed to properties as the HTML fragments
+	from RST translation are processed by the _Example nevow data factory.
 
-		These classes are then changed to properties as the HTML fragments
-		from RST translation are processed by the _Example nevow data factory.
+	To add a new text role, say::
 
-		To add a new text role, say::
+		RSTExtensions.addRole(roleName, roleFunc=None)
 
-			RSTExtensions.addRole(roleName, roleFunc=None)
+	You can pass in a full role function as discussed in
+	...docs/howto/rst-roles.html#define-the-role-function
+	It must, however, add a dachs-ex-<roleName> class to the node. The
+	default function produces a nodes.emphasis item with the proper class.
 
-		You can pass in a full role function as discussed in
-		/usr/share/doc/python-docutils/docs/howto/rst-roles.html (Debian systems).
-		It must, however, add a dachs-ex-<roleName> class to the node. The
-		default function produces a nodes.emphasis item with the proper class.
+	In a pinch, you can pass a propertyName argument to addRole if the
+	desired property name is distinct from the role name in the RST.
+	This is used by tapquery and taprole since we didn't want to change
+	our examples when the standard changed.
 
-		In a pinch, you can pass a propertyName argument to addRole if the
-		desired property name is distinct from the role name in the RST.
-		This is used by tapquery and taprole since we didn't want to change
-		our examples when the standard changed.
+	To add a directive, say::
 
-		To add a directive, say::
+		RSTExtensions.addDirective(dirName, dirClass)
 
-			RSTExtensions.addDirective(dirName, dirClass)
+	In HTML, these classes become properties named like the role name
+	(except you can again use propertyName in a pinch).
+	"""
+	classToProperty: Dict[str, str] = {}
 
-		In HTML, these classes become properties named like the role name
-		(except you can again use propertyName in a pinch).
+	@classmethod
+	def addDirective(cls,
+			name: str,
+			implementingClass: type,
+			propertyName: Optional[str]=None) -> None:
+		directives.register_directive(name, implementingClass)
+		cls.classToProperty["dachs-ex-"+name] = propertyName or name
+
+	@classmethod
+	def makeTextRole(cls,
+			roleName: str,
+			roleFunc: Optional[_RSTRoleFunction]=None, propertyName=None
+			) -> None:
+		"""creates a new text role for roleName.
+
+		See class docstring.
 		"""
-		classToProperty = {}
+		if roleFunc is None:
+			roleFunc = cls._makeDefaultRoleFunc(roleName)
+		roles.register_local_role(roleName, roleFunc)
+		cls.classToProperty["dachs-ex-"+roleName] = propertyName or roleName
+	
+	@classmethod
+	def _makeDefaultRoleFunc(cls, roleName: str) -> _RSTRoleFunction:
+		"""returns an RST interpreted text role parser function returning
+		an emphasis node with a dachs-ex-roleName class.
+		"""
+		def roleFunc(name, rawText, text, lineno, inliner,
+				options={}, content=[]):
+			node = nodes.emphasis(rawText, text)
+			node["classes"] = ["dachs-ex-"+roleName]
+			return [node], []
 
-		@classmethod
-		def addDirective(cls, name, implementingClass, propertyName=None):
-			directives.register_directive(name, implementingClass)
-			cls.classToProperty["dachs-ex-"+name] = propertyName or name
+		return roleFunc
 
-		@classmethod
-		def makeTextRole(cls, roleName, roleFunc=None, propertyName=None):
-			"""creates a new text role for roleName.
+# Generally useful RST extensions (for roles useful in examples,
+# see examplesrender)
+def _bibcodeRoleFunc(name: str, rawText: str, text: str, lineno: int,
+		inliner: Inliner,
+		options: Optional[dict]={}, content: Optional[List[str]]=[]
+		) -> _RSTRoleReturnType:
+	if not couldBeABibcode(text):
+		raise ValueError("Probably not a bibcode: '%s'"%text)
+	node = nodes.reference(rawText, text,
+		refuri="http://adsabs.harvard.edu/abs/%s"%text)
+	node["classes"] = ["bibcode-link"]
+	return [node], []
 
-			See class docstring.
-			"""
-			if roleFunc is None:
-				roleFunc = cls._makeDefaultRoleFunc(roleName)
-			roles.register_local_role(roleName, roleFunc)
-			cls.classToProperty["dachs-ex-"+roleName] = propertyName or roleName
-		
-		@classmethod
-		def _makeDefaultRoleFunc(cls, roleName):
-			"""returns an RST interpreted text role parser function returning
-			an emphasis node with a dachs-ex-roleName class.
-			"""
-			def roleFunc(name, rawText, text, lineno, inliner,
-					options={}, content=[]):
-				node = nodes.emphasis(rawText, text)
-				node["classes"] = ["dachs-ex-"+roleName]
-				return [node], []
+RSTExtensions.makeTextRole("bibcode", _bibcodeRoleFunc)
+del _bibcodeRoleFunc
 
-			return roleFunc
+# RST extensions for documentation writing
 
-	# Generally useful RST extensions (for roles useful in examples,
-	# see examplesrender)
-	def _bibcodeRoleFunc(name, rawText, text, lineno, inliner,
-			options={}, content=[]):
-		if not couldBeABibcode(text):
-			raise ValueError("Probably not a bibcode: '%s'"%text)
-		node = nodes.reference(rawText, text,
-			refuri="http://adsabs.harvard.edu/abs/%s"%text)
-		node["classes"] = ["bibcode-link"]
-		return [node], []
+_explicitTitleRE = re.compile(r'^(.+?)\s*(?<!\x00)<(.*?)>$', re.DOTALL)
 
-	RSTExtensions.makeTextRole("bibcode", _bibcodeRoleFunc)
-	del _bibcodeRoleFunc
+def _dachsdocRoleFunc(name: str, rawText: str, text: str, lineno: int,
+		inliner: Inliner,
+		options: Optional[dict]={}, content: Optional[List[str]]=[]
+		) -> _RSTRoleReturnType:
+	# inspired by sphinx extlinks
+	text = rstutils.unescape(text)
+	mat = _explicitTitleRE.match(text)
+	if mat:
+		title, url = mat.groups()
+	else:
+		title, url = text.split("/")[-1], text
+	url = "http://docs.g-vo.org/DaCHS/"+url
+	return [nodes.reference(title, title, internal=False, refuri=url)
+		], []
 
-	# RST extensions for documentation writing
+RSTExtensions.makeTextRole("dachsdoc", _dachsdocRoleFunc)
+del _dachsdocRoleFunc
 
-	_explicitTitleRE = re.compile(r'^(.+?)\s*(?<!\x00)<(.*?)>$', re.DOTALL)
+def _dachsrefRoleFunc(name: str, rawText: str, text: str, lineno: int,
+		inliner: Inliner,
+		options: Optional[dict]={}, content: Optional[List[str]]=[]
+		) -> _RSTRoleReturnType:
+	# this will guess a link into the ref documentation
+	text = rstutils.unescape(text)
+	fragId = re.sub("[^a-z0-9]+", "-", text.lower())
+	url = "http://docs.g-vo.org/DaCHS/ref.html#"+fragId
+	return [nodes.reference(text, text, internal=False, refuri=url)
+		], []
 
-	def _dachsdocRoleFunc(name, rawText, text, lineno, inliner,
-			options={}, content=[]):
-		# inspired by sphinx extlinks
-		text = rstutils.unescape(text)
-		mat = _explicitTitleRE.match(text)
-		if mat:
-			title, url = mat.groups()
-		else:
-			title, url = text.split("/")[-1], text
-		url = "http://docs.g-vo.org/DaCHS/"+url
-		return [nodes.reference(title, title, internal=False, refuri=url)
-			], []
+RSTExtensions.makeTextRole("dachsref", _dachsrefRoleFunc)
+del _dachsrefRoleFunc
 
-	RSTExtensions.makeTextRole("dachsdoc", _dachsdocRoleFunc)
-	del _dachsdocRoleFunc
+def _samplerdRoleFunc(name: str, rawText: str, text: str, lineno: int,
+		inliner: Inliner,
+		options: Optional[dict]={}, content: Optional[List[str]]=[]
+		) -> _RSTRoleReturnType:
+	# this will turn into a link to a file in the GAVO svn
+	# (usually for RDs)
+	text = rstutils.unescape(text)
+	url = "http://svn.ari.uni-heidelberg.de/svn/gavo/hdinputs/"+text+".rd"
+	return [nodes.reference(text, text, internal=False, refuri=url)
+		], []
 
-	def _dachsrefRoleFunc(name, rawText, text, lineno, inliner,
-			options={}, content=[]):
-		# this will guess a link into the ref documentation
-		text = rstutils.unescape(text)
-		fragId = re.sub("[^a-z0-9]+", "-", text.lower())
-		url = "http://docs.g-vo.org/DaCHS/ref.html#"+fragId
-		return [nodes.reference(text, text, internal=False, refuri=url)
-			], []
-
-	RSTExtensions.makeTextRole("dachsref", _dachsrefRoleFunc)
-	del _dachsrefRoleFunc
-
-	def _samplerdRoleFunc(name, rawText, text, lineno, inliner,
-			options={}, content=[]):
-		# this will turn into a link to a file in the GAVO svn
-		# (usually for RDs)
-		text = rstutils.unescape(text)
-		url = "http://svn.ari.uni-heidelberg.de/svn/gavo/hdinputs/"+text+".rd"
-		return [nodes.reference(text, text, internal=False, refuri=url)
-			], []
-
-	RSTExtensions.makeTextRole("samplerd", _samplerdRoleFunc)
-	del _samplerdRoleFunc
-
-except ImportError:
-	rstcore = NotInstalledModuleStub("docutils") #noflake: conditional import
+RSTExtensions.makeTextRole("samplerd", _samplerdRoleFunc)
+del _samplerdRoleFunc
 
 
 class _UndefinedType(type):
@@ -194,7 +199,7 @@ class _UndefinedType(type):
 		return False
 
 
-class Undefined(object, metaclass=_UndefinedType):
+class Undefined(metaclass=_UndefinedType):
 	"""a sentinel for all kinds of undefined values.
 
 	Do not instantiate.
@@ -227,11 +232,11 @@ class RateLimiter:
 	to revisit this implementation -- this is really for when rate limiting
 	is the exception.
 	"""
-	def __init__(self, timeout):
+	def __init__(self, timeout: float):
 		self.timeout = timeout
-		self.lastEvents = {}
+		self.lastEvents: Dict[str, float] = {}
 	
-	def inDeadtime(self, key):
+	def inDeadtime(self, key: str):
 		now = time.time()
 
 		# no reason to have this work in 1970
@@ -243,7 +248,7 @@ class RateLimiter:
 
 
 @functools.total_ordering
-class QuotedName(object):
+class QuotedName:
 	"""A string-like thing basically representing SQL delimited identifiers.
 
 	This has some features that make handling these relatively painless
@@ -269,13 +274,13 @@ class QuotedName(object):
 	>>> QuotedName('7oh-no"+rob').makeIdentifier()
 	'id7oh2dno222brob'
 	"""
-	def __init__(self, name):
+	def __init__(self, name: str):
 		self.name = name
 	
-	def __hash__(self):
+	def __hash__(self) -> int:
 		return hash(self.name)
 	
-	def __eq__(self, other):
+	def __eq__(self, other: Any) -> bool:
 		if isinstance(other, QuotedName):
 			return self.name==other.name
 		elif isinstance(other, str):
@@ -283,7 +288,7 @@ class QuotedName(object):
 		else:
 			return False
 
-	def __lt__(self, other):
+	def __lt__(self, other: Any) -> bool:
 		if isinstance(other, QuotedName):
 			return self.name<other.name
 		elif isinstance(other, str):
@@ -291,25 +296,25 @@ class QuotedName(object):
 		else:
 			return False
 
-	def __str__(self):
+	def __str__(self) -> str:
 		return '"%s"'%(self.name.replace('"', '""'))
 
-	def __repr__(self):
+	def __repr__(self) -> str:
 		return 'QuotedName(%s)'%repr(self.name)
 
-	def isRegularLower(self):
+	def isRegularLower(self) -> bool:
 		return not not re.match("[a-z][a-z0-9_]*$", self.name)
 
 	def lower(self):  # service to ADQL name resolution
 		return self
 
-	def flatten(self): # ADQL query serialization
+	def flatten(self) -> str: # ADQL query serialization
 		return str(self)
 
-	def capitalize(self):  # service for table head and such
+	def capitalize(self) -> str:  # service for table head and such
 		return self.name.capitalize()
 
-	def makeIdentifier(self):
+	def makeIdentifier(self) -> str:
 		"""returns self as something usable as a SQL regular identifier.
 
 		This will be rather unreadable if there's a substantial number
@@ -324,11 +329,13 @@ class QuotedName(object):
 			id = "id"+id
 		return id
 
-	def __add__(self, other):  # for disambiguateColumns
+	def __add__(self, other: str):  # for disambiguateColumns
 		return QuotedName(self.name+other)
 
 
-class StreamBuffer(object):
+_StreamData = TypeVar('_StreamData', bytes, str)
+
+class StreamBuffer(Generic[_StreamData]):
 	"""a buffer that takes data in arbitrary chunks and returns
 	them in chops of chunkSize bytes.
 
@@ -337,24 +344,24 @@ class StreamBuffer(object):
 
 	When everything is written, you must all doneWriting.
 	"""
-	# XXX TODO: Can we make a reasoned  choice for (default) chunkSize?
 	chunkSize = 50000
 
-	def __init__(self, chunkSize=None, binary=True):
-		self.buffer = collections.deque()
+	def __init__(self, chunkSize: Optional[int]=None, binary: bool=True):
+		self.buffer: collections.deque = collections.deque()
 		if chunkSize is not None:
 			self.chunkSize = chunkSize
 		self.curSize = 0
 		self.lock = threading.Lock()
 		self.finished = False
-		self.joiner = b"" if binary else ""
+		# annotation problem: understand how to make mypy grok this
+		self.joiner: _StreamData = b"" if binary else "" # type: ignore
 	
-	def add(self, data):
+	def add(self, data: _StreamData) -> None:
 		with self.lock:
 			self.buffer.append(data)
 			self.curSize += len(data)
 	
-	def get(self, numBytes=None):
+	def get(self, numBytes: Optional[int]=None) -> Optional[_StreamData]:
 		if numBytes is None:
 			numBytes = self.chunkSize
 
@@ -386,7 +393,7 @@ class StreamBuffer(object):
 
 	# XXX TODO: refactor get and getToChar to use as much common code
 	# as sensible
-	def getToChar(self, char):
+	def getToChar(self, char: _StreamData) -> Optional[_StreamData]:
 		"""returns the the buffer up to the first occurrence of char.
 
 		If char is not present in the buffer, the function returns None.
@@ -419,18 +426,20 @@ class StreamBuffer(object):
 
 		raise AssertionError("This cannot happen")  # pragma: no cover
 	
-	def getRest(self):
+	def getRest(self) -> _StreamData:
 		"""returns the entire buffer as far as it is left over.
 		"""
 		result = self.joiner.join(self.buffer)
 		self.buffer = collections.deque()
 		return result
 
-	def doneWriting(self):
+	def doneWriting(self) -> None:
 		self.finished = True
 
 
-def grouped(n, seq):
+_T = TypeVar("_T")
+
+def grouped(n: int, seq: Sequence[_T]) -> List[List[_T]]:
 	"""yields items of seq in groups n elements.
 
 	If len(seq)%n!=0, the last elements are discarded.
@@ -440,10 +449,12 @@ def grouped(n, seq):
 	>>> list(grouped(3, range(9)))
 	[(0, 1, 2), (3, 4, 5), (6, 7, 8)]
 	"""
-	return list(zip(*([iter(seq)]*n)))
+	# annotation problem: understand why mypy doesn't understand this
+	return list(zip(*([iter(seq)]*n))) # type: ignore
 
 
-def getfirst(args, key, default=Undefined):
+# annotation problem: https://github.com/python/mypy/issues/3737
+def getfirst(args: Dict[str, _T], key: str, default: _T=Undefined) -> _T: # type: ignore
 	"""returns the first value of key in the web argument-like object args.
 
 	args is a dictionary mapping keys to lists of values.  If key is present,
@@ -480,7 +491,7 @@ def getfirst(args, key, default=Undefined):
 		return default
 
 
-def sendUIEvent(eventName, *args):
+def sendUIEvent(eventName: str, *args) -> None:
 	"""sends an eventName to the DC event dispatcher.
 
 	If no event dispatcher is available, do nothing.
@@ -499,7 +510,7 @@ def sendUIEvent(eventName, *args):
 		pass
 
 
-def logOldExc(exc):
+def logOldExc(exc: Exception) -> Exception:
 	"""logs the mutation of the currently handled exception to exc.
 
 	This just does a notifyExceptionMutation using sendUIEvent; it should
@@ -509,7 +520,7 @@ def logOldExc(exc):
 	return exc
 
 
-def getFortranRec(f):
+def getFortranRec(f: BinaryIO) -> Optional[bytes]:
 	"""reads a "fortran record" from f and returns the payload.
 
 	A "fortran record" comes from an unformatted file and has a
@@ -521,7 +532,7 @@ def getFortranRec(f):
 	Of course, f must be open in binary mode.
 	"""
 	try:
-		startPos = f.tell()
+		startPos: Union[int, str] = f.tell()
 	except IOError:
 		startPos = "(stdin)"
 	rawLength = f.read(4)
@@ -540,7 +551,7 @@ def getFortranRec(f):
 	return data
 
 
-def iterFortranRecs(f, skip=0):
+def iterFortranRecs(f: BinaryIO, skip: int=0) -> Generator[bytes, None, None]:
 	"""iterates over the fortran records in f.
 
 	For details, see getFortranRec.
@@ -555,7 +566,7 @@ def iterFortranRecs(f, skip=0):
 		yield rec
 
 
-def getWithCache(url, cacheDir, extraHeaders={}):
+def getWithCache(url: str, cacheDir: Filename, extraHeaders: dict={}) -> bytes:
 	"""returns the content of url, from a cache if possible.
 
 	Of course, you only want to use this if there's some external guarantee
@@ -569,15 +580,15 @@ def getWithCache(url, cacheDir, extraHeaders={}):
 		with open(cacheName, "rb") as f:
 			return f.read()
 	else:
-		with urllib.request.urlopen(url) as f:
+		with urlrequest.urlopen(url) as f:
 			doc = f.read()
 		with open(cacheName, "wb") as f:
 			f.write(doc)
-		urllib.request.urlcleanup()
+		urlrequest.urlcleanup()
 		return doc
 
 
-def rstxToHTMLWithWarning(source, **userOverrides):
+def rstxToHTMLWithWarning(source: str, **userOverrides) -> Tuple[str, str]:
 	"""returns HTML and a string with warnings for a piece of ReStructured
 	text.
 
@@ -605,7 +616,7 @@ def rstxToHTMLWithWarning(source, **userOverrides):
 	return parts["fragment"], warnAccum.getvalue()
 
 
-def rstxToHTML(source, **userOverrides):
+def rstxToHTML(source: str, **userOverrides) -> str:
 	"""returns HTML for a piece of ReStructured text.
 
 	source can be a unicode string or a byte string in utf-8.
@@ -647,30 +658,31 @@ class CaseSemisensitiveDict(dict):
 		dict.__init__(self, *args, **kwargs)
 		self._normCasedCache = None
 
-	def __getitem__(self, key):
+	def __getitem__(self, key: str) -> Any:
 		try:
 			return dict.__getitem__(self, key)
 		except KeyError:
 			pass # try again with normalized case.
 		return self._normCased[key.upper()]
 
-	def __setitem__(self, key, value):
+	def __setitem__(self, key: str, value: Any) -> None:
 		self._normCasedCache = None
 		dict.__setitem__(self, key, value)
 
-	def __contains__(self, key):
+	def __contains__(self, key: object) -> bool:
+		key = cast(str, key)
 		return dict.__contains__(self, key) or key.upper() in self._normCased
 
-	def __delitem__(self, key):
+	def __delitem__(self, key: str) -> None:
 		self.pop(key, None)
 
-	def get(self, key, default=None):
+	def get(self, key: str, default: Any=None) -> Any:
 		try:
 			return self[key]
 		except KeyError:
 			return default
 
-	def pop(self, key, default=KeyError):
+	def pop(self, key: str, default: Any=KeyError) -> Any:
 		try:
 			return dict.pop(self, key)
 		except KeyError:
@@ -686,21 +698,22 @@ class CaseSemisensitiveDict(dict):
 		return CaseSemisensitiveDict(dict.copy(self))
 
 	@property
-	def _normCased(self):
+	def _normCased(self) -> Dict[str, Any]:
 		if self._normCasedCache is None:
 			self._normCasedCache = dict((k.upper(), v)
 				for k, v in self.items())
 		return self._normCasedCache
 
 	@classmethod
-	def fromDict(cls, aDict):
+	def fromDict(cls: Type['CaseSemisensitiveDict'],
+			aDict: Union[dict, 'CaseSemisensitiveDict']) -> 'CaseSemisensitiveDict':
 		if isinstance(aDict, CaseSemisensitiveDict):
 			return aDict
 		else:
 			return cls(aDict)
 
 
-def getCleanBytes(b):
+def getCleanBytes(b: StrOrBytes) -> bytes:
 	"""returns the bytes b in an ASCII representation.
 
 	This is zlib-compressed base64 stuff.  b can be a string, too, in which
@@ -712,7 +725,7 @@ def getCleanBytes(b):
 		zlib.compress(b)).replace(b"\n", b"")
 
 
-def getDirtyBytes(b):
+def getDirtyBytes(b: bytes) -> bytes:
 	"""returns b decoded and uncompressed.
 
 	This is the inverse operation of getCleanBytes.  b must be bytes, and
@@ -744,7 +757,8 @@ def _makeKVLGrammar():
 
 _KVL_GRAMMAR = _makeKVLGrammar()
 
-def parseKVLine(aString):
+
+def parseKVLine(aString: str):
 	"""returns a dictionary for a "key-value line".
 
 	key-value lines represent string-valued dictionaries
@@ -757,8 +771,8 @@ def parseKVLine(aString):
 
 _IDENTIFIER_PATTERN = re.compile("[A-Za-z_]+$")
 
-def makeKVLine(aDict):
-	"""serialized a dictionary to a key-value line.
+def makeKVLine(aDict: StrToStrMap) -> str:
+	"""serializes a dictionary to a key-value line.
 
 	See parseKVLine for details.
 	"""
