@@ -24,6 +24,12 @@ from gavo.utils import excs
 from gavo.utils import misctricks
 from gavo.utils import texttricks
 
+from gavo.utils.dachstypes import (
+	Any, Dict,
+	IO, Iterator,
+	Optional, PlainXMLEvent,
+	Tuple, Type)
+
 class ErrorPosition(object):
 	"""A wrapper for an error position.
 
@@ -31,14 +37,14 @@ class ErrorPosition(object):
 	for missing or unknown values.
 	"""
 	fName = None
-	def __init__(self, fName, line, column):
+	def __init__(self, fName: str, line: int, column: int) -> None:
 		self.line = line or '?'
 		self.col = column
 		if self.col is None:
 			self.col = '?'
 		self.fName = fName
 	
-	def __str__(self):
+	def __str__(self) -> str:
 		if self.fName:
 			return "%s, (%s, %s)"%(self.fName, self.line, self.col)
 		else:
@@ -51,6 +57,14 @@ class iterparse(object):
 	To keep things simple downstream, we swallow all namespace prefixes,
 	if present.
 
+	The events this yields are one of:
+
+	* ("start", element-name, attributes)
+	* ("end", element-name, None)
+	* ("data", None, payload)
+	
+	utils.dachstypes.PlainXMLEvent is a type for these.
+
 	iterparse is constructed with a source (anything that can read(source))
 	and optionally a custom error class.  This error class needs to
 	have the message as the first argument.  Since expat error messages
@@ -62,10 +76,12 @@ class iterparse(object):
 	event.  The *end* of the construct that caused an event can
 	be retrieved using pos.
 	"""
-	chunkSize = 2**20
+	chunkSize: int = 2**20
 	"The number of bytes handed to expat from iterparse at one go."
 
-	def __init__(self, source, parseErrorClass=excs.StructureError):
+	def __init__(self,
+			source: IO,
+			parseErrorClass: Type[excs.Error] = excs.StructureError) -> None:
 		self.source = source
 		self.parseErrorClass = parseErrorClass
 
@@ -80,30 +96,32 @@ class iterparse(object):
 		self.parser = expat.ParserCreate()
 		self.parser.buffer_text = True
 		self.lastLine, self.lastColumn = 1, 0
-		self.evBuf = collections.deque()
+		self.evBuf: collections.deque[Tuple[
+				PlainXMLEvent, Optional[Tuple[int, int]]]
+			] = collections.deque()
 		self.parser.StartElementHandler = self._startElement
 		self.parser.EndElementHandler = self._endElement
 		self.parser.CharacterDataHandler = self._characters
 
-	def __iter__(self):
+	def __iter__(self) -> Iterator[PlainXMLEvent]:
 		return self
 	
-	def _startElement(self, name, attrs):
+	def _startElement(self, name: str, attrs: Dict[str, str]) -> None:
 		self.evBuf.append(
 			(("start", name.split(":")[-1], attrs),
 				(self.parser.CurrentLineNumber, self.parser.CurrentColumnNumber)))
 	
-	def _endElement(self, name):
+	def _endElement(self, name: str) -> None:
 		self.evBuf.append((("end", name.split(":")[-1], None),
 			(self.parser.CurrentLineNumber, self.parser.CurrentColumnNumber)))
 	
-	def _characters(self, data):
+	def _characters(self, data: str) -> None:
 		self.evBuf.append((("data", None, data), None))
 
-	def pushBack(self, type, name, payload):
+	def pushBack(self, type: str, name: str, payload: str) -> None:
 		self.evBuf.appendleft(((type, name, payload), None))
 
-	def __next__(self):
+	def __next__(self) -> PlainXMLEvent:
 		while not self.evBuf:
 			try:
 				nextChunk = self.source.read(self.chunkSize)
@@ -115,8 +133,9 @@ class iterparse(object):
 			except expat.ExpatError as ex:
 				srcDesc = getattr(self.source, "name", "(internal source)")
 				newEx = self.parseErrorClass(srcDesc+" "+str(ex))
-				newEx.posInMsg = True  # see base.xmlstruct
-				newEx.inFile = srcDesc
+				# see base.xmlstruct for the following ignore.
+				newEx.posInMsg = True  # type: ignore
+				newEx.inFile = srcDesc # type: ignore
 				raise misctricks.logOldExc(newEx)
 
 		if not self.evBuf:
@@ -126,22 +145,28 @@ class iterparse(object):
 			self.lastLine, self.lastColumn = pos
 		return event
 
-	def close(self):
+	def close(self) -> None:
 		self.parser.Parse("", True)
 		self.parser.StartElementHandler =\
 		self.parser.EndElementHandler = \
 		self.parser.CharacterDataHandler = None
 
 	@property
-	def pos(self):
+	def pos(self) -> ErrorPosition:
 		return ErrorPosition(self.inputName, self.lastLine, self.lastColumn)
 
-	def getParseError(self, msg):
+	def getParseError(self, msg: str) -> Exception:
+		"""returns an instance of what you passed in in parseErrorClass
+		with location information.
+		"""
 		res = self.parseErrorClass("At %s: %s"%(self.pos, msg))
-		res.posInMsg = True # see base.xmlstruct
+		# see base.xmlstruct for the following ignore
+		res.posInMsg = True # type: ignore
 		return res
 
 
+# Not type annotating the SAX interface methods.  See xml.sax.handler
+# declarations for these.
 class StartEndHandler(ContentHandler):
 	"""This class provides startElement, endElement and characters
 	methods that translate events into method calls.
@@ -168,13 +193,18 @@ class StartEndHandler(ContentHandler):
 		self.contentsStack = [[]]
 		self._initialize()
 
-	def _initialize(self):
+	def _initialize(self) -> None:
 		pass
 
-	def processingInstruction(self, target, data):
+	def processingInstruction(self, target: str, data:str) -> None:
 		self.contentsStack[-1].append(data)
 
-	def cleanupName(self, name):
+	def cleanupName(self, name: str) -> str:
+		"""returning an element name replacing _ for -.
+
+		That's so people can define methods for such names, and if that
+		leads to clashes, the schema designers were sadists.
+		"""
 		return name.split(":")[-1].replace("-", "_")
 
 	def startElementNS(self, namePair, qName, attrs):
@@ -186,7 +216,7 @@ class StartEndHandler(ContentHandler):
 				newAttrs["{%s}%s"%(ns, name)] = attrs[(ns, name)]
 		self.startElement(namePair[1], newAttrs)
 
-	def startElement(self, name, attrs):
+	def startElement(self, name: str, attrs) -> None:
 		self.contentsStack.append([])
 		name = self.cleanupName(name)
 		self.elementStack.append((name, attrs))
@@ -198,7 +228,7 @@ class StartEndHandler(ContentHandler):
 	def endElementNS(self, namePair, qName):
 		self.endElement(namePair[1])
 
-	def endElement(self, name, suppress=False):
+	def endElement(self, name: str, suppress: Optional[bool]=False) -> None:
 		contents = "".join(self.contentsStack.pop())
 		name = self.cleanupName(name)
 		_, attrs = self.elementStack.pop()
@@ -214,23 +244,24 @@ class StartEndHandler(ContentHandler):
 	def characters(self, chars):
 		self.contentsStack[-1].append(chars)
 	
-	def getResult(self):
+	def getResult(self) -> Any:
 		return self.contentsStack[0][0]
 
-	def getParentTag(self, depth=1):
+	def getParentTag(self, depth:int=1) -> Optional[str]:
 		"""Returns the name of the parent element.
 
 		This only works as written here in end handlers.  In start handlers,
-		you have to path depth=2 (since their tag already is on the stack.
+		you have to path depth=2 (since their tag already is on the stack).
 		"""
 		if self.elementStack:
 			return self.elementStack[-depth][0]
+		return None
 	
-	def parse(self, stream):
+	def parse(self, stream: IO) -> ContentHandler:
 		xml.sax.parse(stream, self)
 		return self
 	
-	def parseString(self, string):
+	def parseString(self, string: str) -> ContentHandler:
 		xml.sax.parseString(string, self)
 		return self
 
