@@ -142,7 +142,7 @@ def _byteaMapperFactory(colDesc):
 registerDefaultMF(_byteaMapperFactory)
 
 
-GEOMETRY_ARRAY_TYPES = set(["spoint", "spoly", "scircle", "sbox"])
+GEOMETRY_TYPES = set(["spoint", "spoly", "scircle", "sbox", "smoc"])
 
 def _pgSphereMapperFactory(colDesc):
 	"""A factory for functions turning pgsphere types to DALI arrays.
@@ -151,8 +151,8 @@ def _pgSphereMapperFactory(colDesc):
 # we still map this manually right now in order to help non-VOTable
 # output formats; this will not work (properly) when the output
 # format doesn't have array support.  For these, just create
-# a new mapper factory for GEOMETRY_ARRAY_TYPES.
-	if not colDesc["dbtype"] in GEOMETRY_ARRAY_TYPES:
+# a new mapper factory for GEOMETRY_TYPES.
+	if not colDesc["dbtype"] in GEOMETRY_TYPES:
 		return
 
 	def mapper(val):
