@@ -19,12 +19,14 @@ import string
 import time
 from email import utils as emailutils
 
-try:
-	from astropy.coordinates import angle_formats
-except ImportError:
-	# old astropy
-	from astropy.coordinates import angle_utilities as angle_formats
+from gavo.utils.dachstypes import (Any, Filename, Generator,
+	Optional, StrToStrMap, TextIO, Tuple, Union)
 
+try:
+	# old astropy
+	from astropy.coordinates import angle_utilities as angle_formats # type: ignore
+except ImportError:
+	from astropy.coordinates.angles import formats as angle_formats # type: ignore
 
 from gavo.utils import codetricks
 from gavo.utils import misctricks
@@ -53,7 +55,7 @@ xmlEntities = {
 }
 
 
-def formatSize(val, sf=1):
+def formatSize(val: float, sf: int=1) -> str:
 	"""returns a human-friendly representation of a file size.
 	"""
 	if val<1e3:
@@ -66,7 +68,7 @@ def formatSize(val, sf=1):
 		return "%.*fGiB"%(sf, val/1024./1024./1024)
 
 
-def makeEllipsis(aStr, maxLen=60, ellChars="..."):
+def makeEllipsis(aStr: str, maxLen: int=60, ellChars: str="...") -> str:
 	"""returns aStr cropped to maxLen if necessary.
 
 	Cropped strings are returned with an ellipsis marker.
@@ -77,7 +79,7 @@ def makeEllipsis(aStr, maxLen=60, ellChars="..."):
 	return aStr
 
 
-def makeLeftEllipsis(aStr, maxLen=60):
+def makeLeftEllipsis(aStr: str, maxLen: int=60):
 	"""returns aStr shortened to maxLen by dropping prefixes if necessary.
 
 	Cropped strings are returned with an ellipsis marker.
@@ -91,7 +93,7 @@ def makeLeftEllipsis(aStr, maxLen=60):
 	return aStr
 
 
-def makeSourceEllipsis(sourceToken):
+def makeSourceEllipsis(sourceToken: Any) -> str:
 	"""returns a string hopefully representative for a source token.
 
 	These are, in particular, passed around within rsc.makeData.  Usually,
@@ -107,7 +109,7 @@ def makeSourceEllipsis(sourceToken):
 
 
 @codetricks.document
-def getFileStem(fPath):
+def getFileStem(fPath: str):
 	"""returns the file stem of a file path.
 
 	The base name is what remains if you take the base name and split off
@@ -129,7 +131,10 @@ def getFileStem(fPath):
 	return os.path.splitext(os.path.basename(fPath))[0]
 	
 
-def formatSimpleTable(data, stringify=True, titles=None):
+def formatSimpleTable(
+		data: list[list[Any]],
+		stringify: bool=True,
+		titles: Optional[list[str]]=None) -> str:
 	"""returns a string containing a text representation of tabular data.
 
 	All columns of data are simply stringified, then the longest member
@@ -162,7 +167,10 @@ def formatSimpleTable(data, stringify=True, titles=None):
 
 
 @codetricks.document
-def getRelativePath(fullPath, rootPath, liberalChars=True):
+def getRelativePath(
+		fullPath: str,
+		rootPath: str,
+		liberalChars: bool=True) -> str:
 	"""returns rest if fullPath has the form rootPath/rest and raises an
 	exception otherwise.
 
@@ -181,7 +189,7 @@ def getRelativePath(fullPath, rootPath, liberalChars=True):
 	return res
 
 
-def resolvePath(rootPath, relPath):
+def resolvePath(rootPath: str, relPath: str) -> str:
 	"""joins relPath to rootPath and makes sure the result really is
 	in rootPath.
 	"""
@@ -196,7 +204,7 @@ def resolvePath(rootPath, relPath):
 	return fullPath
 
 
-def fixIndentation(code, newIndent, governingLine=0):
+def fixIndentation(code: str, newIndent: str, governingLine: int=0) -> str:
 	"""returns code with all whitespace from governingLine removed from
 	every line and newIndent prepended to every line.
 
@@ -218,8 +226,9 @@ def fixIndentation(code, newIndent, governingLine=0):
 	reserved, codeLines = codeLines[:governingLine], codeLines[governingLine:]
 	while codeLines:
 		if codeLines[0].strip():
-			firstIndent = re.match("^\s*", codeLines[0]).group()
-			break
+			if mat := re.match("^\s*", codeLines[0]):
+				firstIndent = mat.group()
+				break
 		else:
 			reserved.append(codeLines.pop(0))
 	if codeLines:
@@ -238,7 +247,7 @@ def fixIndentation(code, newIndent, governingLine=0):
 
 
 @functools.lru_cache()
-def _getREForPercentExpression(format):
+def _getREForPercentExpression(format: str) -> re.Pattern:
 	"""helps parsePercentExpression.
 	"""
 	parts = re.split(r"(%\w)", format)
@@ -259,7 +268,7 @@ def _getREForPercentExpression(format):
 	return re.compile("".join(newReParts)+"$")
 
 
-def parsePercentExpression(literal, format):
+def parsePercentExpression(literal: str, format: str) -> dict:
 	"""returns a dictionary of parts in the %-template format.
 
 	format is a template with %<conv> conversions, no modifiers are
@@ -285,7 +294,7 @@ def parsePercentExpression(literal, format):
 	return mat.groupdict()
 
 
-def parseAssignments(assignments):
+def parseAssignments(assignments: str) -> StrToStrMap:
 	"""returns a name mapping dictionary from a list of assignments.
 
 	This is the preferred form of communicating a mapping from external names
@@ -306,7 +315,7 @@ def parseAssignments(assignments):
 
 
 @codetricks.document
-def hmsToDeg(hms, sepChar=None):
+def hmsToDeg(hms: str, sepChar: Optional[str]=None) -> float:
 	"""returns the time angle (h m s.decimals) as a float in degrees.
 
 	>>> "%3.8f"%hmsToDeg("22 23 23.3")
@@ -322,14 +331,14 @@ def hmsToDeg(hms, sepChar=None):
 	hms = hms.strip()
 	try:
 		if sepChar=="":
-			parts = hms[:2], hms[2:4], hms[4:]
+			parts = [hms[:2], hms[2:4], hms[4:]]
 		else:
 			parts = hms.split(sepChar)
 		if len(parts)==3:
 			hours, minutes, seconds = parts
 		elif len(parts)==2:
 			hours, minutes = parts
-			seconds = 0
+			seconds = "0"
 		else:
 			raise ValueError("Too many parts")
 		timeSeconds = int(hours)*3600+float(minutes)*60+float(seconds or "0")
@@ -340,7 +349,7 @@ def hmsToDeg(hms, sepChar=None):
 
 
 @codetricks.document
-def dmsToDeg(dmsAngle, sepChar=None):
+def dmsToDeg(dmsAngle: str, sepChar: Optional[str]=None) -> float:
 	"""returns the degree minutes seconds-specified dmsAngle as a
 	float in degrees.
 
@@ -362,17 +371,17 @@ def dmsToDeg(dmsAngle, sepChar=None):
 		sign, dmsAngle = -1, dmsAngle[1:].strip()
 	try:
 		if sepChar=="":
-			parts = dmsAngle[:2], dmsAngle[2:4], dmsAngle[4:]
+			parts = [dmsAngle[:2], dmsAngle[2:4], dmsAngle[4:]]
 		else:
 			parts = dmsAngle.split(sepChar)
 		if len(parts)==3:
 			deg, min, sec = parts
 		elif len(parts)==2:
 			deg, min = parts
-			sec = 0
+			sec = "0"
 		else:
 			raise ValueError("Invalid # of parts")
-		arcSecs = sign*(int(deg)*3600+float(min)*60+float(sec or 0))
+		arcSecs = sign*(int(deg)*3600+float(min)*60+float(sec or "0"))
 	except ValueError:
 		raise misctricks.logOldExc(
 			ValueError("Invalid dms value with sepChar %s: %s"%(
@@ -380,17 +389,17 @@ def dmsToDeg(dmsAngle, sepChar=None):
 	return arcSecs/3600
 
 
-def fracHoursToDeg(fracHours):
+def fracHoursToDeg(fracHours: float) -> float:
 	"""returns the time angle fracHours given in decimal hours in degrees.
 	"""
 	return float(fracHours)*360./24.
 
 
 @codetricks.document
-def degToHms(deg,
-		sepChar=" ",
-		secondFracs=3,
-		truncate=False):
+def degToHms(deg: float,
+		sepChar: str=" ",
+		secondFracs: int=3,
+		truncate: bool=False) -> str:
 	"""converts a float angle in degrees to an time angle (hh:mm:ss.mmm).
 
 	This takes a lot of optional arguments:
@@ -431,7 +440,8 @@ def degToHms(deg,
 
 
 @codetricks.document
-def hoursToHms(decimal_hours, sepChar=":", secondFracs=0):
+def hoursToHms(decimal_hours: float,
+		sepChar: str=":", secondFracs:int =0) -> str:
 	"""returns a time span in hours in sexagesmal time (h:m:s).
 
 	The optional arguments are as for degToHms.
@@ -454,12 +464,12 @@ def hoursToHms(decimal_hours, sepChar=":", secondFracs=0):
 
 
 @codetricks.document
-def degToDms(deg,
-		sepChar=" ",
-		secondFracs=2,
-		preserveLeading=False,
-		truncate=False,
-		addSign=True):
+def degToDms(deg: float,
+		sepChar: str=" ",
+		secondFracs: int=2,
+		preserveLeading: bool=False,
+		truncate: bool=False,
+		addSign: bool=True) -> str:
 	"""converts a float angle in degrees to a sexagesimal string.
 
 	This takes a lot of optional arguments:
@@ -503,7 +513,9 @@ def degToDms(deg,
 
 
 @codetricks.document
-def makeIAUId(prefix, long, lat, longSec=0, latSec=0):
+def makeIAUId(prefix: str,
+		long: float, lat: float,
+		longSec: int=0, latSec: int=0) -> str:
 	"""returns an (equatorial) IAU identifier for an object at long and lat.
 
 	The rules are given on https://cds.unistra.fr/Dic/iau-spec.html
@@ -518,7 +530,7 @@ def makeIAUId(prefix, long, lat, longSec=0, latSec=0):
 	return f"{prefix}{longPart}{latPart}"
 
 
-def datetimeToRFC2616(dt):
+def datetimeToRFC2616(dt: datetime.datetime) -> str:
 	"""returns a UTC datetime object in the format required by http.
 
 	This may crap when you fuzz with the locale.  In general, when handling
@@ -528,25 +540,28 @@ def datetimeToRFC2616(dt):
 	return dt.strftime('%a, %d %b %Y %H:%M:%S GMT')
 
 
-def parseRFC2616Date(s):
+def parseRFC2616Date(s: str) -> float:
 	"""returns seconds since unix epoch representing UTC from the HTTP-compatible
 	time specification s.
 	"""
 	parts = emailutils.parsedate_tz(s)
+	assert parts is not None
 	return emailutils.mktime_tz(parts)
 
 
 # The following timegm implementation is due to Frederik Lundh
-def _d(y, m, d, days=(0,31,59,90,120,151,181,212,243,273,304,334,365)):
-		return (((y - 1901)*1461)/4 + days[m-1] + d + (
-			(m > 2 and not y % 4 and (y % 100 or not y % 400)) and 1))
+def _d(y: int, m: int, d: int,
+		days: Tuple[int, ...]=(0,31,59,90,120,151,181,212,243,273,304,334,365)
+		) -> int:
+	return (((y - 1901)*1461)//4 + days[m-1] + d + (
+		(m > 2 and not y % 4 and (y % 100 or not y % 400)) and 1))
 
-def timegm(tm, epoch=_d(1970,1,1)):
+def timegm(tm: time.struct_time, epoch: float=_d(1970,1,1)):
 		year, month, day, h, m, s = tm[:6]
 		return (_d(year, month, day) - epoch)*86400 + h*3600 + m*60 + s
 
 
-def formatRFC2616Date(secs=None):
+def formatRFC2616Date(secs: Optional[float]=None) -> str:
 	"""returns an RFC2616 date string for UTC seconds since unix epoch.
 	"""
 	if secs is None:
@@ -560,7 +575,7 @@ _isoDTRE = re.compile(r"(?P<year>\d\d\d\d)-?(?P<month>\d\d)-?(?P<day>\d\d)"
 
 
 @codetricks.document
-def parseISODT(literal, useTime=False):
+def parseISODT(literal: str, useTime: bool=False) -> datetime.datetime:
 	"""returns a datetime object for a ISO time literal.
 
 	There's no real timezone support yet, but we accept and ignore various
@@ -587,7 +602,7 @@ def parseISODT(literal, useTime=False):
 	Traceback (most recent call last):
 	ValueError: second must be in 0..59
 	>>> parseISODT("2016-12-31T23:59:60", useTime=True)
-	datetime.datetime(2017, 1, 1, 1, 0)
+	datetime.datetime(2017, 1, 1, 0, 0)
 	>>> parseISODT("junk")
 	Traceback (most recent call last):
 	ValueError: Bad ISO datetime literal: junk (required format: yyyy-mm-ddThh:mm:ssZ)
@@ -609,16 +624,17 @@ def parseISODT(literal, useTime=False):
 		parts["secFracs"] = "0"+parts["secFracs"]
 	
 	if useTime:
-		return datetime.datetime.fromtimestamp(
+		return datetime.datetime.utcfromtimestamp(
 			calendar.timegm((
-				int(parts["year"]), int(parts["month"]), int(parts["day"]),
-				int(parts["hour"]), int(parts["minute"]), int(parts["seconds"]),
+				# type calculus broken for RE results in mypy 1.0.1
+				int(parts["year"]), int(parts["month"]), int(parts["day"]),  # type: ignore
+				int(parts["hour"]), int(parts["minute"]), int(parts["seconds"]),  # type: ignore
 				-1, -1, -1)))
 
 	else:
-		return datetime.datetime(int(parts["year"]), int(parts["month"]),
-			int(parts["day"]), int(parts["hour"]), int(parts["minute"]),
-			int(parts["seconds"]), int(float(parts["secFracs"])*1000000))
+		return datetime.datetime(int(parts["year"]), int(parts["month"]), # type: ignore
+			int(parts["day"]), int(parts["hour"]), int(parts["minute"]),    # type: ignore
+			int(parts["seconds"]), int(float(parts["secFracs"])*1000000))   # type: ignore
 
 
 _SUPPORTED_DT_FORMATS =[
@@ -626,7 +642,13 @@ _SUPPORTED_DT_FORMATS =[
 	'%Y-%m-%d %H:%M:%S',
 	'%Y-%m-%d',]
 
-def parseDefaultDatetime(literal):
+def parseDefaultDatetime(literal: Optional[Union[str, datetime.datetime]]
+		) -> Optional[datetime.datetime]:
+	"""returns a datetime from string or passes through datetimes and Nones.
+
+	The function will try to parse a string in various ways; we will
+	try not to drop formats from one minor version to the next.
+	"""
 	if literal is None or isinstance(literal, datetime.datetime):
 		return literal
 	if literal.endswith("Z"):
@@ -642,20 +664,26 @@ def parseDefaultDatetime(literal):
 	return parseISODT(literal)
 
 
-def parseDefaultDate(literal):
+def parseDefaultDate(literal: Optional[Union[str, datetime.date]]
+		) -> Optional[datetime.date]:
+	"""parseDefaultDatetime's little sister.
+	"""
 	if literal is None or isinstance(literal, datetime.date):
 		return literal
 	return datetime.date(*time.strptime(literal, '%Y-%m-%d')[:3])
 
 
-def parseDefaultTime(literal):
+def parseDefaultTime(literal: Optional[Union[str, datetime.time]]
+		) -> Optional[datetime.time]:
+	"""parseDefaultDatetime's other little sister.
+	"""
 	if literal is None or isinstance(literal, datetime.time):
 		return literal
 	# as long as we're builing on top of time, we can't do fractional seconds
 	return datetime.time(*time.strptime(literal, '%H:%M:%S')[3:6])
 
 
-def roundToSeconds(dt):
+def roundToSeconds(dt: datetime.datetime) -> datetime.datetime:
 	"""returns a datetime instance rounded to whole seconds.
 
 	This also recklessly clears any time zone marker.  So, don't pass
@@ -668,7 +696,7 @@ def roundToSeconds(dt):
 		return dt.replace(microsecond=0, tzinfo=None)
 
 
-def formatISODT(dt):
+def formatISODT(dt: datetime.datetime) -> str:
 	"""returns some ISO8601 representation of a datetime instance.
 
 	The reason for preferring this function over a simple str is that
@@ -701,7 +729,7 @@ def formatISODT(dt):
 
 
 @codetricks.document
-def formatFloat(f):
+def formatFloat(f: float) -> str:
 	"""returns floating-point numbers somewhat suitable for human consumption.
 
 	The idea of this function is to slowly migrate ad-hoc formatting of
@@ -737,14 +765,14 @@ class NameMap(object):
 	The qp-decoded strings are assumed to be utf-8 encoded, but there's a
 	constructor argument to change that.
 	"""
-	def __init__(self, src, missingOk=False, enc="utf-8"):
+	def __init__(self, src: Filename, missingOk: bool=False, enc: str="utf-8"):
 		self._parseSrc(src, missingOk, enc)
 	
-	def __contains__(self, name):
+	def __contains__(self, name: str) -> bool:
 		return name in self.namesDict
 
-	def _parseSrc(self, src, missingOk, enc):
-		self.namesDict = {}
+	def _parseSrc(self, src: Filename, missingOk: bool, enc: str) -> None:
+		self.namesDict: StrToStrMap = {}
 		try:
 			f = open(src, "rb")
 		except IOError:
@@ -765,7 +793,7 @@ class NameMap(object):
 				"Syntax error in %s: Line %s not understood."%(src, repr(ln))))
 		f.close()
 	
-	def resolve(self, name):
+	def resolve(self, name: str) -> str:
 		return self.namesDict[name.lower()]
 
 
@@ -778,7 +806,7 @@ _STANDARD_ENTITIES = {
 }
 
 
-def _decodeEntityref(matob):
+def _decodeEntityref(matob: re.Match) -> str:
 	entRef = matob.group(1)
 	if entRef in _STANDARD_ENTITIES:
 		return _STANDARD_ENTITIES[entRef]
@@ -790,17 +818,27 @@ def _decodeEntityref(matob):
 		raise ValueError("Unknown entity reference: &%s;"%entRef)
 
 
-def replaceXMLEntityRefs(unicodeString):
+def replaceXMLEntityRefs(unicodeString: str) -> str:
+	"""retplaces all known HTML entities in unicodeString with actual unicode
+	chars.
+
+	(and dies on unknown entities).
+
+	TODO: this is unused and probably not very useful to clients.  Discard?
+	"""
 	return entityrefPat.sub(_decodeEntityref, unicodeString)
 
 
-def ensureOneSlash(s):
+def ensureOneSlash(s: str) -> str:
 	"""returns s with exactly one trailing slash.
 	"""
 	return s.rstrip("/")+"/"
 
 
-def _iterSimpleTextNoContinuation(f):
+_SimpleTextGenerator = Generator[Tuple[int, str], None, None]
+
+def _iterSimpleTextNoContinuation(f: TextIO
+		) -> _SimpleTextGenerator:
 	"""helps iterSimpleText.
 	"""
 	for (lineNumber, curLine) in enumerate(f):
@@ -810,7 +848,7 @@ def _iterSimpleTextNoContinuation(f):
 
 
 @codetricks.document
-def iterSimpleText(f):
+def iterSimpleText(f: TextIO) -> _SimpleTextGenerator:
 	"""iterates over ``(physLineNumber, line)`` in f with some usual
 	conventions for simple data files.
 
@@ -853,14 +891,16 @@ def iterSimpleText(f):
 
 _RANDOM_STRING_OK_CHARS = string.ascii_letters+string.digits+"_.,"
 
-def getRandomString(length):
+def getRandomString(length: int) -> str:
 	"""returns a random string of harmless printable characters.
 	"""
 	return "".join(
 		random.choice(_RANDOM_STRING_OK_CHARS) for c in range(length))
 
 
-def safe_str(val):
+def safe_str(val: Any) -> str:
+	"""returns a reasonable string from pretty much anything.
+	"""
 	if isinstance(val, str):
 		return val
 	elif isinstance(val, bytes):
@@ -869,7 +909,7 @@ def safe_str(val):
 		return str(val)
 
 
-def bytify(s):
+def bytify(s: Union[str, bytes]) -> bytes:
 	"""returns s utf-8 encoded if it is a string, unmodified otherwise.
 	"""
 	if isinstance(s, str):
@@ -877,7 +917,7 @@ def bytify(s):
 	return s
 
 
-def debytify(b, enc="ascii"):
+def debytify(b: Union[str, bytes], enc: str="ascii"):
 	"""returns a bytestring b as a normal string.
 
 	This will return b unless it's bytes.  If it is bytes, it will be
@@ -890,7 +930,7 @@ def debytify(b, enc="ascii"):
 
 DEFUSE_NONPRINTABLE = bytes.maketrans(bytes(range(32)), b" "*32)
 
-def defuseFileName(fName, replaceSlash=True):
+def defuseFileName(fName: Union[Filename, bytes], replaceSlash: bool=True):
 	"""returns fName without any non-ASCII or slashes but in a way that
 	people can still work out what the file name has been.
 
@@ -898,9 +938,8 @@ def defuseFileName(fName, replaceSlash=True):
 	a string that's guaranteed to be ASCII only.  With replaceSlash=False,
 	it can also double as a reasonable asciification.
 	"""
-	if isinstance(fName, str):
-		fName = fName.encode("utf-8")
-	res = quopri.encodestring(fName
+	nameBytes = fName.encode("utf-8") if isinstance(fName, str) else fName
+	res = quopri.encodestring(nameBytes
 		).translate(DEFUSE_NONPRINTABLE
 		).decode("ascii")
 	if replaceSlash:
@@ -908,7 +947,7 @@ def defuseFileName(fName, replaceSlash=True):
 	return res
 
 
-def parseAccept(aString):
+def parseAccept(aString: str) -> StrToStrMap:
 	"""parses an RFC 2616 accept header and returns a dict mapping media
 	type patterns to their (unparsed) parameters.
 
