@@ -182,6 +182,13 @@ def getXtypeEncoderCode(field):
 		return loopify([
 			"if isinstance(val, pgsphere.PgSAdapter):",
 			"  val = val.asDALI()"])
+	
+	elif field.xtype=="json":
+		return [
+			"if isinstance(val, (dict, list)):",
+			"  val = json.dumps(val)",
+			"else:",
+			"  val = coding.forceASCII(val)"]
 
 	else:
 		return []
