@@ -4,13 +4,13 @@ An interface to querying TAP servers (i.e., a TAP client).
 This is deprecated.  Use pyvo.dal.tap instead.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
-import io 
+import io
 import http.client
 import socket
 import time
@@ -69,12 +69,12 @@ class WrongStatus(ProtocolError):
 class RemoteError(Error):
 	"""is raised when the remote size signals an error.
 
-	The content of the remote error document can be retrieved in the 
+	The content of the remote error document can be retrieved in the
 	remoteMessage attribute.
 	"""
 	def __init__(self, remoteMessage):
 		self.remoteMessage = remoteMessage
-		Error.__init__(self, 
+		Error.__init__(self,
 			"Remote: "+remoteMessage,
 			hint="This means that"
 			" something in your query was bad according to the server."
@@ -110,7 +110,7 @@ class _FormData(MIMEMultipart):
 	"""
 	def __init__(self):
 		MIMEMultipart.__init__(self, "form-data")
-		self.set_param("boundary", "========== bounda r y 930 ")
+		self.set_param("boundary", "========== bounda r y 930")
 		self.epilogue = ""
 	
 	def addFile(self, paramName, fileName, data):
@@ -138,12 +138,12 @@ class _FormData(MIMEMultipart):
 		"""returns a string serialisation of this message suitable for HTTP
 		upload.
 
-		This is as_string, except we're introducing crlfs when it seems
-		the line separator is just an lf.
+		This is as_string, except we're cutting off the header (which is
+		part of the HTTP header for us), and we are changing the line separator
+		to crlf.
 		"""
-		data = self.as_string()
-		if not "\r\n" in data:
-			data = data.replace("\n", "\r\n")
+		data = self.as_string().split("\n\n", 1)[-1]
+		data = data.replace("\n", "\r\n")
 		return data
 
 	@classmethod
@@ -340,7 +340,7 @@ def _pruneAttrNS(attrs):
 
 class _CapabilitiesParser(utils.StartEndHandler):
 # VOSI; each capability is a dict with at least a key interfaces.
-# each interface is a dict with key type (namespace prefix not expanded; 
+# each interface is a dict with key type (namespace prefix not expanded;
 # change that?), accessURL, and use.
 	def __init__(self):
 		utils.StartEndHandler.__init__(self)
@@ -438,7 +438,7 @@ class LocalResult(object):
 
 
 def _canUseFormEncoding(params):
-	"""returns true if userParams can be transmitted in a 
+	"""returns true if userParams can be transmitted in a
 	x-www-form-urlencoded payload.
 	"""
 	for val in list(params.values()):
@@ -462,8 +462,8 @@ def request(scheme, host, path, data="", customHeaders={}, method="GET",
 	data usually is a byte string, but you can also pass a dictionary
 	which then will be serialized using _FormData above.
 
-	You can set followRedirects to True.  This means that the 
-	303 "See other" codes that many UWS action generate will be followed 
+	You can set followRedirects to True.  This means that the
+	303 "See other" codes that many UWS action generate will be followed
 	and the document at the other end will be obtained.  For many
 	operations this will lead to an error; only do this for slightly
 	broken services.
@@ -515,7 +515,7 @@ def request(scheme, host, path, data="", customHeaders={}, method="GET",
 	if followRedirects and resp.status in ACCEPTED_REDIRECTS:
 		parts = urllib.parse.urlparse(resp.getheader("location"))
 		return request(parts.scheme, parts.netloc, parts.path,
-			method="GET", expectedStatus=expectedStatus, 
+			method="GET", expectedStatus=expectedStatus,
 			followRedirects=followRedirects-1)
 
 	if expectedStatus is not None:
@@ -529,7 +529,7 @@ def _makeAtomicValueGetter(methodPath, parser):
 # This is for building ADQLTAPJob's properties (phase, etc.)
 	def getter(self):
 		destURL = self.jobPath+methodPath
-		response = request(self.destScheme, self.destHost, destURL, 
+		response = request(self.destScheme, self.destHost, destURL,
 			expectedStatus=200)
 		return _parseWith(parser(), response.data)
 	return getter
@@ -539,7 +539,7 @@ def _makeAtomicValueSetter(methodPath, serializer, parameterName):
 # This is for building ADQLTAPJob's properties (phase, etc.)
 	def setter(self, value):
 		destURL = self.jobPath+methodPath
-		request(self.destScheme, self.destHost, destURL, 
+		request(self.destScheme, self.destHost, destURL,
 			{parameterName: serializer(value)}, method="POST",
 			expectedStatus=303)
 	return setter
@@ -572,7 +572,7 @@ class ADQLTAPJob(_WithEndpoint):
 
 	See :dachsdoc:`adql.html` for details.
 	"""
-	def __init__(self, endpointURL, query=None, jobId=None, lang="ADQL", 
+	def __init__(self, endpointURL, query=None, jobId=None, lang="ADQL",
 			userParams={}, timeout=None):
 		self._defineEndpoint(endpointURL)
 		self.timeout = timeout
@@ -615,7 +615,7 @@ class ADQLTAPJob(_WithEndpoint):
 		if self.jobPath is not None:
 			if usePOST:
 				request(self.destScheme, self.destHost, self.jobPath, method="POST",
-					data={"ACTION": "DELETE"}, expectedStatus=303, 
+					data={"ACTION": "DELETE"}, expectedStatus=303,
 					timeout=self.timeout)
 			else:
 				request(self.destScheme, self.destHost, self.jobPath, method="DELETE",
@@ -624,14 +624,14 @@ class ADQLTAPJob(_WithEndpoint):
 	def start(self):
 		"""asks the remote side to start the job.
 		"""
-		request(self.destScheme, self.destHost, self.jobPath+"/phase", 
-			{"PHASE": "RUN"}, method="POST", expectedStatus=303, 
+		request(self.destScheme, self.destHost, self.jobPath+"/phase",
+			{"PHASE": "RUN"}, method="POST", expectedStatus=303,
 			timeout=self.timeout)
 
 	def abort(self):
 		"""asks the remote side to abort the job.
 		"""
-		request(self.destScheme, self.destHost, self.jobPath+"/phase", 
+		request(self.destScheme, self.destHost, self.jobPath+"/phase",
 			{"PHASE": "ABORT"}, method="POST", expectedStatus=303,
 			timeout=self.timeout)
 
@@ -660,7 +660,7 @@ class ADQLTAPJob(_WithEndpoint):
 		"""
 		attempts = 0
 		while True:
-			curPhase = self.phase 
+			curPhase = self.phase
 			if curPhase in phases:
 				break
 			time.sleep(pollInterval)
@@ -689,7 +689,7 @@ class ADQLTAPJob(_WithEndpoint):
 	destruction = property(
 		_makeAtomicValueGetter("/destruction", _makeFlatParser(
 			lambda data: utils.parseISODT(data.decode("ascii")))),
-		_makeAtomicValueSetter("/destruction", 
+		_makeAtomicValueSetter("/destruction",
 			lambda dt: dt.strftime("%Y-%m-%dT%H:%M:%S.000"), "DESTRUCTION"))
 
 	def makeJobURL(self, jobPath):
@@ -795,10 +795,10 @@ class ADQLTAPJob(_WithEndpoint):
 			uploadFragments.append("%s,param:%s"%(name, uploadKey))
 
 		form.addParam("UPLOAD", ";".join(uploadFragments))
-		request(self.destScheme, self.destHost, self.jobPath+"/parameters", 
+		request(self.destScheme, self.destHost, self.jobPath+"/parameters",
 			method="POST",
-			data=form.forHTTPUpload(), expectedStatus=303, 
-			customHeaders={"content-type": 
+			data=form.forHTTPUpload(), expectedStatus=303,
+			customHeaders={"content-type":
 				form.get_content_type()+'; boundary="%s"'%(form.get_boundary())})
 
 
@@ -812,7 +812,7 @@ class ADQLSyncJob(_WithEndpoint):
 	Methods that make no sense at all for sync jobs ("phase") silently
 	return some more or less sensible fakes.
 	"""
-	def __init__(self, endpointURL, query=None, jobId=None, lang="ADQL", 
+	def __init__(self, endpointURL, query=None, jobId=None, lang="ADQL",
 			userParams={}, timeout=None):
 		self._defineEndpoint(endpointURL)
 		self.query, self.lang = query, lang
@@ -823,7 +823,7 @@ class ADQLSyncJob(_WithEndpoint):
 		self.timeout = timeout
 	
 	def postToService(self, params):
-		return request(self.destScheme, self.destHost, self.destPath+"/sync", 
+		return request(self.destScheme, self.destHost, self.destPath+"/sync",
 			params,
 			method="POST", followRedirects=5, expectedStatus=200,
 			setResponse=self._setErrorFromServer, timeout=self.timeout)
@@ -959,7 +959,7 @@ class ADQLEndpoint(_WithEndpoint):
 		or the returned document doesn't parse.
 		"""
 		try:
-			response = request(self.destScheme, self.destHost, 
+			response = request(self.destScheme, self.destHost,
 				self.destPath+"/availability", expectedStatus=200)
 			res = _parseWith(_AvailabilityParser(), response.data)
 		except WrongStatus:
@@ -976,8 +976,8 @@ class ADQLEndpoint(_WithEndpoint):
 		If the remote server doesn't return capabilities as expected, an
 		empty dict is returned.
 		"""
-		return _parseWith(_CapabilitiesParser(), 
-				request(self.destScheme, self.destHost, 
+		return _parseWith(_CapabilitiesParser(),
+				request(self.destScheme, self.destHost,
 					self.destPath+"/capabilities").data)
 
 	@property
