@@ -12,7 +12,6 @@ import json
 import re
 from urllib import parse as urlparse
 
-from multipart import multipart
 from twisted.python import failure
 from twisted.web import resource
 from twisted.web import server
@@ -20,14 +19,13 @@ from twisted.web import static
 from twisted.web import template
 from twisted.web.template import tags as T
 from twisted.web import http
-from twisted.web.http import parse_qs
 
 from gavo import base
+from gavo import formal
 from gavo import svcs
 from gavo import utils
 from gavo.base import meta
 from gavo.protocols import creds
-from gavo.protocols import dali
 
 
 # monkeypatching static's mime magic
@@ -299,7 +297,7 @@ class BinaryItem:
 			utils.makeEllipsis(repr(self.original)))
 
 
-class Request(server.Request):
+class Request(formal.Request):
 	"""a custom request class used in DaCHS' application server.
 
 	Overridden behaviour:
@@ -319,15 +317,7 @@ class Request(server.Request):
 	def __init__(self, *args, **kwargs):
 		server.Request.__init__(self, *args, **kwargs)
 		self.files, self.args = {}, {}
-		self.finished = False
-		self.notifyFinish().addBoth(self._markFinished)
 	
-	def _markFinished(self, ignored):
-		# I'm adding a finished attribute for convenience so clients
-		# can avoid finishing requests that don't need that more
-		# easily.
-		self.finished = True
-
 	def gotLength(self, length):
 		server.Request.gotLength(self, length)
 
@@ -529,110 +519,3 @@ class Request(server.Request):
 
 	def processingFailed(self, failure):
 		return produceErrorDocument(failure, self)
-
-	def requestReceived(self, command, path, version):
-		# overridden because I want to use multipart to parse
-		# POST payloads.
-		#
-		# This is essentially requestReceived as of twisted 22.1 with
-		# the entire POST processing replaced.  Oh, and there's a
-		# TAP path hack to avoid breaking links to pre-TAP-corified
-		# DaCHS versions.
-		#
-		# TODO: parsing this is still sync, which for large request bodies
-		# can take quite a while, during which we are otherwise inaccessible.
-		# I'd have to research hard how I can yield to the reactor from
-		# in here...
-
-		# Brute-force TAP compatibility redirect, 2021
-		# As long as old registry records may still contain the legacy
-		# TAP URL, just brute-force in the new one.  For now, DaCHS doesn't
-		# have a mechanism for replacing multiple segments of a path with
-		# multiple other segments (vanity can just do one).
-		# I suppose this ugly stuff can go ~2025; or we'll need an alternative
-		# way of doing this kind of thing.
-		path = path.replace(
-			b"__system__/tap/run/tap/",
-			b"__system__/tap/run/")
-
-		self.content.seek(0, 0)
-
-		self.method, self.uri = command, path
-		self.clientproto = version
-		x = self.uri.split(b"?", 1)
-
-		if len(x) == 1:
-			self.path = self.uri
-		else:
-			self.path, argstring = x
-			self.args = parse_qs(argstring, 1)
-
-		# Argument processing
-		ctype = self.requestHeaders.getRawHeaders(b"content-type")
-		if ctype is not None:
-			ctype = ctype[0]
-
-		if self.method == b"POST" and ctype:
-			chunk_size = 1048576
-
-			def addArg(field):
-				# The string/byte situation on urlparse.unquote* is confusingly
-				# bizarre.  If I want to be compatible with baseline twisted
-				# args, I don't think I get around the odd quote/dequote thing
-				# I'm doing here; and yes, it's latin1 on the incoming string,
-				# as any non-ascii in there probably isn't utf-8.
-				if field.value is None:
-					# ignore empty parameters coming in (it's what cgi did, too)
-					return
-				self.args.setdefault(field.field_name, []).append(
-					urlparse.unquote_plus(
-						field.value.decode("latin-1")).encode("utf-8"))
-
-			def addFile(file_obj):
-				# some versions of multipart pass in the files already
-				# consumed.  Until this is fixed, rewind them here.
-				file_obj.file_object.seek(0)
-				self.files.setdefault(file_obj.field_name.decode("latin-1"), []
-				    ).append(file_obj)
-
-			# we could enfoce maxUploadSize using MAX_BODY_SIZE in
-			# multipart; however, that just skips the extra bytes.
-			# I'd like to return a 413, and hence I count myself.
-			total_read = 0
-
-			try:
-				strippedCtype, ctypeParams = multipart.parse_options_header(ctype)
-				parser = multipart.FormParser(
-					strippedCtype.decode("latin-1"), addArg, addFile,
-					boundary=ctypeParams.get(b"boundary"))
-
-				while True:
-					stuff = self.content.read(chunk_size)
-					if stuff==b"":
-						break
-
-					total_read += len(stuff)
-					if total_read>self.maxUploadSize:
-						raise svcs.RequestEntityTooLarge(
-							f"Would only accept {self.maxUploadSize} bytes of upload here.")
-
-					parser.write(stuff)
-				parser.finalize()
-
-			except svcs.RequestEntityTooLarge as ex:
-				return dali.serveDALIError(self, ex, httpStatus=413)
-
-			except Exception as e:
-				# It was a bad request, or we got a signal.
-				base.ui.notifyError(f"Fatal error while parsing request: {e}")
-				self.channel._respondToBadRequestAndDisconnect()
-				if isinstance(e, (TypeError, ValueError, KeyError)):
-					return
-				else:
-					# If it's not a "userspace" error, reraise
-					raise
-
-			self.content.seek(0, 0)
-
-		self.process()
-
