@@ -2,7 +2,7 @@
 Infrastructure pages.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -104,7 +104,7 @@ class TemplatedPage(grend.CustomTemplateMixin, grend.ServiceBasedPage):
 		self.metaCarrier = meta.MetaMixin()
 		self.metaCarrier.setMetaParent(self.service)
 		if isinstance(fName, str):
-			self.metaCarrier.setMeta("_dataUpdated", 
+			self.metaCarrier.setMeta("_dataUpdated",
 				datetime.datetime.fromtimestamp(os.path.getmtime(fName)))
 
 
@@ -152,10 +152,10 @@ class StaticFile(static.File):
 		".js": minifyJS,
 	}
 
-	def __init__(self, fName, cacheRD):
+	def __init__(self, fName, cacheRD, defaultType="text/html"):
 		self.cacheRD = cacheRD
 		self.overriddenContent = None
-		static.File.__init__(self, fName)
+		static.File.__init__(self, fName, defaultType=defaultType)
 
 	def openForReading(self):
 		if self.overriddenContent is None:
@@ -215,7 +215,7 @@ class GavoJquery(resource.Resource):
 	"""synthesises a query-gavo.js from Debian packages.
 	"""
 	INSTALL_ROOT = "/usr/share/javascript/"
-	PARTS = ["jquery/jquery.js", 
+	PARTS = ["jquery/jquery.js",
 		io.StringIO("""
 				(function( factory ) {
 				if ( typeof define === "function" && define.amd ) {
@@ -294,8 +294,14 @@ class GavoJquery(resource.Resource):
 			if hasattr(part, "read"):
 				parts.append(part.read())
 			else:
-				with open(os.path.join(self.INSTALL_ROOT, part), "r") as f:
-					parts.append(f.read())
+				try:
+					with open(os.path.join(self.INSTALL_ROOT, part), "r") as f:
+						parts.append(f.read())
+				except IOError:
+					# We don't worry about missing modules here; this lets
+					# us accept multiple versions of the Debian packages
+					# of jquery and jquery-ui.
+					pass
 		stuff = "\n".join(parts)
 
 		self.modStamp = time.time()
@@ -317,7 +323,7 @@ class GavoJquery(resource.Resource):
 
 DEBIAN_FALLBACKS = {
 	'js/jquery-gavo.js': GavoJquery(),
-	'js/jquery.flot.js': 
+	'js/jquery.flot.js':
 		"/usr/share/javascript/jquery-flot/jquery.flot.js",
 }
 
@@ -464,7 +470,7 @@ class ServiceUnavailable(resource.Resource):
 		request.setResponseCode(503)
 		request.setHeader("retry-after", "3600")
 
-		with open(os.path.join(base.getConfig("stateDir"), "MAINT"), 
+		with open(os.path.join(base.getConfig("stateDir"), "MAINT"),
 				"r", encoding="utf-8") as f:
 			try:
 				maintText = f.read()
@@ -519,7 +525,7 @@ class CurReaders(resource.Resource):
 		request.setHeader("content-type", "text/plain")
 # TODO: the following must be amended for IPv6
 		hostsActive = len([1
-			for r in reactor.getReaders() 
+			for r in reactor.getReaders()
 			if hasattr(r, "getHost") and r.getHost().host!='0.0.0.0'])
 		return utils.bytify(str(hostsActive)+"\n")
 
@@ -546,7 +552,7 @@ class _ThirdPartyCacher(resource.Resource):
 	
 	def render(self, request):
 		retriever = client.Agent(reactor)
-		d = retriever.request(b"GET", 
+		d = retriever.request(b"GET",
 			self.parent.urlMapping[self.keyToGet])
 		d.addCallback(self.startReadingRemote, request)
 		d.addErrback(self.stopAndForget, request)
@@ -605,7 +611,7 @@ class ThirdPartyCachePage(resource.Resource):
 	"""
 	#	NOTE: this maps bytes to bytes!
 	urlMapping = {
-		b"aladin.min.js": 
+		b"aladin.min.js":
 			b"https://aladin.u-strasbg.fr/AladinLite/api/v2/latest/aladin.min.js",
 		b"aladin.min.css":
 			b"https://aladin.u-strasbg.fr/AladinLite/api/v2/latest/aladin.min.css",
