@@ -3,7 +3,7 @@
 Renderers that take services "as arguments".
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -48,7 +48,7 @@ def makeBibTeXForMetaCarrier(mc):
 	"""
 	creationDate = utils.parseISODT(
 		base.getMetaText(mc,
-			"creationDate", 
+			"creationDate",
 			raiseOnFail=True))
 	year = creationDate.year
 			
@@ -65,8 +65,8 @@ def makeBibTeXForMetaCarrier(mc):
 				int(re.sub("[^\d]+", "", creationDate.isoformat())))
 	label = "vo:"+re.sub("[^\w]", "_", label)
 
-	authors = " and ".join(m.getContent() 
-				for m in mc.iterMeta("creator.name", 
+	authors = " and ".join(m.getContent()
+				for m in mc.iterMeta("creator.name",
 						propagate=True))
 
 	res = [
@@ -205,7 +205,7 @@ class RendExplainer(object):
 			" to your SOAP library; the function signature is"
 			"  useService(",
 			generateArguments(),
-			").  See also our ", 
+			").  See also our ",
 			T.a(render="rootlink", href="/static/doc/soaplocal.shtml")[
 				"local soap hints"]]
 
@@ -214,7 +214,7 @@ class RendExplainer(object):
 		res = T.transparent["a custom rendering of the service, typically"
 			" for interactive web applications."]
 		if svcs.getRenderer("custom").isBrowseable(service):
-			res["  See also the ", 
+			res["  See also the ",
 				T.a(href=service.getURL("custom"))["entry page"], "."]
 		return res
 	
@@ -246,6 +246,22 @@ class RendExplainer(object):
 						base.getMetaText(service, "testQuery.size.dec", default="3")))[
 					"Validate"]]]
 
+	@classmethod
+	def _explain_siap2_xml(cls, service):
+		return T.transparent["a standard SIAP version 2 interface as defined by the"
+			" IVOA to access collections of celestial images; SIAP clients"
+			" use ", service.getURL("siap2.xml"), " to access the service",
+			T.transparent(render="ifadmin")[" – ",
+				T.a(href="http://voparis-validator.obspm.fr/validator.php?"
+					"spec=Simple+Image+Access+2.0"
+					"&serviceURL=%s"%urllib.parse.quote(service.getURL("siap2.xml"))
+					+"&FOV=&BAND=&TIME=&POL=&SPATRES=&EXPTIME=&ID=&COLLECTION=&FACILITY=&INSTRUMENT=&DPTYPE=image&CALIB=&TARGET=&TIMERES=&SPECRP=&FORMAT=&MAXREC=&format=XHTML&REQUEST=query"
+					+"&POS=CIRCLE+{}+{}+{}".format(
+						base.getMetaText(service, "testQuery.pos.ra", default="180"),
+						base.getMetaText(service, "testQuery.pos.dec", default="60"),
+						base.getMetaText(service, "testQuery.size.ra", default="3")))[
+					"Validate"]]]
+
 	@classmethod
 	def _explain_scs_xml(cls, service):
 		return T.transparent["a standard SCS interface as defined by the"
@@ -300,7 +316,7 @@ class RendExplainer(object):
 			" a client programme to use this."
 			"  In such clients, you can find this service by its"
 			" IVOID, ",
-			T.code(render="meta")["identifier"], 
+			T.code(render="meta")["identifier"],
 			", or access it by entering its base URL ",
 			T.code[service.getURL("dali")],
 			" directly.  Using an XSL-enabled web browser you can, in a pinch,"
@@ -379,7 +395,24 @@ class RendExplainer(object):
 			" of this service.  By default, this will return a FITS MOC,"
 			" but browsers and similar clients declaring they accept PNGs"
 			" will get a sky plot showing the coverage."]
-		
+	
+	@classmethod
+	def _explain_examples(cls, service):
+		return T.transparent["a list of examples for"
+			" how to use this resource.  You can read ",
+			T.a(href=service.getURL("examples"))["the examples"],
+			" with your browser.  RDFa-aware clients can figure out the"
+			" examples at ",
+			service.getURL("api")]
+
+	@classmethod
+	def _explain_hips(cls, service):
+		return T.transparent["a HiPS endpoint for fast zooming and panning"
+			" through in particular images.  Paste the access URL ",
+			T.a(href=service.getURL("hips"))[service.getURL("hips")],
+			" into a HiPS client or click it when you have a javascript-"
+			"enabled browser."]
+
 	@classmethod
 	def _explainEverything(cls, service):
 		return T.transparent["a renderer with some custom access method that"
@@ -387,7 +420,7 @@ class RendExplainer(object):
 
 	@classmethod
 	def explain(cls, renderer, service):
-		return getattr(cls, "_explain_"+renderer.replace(".", "_"), 
+		return getattr(cls, "_explain_"+renderer.replace(".", "_"),
 			cls._explainEverything)(service)
 
 
@@ -447,13 +480,13 @@ class ServiceInfoRenderer(MetaRenderer, utils.IdManagerMixin):
 		return res
 
 	def data_rendAvail(self, request, tag):
-		return [{"rendName": rend, 
+		return [{"rendName": rend,
 				"rendExpl": RendExplainer.explain(rend, self.service)}
 			for rend in self.service.allowed]
 
 	def data_publications(self, request, tag):
-		res = [{"sets": ",".join(p.sets), "render": p.render} 
-			for p in self.service.publications 
+		res = [{"sets": ",".join(p.sets), "render": p.render}
+			for p in self.service.publications
 			if p.sets and p.render not in registry.HIDDEN_RENDERERS]
 		return sorted(res, key=lambda v: v["render"])
 
@@ -475,7 +508,7 @@ class ServiceInfoRenderer(MetaRenderer, utils.IdManagerMixin):
 class TableInfoRenderer(MetaRenderer):
 	"""A renderer for displaying table information.
 
-	Since tables don't necessarily have associated services, this 
+	Since tables don't necessarily have associated services, this
 	renderer cannot use a service to sit on.  Instead, the table is
 	being passed in as as an argument. There's a built-in vanity tableinfo
 	that sits on //dc_tables#show using this renderer (it could really
@@ -576,7 +609,7 @@ class TableNoteRenderer(MetaRenderer):
 	name = "tablenote"
 
 	def render(self, request):
-		if not hasattr(self, "noteTag"):  
+		if not hasattr(self, "noteTag"):
 			# _retrieveTableDef did not run, i.e., no tableName was given
 			raise svcs.UnknownURI(
 				"You must provide table name and note tag to this renderer.")
@@ -598,7 +631,7 @@ class TableNoteRenderer(MetaRenderer):
 
 		if len(segments)==2:
 			self._retrieveNote(segments[0], segments[1])
-		elif len(segments)==3: # segments[0] may be anything, 
+		elif len(segments)==3: # segments[0] may be anything,
 			# but conventionally "inner"
 			self._retrieveNote(segments[1], segments[2])
 			self.loader = self.innerLoader
@@ -620,7 +653,7 @@ class TableNoteRenderer(MetaRenderer):
 <html xmlns="xmlns=http://www.w3.org/1999/xhtml"
 	xmlns:n="http://nevow.com/ns/nevow/0.1">
 <head>
-	<title>\getConfig{web}{sitename} – Note for table 
+	<title>\getConfig{web}{sitename} – Note for table
 		<n:invisible n:render="unicode" n:data="tableName"/></title>
 	<n:invisible n:render="commonhead"/>
 	<style>
@@ -681,7 +714,7 @@ class CoverageRenderer(MetaRenderer):
 		a web browser.
 		"""
 		acceptDict = utils.parseAccept(request.getHeader("accept"))
-		return ("image/png" in acceptDict 
+		return ("image/png" in acceptDict
 				or "image/webp" in acceptDict # firefox ~78 hack
 				or "image/*" in acceptDict
 				or "text/html" in acceptDict)
@@ -743,12 +776,12 @@ class RDInfoPage(grend.CustomTemplateMixin, grend.ResourceBasedPage):
 	customTemplate = svcs.loadSystemTemplate("rdinfo.html")
 
 	def data_services(self, request, tag):
-		return sorted(self.rd.services, 
+		return sorted(self.rd.services,
 			key=lambda s: base.getMetaText(s, "title", default=s.id))
 	
 	def data_tables(self, request, tag):
-		return sorted((t for t in self.rd.tables 
-			if t.onDisk 
+		return sorted((t for t in self.rd.tables
+			if t.onDisk
 				and not t.temporary
 				and not t.hasProperty("internal")),
 			key=lambda t: t.id)
@@ -785,7 +818,7 @@ class RDInfoPage(grend.CustomTemplateMixin, grend.ResourceBasedPage):
 		if tableDef.adql:
 			adqlNote = T.span(class_="adqlnote")[" ",
 				"–", " queryable through ",
-				T.a(href="/tap")["TAP"], " and ", 
+				T.a(href="/tap")["TAP"], " and ",
 				T.a(href="/adql")["ADQL"],
 				" "]
 
@@ -839,9 +872,9 @@ class RDInfoRenderer(grend.CustomTemplateMixin, grend.ServiceBasedPage):
 		with base.getTableConn() as conn:
 			return [row[0] for row in
 				conn.query(
-					"""SELECT DISTINCT sourceRD 
+					"""SELECT DISTINCT sourceRD
 					FROM (
-						SELECT sourceRD FROM dc.resources 
+						SELECT sourceRD FROM dc.resources
 						WHERE NOT deleted) as q
 					ORDER BY sourceRD""")]
 
@@ -867,7 +900,7 @@ class ResourceRecordMaker(resource.Resource):
 	This is basically like OAI-PMH getRecord, except we're using rd/id/svcid
 	from our path.
 
-	Also (and that's fairly important for purx), this will use the 
+	Also (and that's fairly important for purx), this will use the
 	_metadataUpdated meta for a modified header.
 	"""
 	def render(self, request):
