@@ -17,6 +17,7 @@ from gavo import base
 from gavo import utils
 from gavo.protocols import products
 from gavo.web import grend
+from gavo.web import producttar
 from gavo.web import weberrors
 
 
@@ -61,19 +62,31 @@ class ProductRenderer(grend.ServiceBasedPage):
 	pathFromSegments = ""
 
 	def render(self, request):
-		data = {"accref":
-			products.RAccref.fromRequest(self.pathFromSegments, request)}
+		if len(request.strargs.get("accref", []))>1:
+			# more than one accref passed in: produce a tar
+			assert not self.pathFromSegments
+			data = request.strargs
+		else:
+			data = {"accref":
+					products.RAccref.fromRequest(self.pathFromSegments, request)}
 
-		# deferring here and going to all the trouble of running a core
-		# is probably overkill; currently, the main thing that'd require
-		# handwork is figuring out authentication in parallel with what
-		# needs to be done for tar files.  We should do better, indded.
 		self.runAsyncWithFormalData(data, request
 			).addCallback(self._deliver, request
 			).addErrback(weberrors.renderDCErrorPage, request)
 		return server.NOT_DONE_YET
 	
 	def _deliver(self, result, request):
+		if len(result)>1:
+			# TODO: presumably we should have the option of producing
+			# tars with 0 and 1 item(s); we should probably have
+			# a special renderer for that.
+			return producttar.getTarMaker()._streamOutTar(
+				result, request, self.queryMeta, "products.tar")
+
+		else:
+			return self._deliverSingle(result, request)
+
+	def _deliverSingle(self, result, request):
 		doPreview = self.queryMeta.ctxArgs.get("preview")
 		product = result[0]
 
