@@ -110,12 +110,13 @@ class ProductTarMaker(object):
 				continue # just skip files that somehow don't exist any more
 
 			elif isinstance(src, products.UnauthorizedProduct):
-				outputTar.addfile(*self._getEmbargoedFile(src.name))
+				outputTar.addfile(*self._getEmbargoedFile(
+					nameGen.makeName(src.name)))
 
 			elif isinstance(src, products.FileProduct):
 				# actual file in the file system
-				targetName = nameGen.makeName(src.name)
-				outputTar.add(str(src.rAccref.localpath), targetName)
+				outputTar.add(str(src.rAccref.localpath),
+					nameGen.makeName(src.name))
 
 			else: # anything else is read from the src
 				outputTar.addfile(*self._getTarInfoFromProduct(src,
