@@ -16,7 +16,6 @@ import time
 from twisted.python import threadable
 threadable.init()
 
-from twisted import copyright
 from twisted.web import resource
 from twisted.web import server
 from twisted.web import static
@@ -290,11 +289,9 @@ class ArchiveService(nevowc.TemplatedPage):
 
 		if False:
 			from gavo.helpers import testtricks
-			testtricks.memdebug(svcs.QueryMeta)
+			testtricks.memdebug(common.Request)
 
-		request.setHeader(b"server", "DaCHS/%s twistedWeb/%s"%(
-			base.__version__,
-			copyright.version))
+		request.setHeader(b"server", base.SERVER_SOFTWARE)
 
 		if request.requestHeaders.hasHeader("x-forwarded-host"):
 			# we need the externally visible host in our request even
