@@ -11,6 +11,7 @@ DALI-specified protocols are in dalirender, UWS/async is in asyncrender.
 
 
 import datetime
+import json
 import os
 
 from twisted.internet import defer
@@ -133,6 +134,7 @@ class DALRenderer(grend.ServiceBasedPage):
 			parent_=self.service.rd)
 
 		data = rsc.makeData(dataDesc)
+		data.contributingMetaCarriers.append(self.service)
 		data.tables["results"].votCasts = self._outputTableCasts
 		data.setMeta("_type", "meta")
 		data.addMeta("info",
@@ -184,9 +186,13 @@ class DALRenderer(grend.ServiceBasedPage):
 				request.finish()
 			return server.NOT_DONE_YET
 
+		data.contributingMetaCarriers.append(self.service)
+
 		data.addMeta("info", "", infoName="QUERY_STATUS",
 			infoValue=base.getMetaText(data.getPrimaryTable(),
 				"_queryStatus"))
+		data.addMeta("info", "", infoName="request",
+			infoValue=utils.debytify(request.uri))
 
 		if self.standardId:
 			data.addMeta("info",
@@ -206,7 +212,7 @@ class DALRenderer(grend.ServiceBasedPage):
 
 		request.setHeader("content-type", requestedType)
 		request.setHeader('content-disposition',
-			'attachment; filename="result.%s"'%
+			'attachment; filename="result%s"'%
 			formats.getExtensionFor(requestedType))
 
 		formatterArgs = {}
@@ -278,7 +284,8 @@ class SCSRenderer(DALRenderer):
 			V.DESCRIPTION[base.getMetaText(self.service, "description")],
 			V.RESOURCE(type="results")[
 				V.INFO(ID="Error", name="Error",
-						value=str(msg).replace('"', '\\"')),]], request)
+						value=str(msg).replace('"', '\\"')),
+				V.INFO(name="request", value=utils.debytify(request.uri))]], request)
 		request.write("\n")
 		request.finish()
 
@@ -537,6 +544,8 @@ class SSAPRenderer(UnifiedDALRenderer):
 			# the thing isn't already rendered, so we can add some additional
 			# metadata
 
+			data.contributingMetaCarriers.append(self.service)
+
 			data.setMeta("_type", "results")
 			data.addMeta("_votableRootAttributes",
 				'xmlns:ssa="http://www.ivoa.net/xml/DalSsap/v1.0"')
@@ -705,6 +714,46 @@ class RegistryRenderer(grend.ServiceBasedPage):
 			request.finish()
 
 
+class BiblinksRenderer(grend.ServiceBasedPage):
+	"""A renderer to put out bibliography links.
+
+	This would work with a constant-query dbCore, typically against
+	dc.biblinks.
+
+	(Since 2.8.2)
+	"""
+	name = "biblinks.json"
+	urlUse = "full"
+	resultType = "application/json"
+
+	def render(self, request):
+		self.runAsync(request.strargs,
+			).addCallback(self._renderResult, request)
+		return server.NOT_DONE_YET
+	
+	def _renderResult(self, result, request):
+		request.setHeader("content-type", "application/json")
+
+		def writeTo(f):
+			outRecs = []
+			for inRec in result.getPrimaryTable().rows:
+				outRec = {
+					"bib-ref": inRec["bib_ref"],
+					"relationship": inRec["relationship"],
+					"dataset-ref": inRec["dataset_ref"],
+				}
+				if inRec.get("bib_format"):
+					outRec["bib-format"] = inRec["bib_format"]
+				if inRec.get("cardinality"):
+					outRec["cardinality"] = inRec["cardinality"]
+
+				outRecs.append(outRec)
+
+			json.dump(outRecs, f)
+
+		return streaming.streamOut(writeTo, request, None)
+
+
 def addAttachmentHeaders(request, mime=None):
 	"""adds a content-disposition header to request with a filename guessed
 	based on datalink arguments.
