@@ -23,7 +23,6 @@ from twisted.web.template import tags as T
 from gavo import base
 from gavo import svcs
 from gavo import utils
-from gavo.base import config
 from gavo.formal import nevowc
 from gavo.web import common
 
@@ -115,7 +114,7 @@ class ErrorPage(nevowc.TemplatedPage, common.CommonRenderers):
 	
 	@template.renderer
 	def footer(self, request, tag):
-		contactMail = str(config.getMeta("contact.email"))
+		contactMail = str(base.getGlobalMeta("contact.email"))
 		return tag[
 			T.hr,
 			T.address[T.a(href="mailto:%s"%contactMail)[contactMail]]]
@@ -346,7 +345,7 @@ def _formatFailure(flr):
 		" for a report at"
 		" %s</p>"%(flr.value.__class__.__name__,
 			common.escapeForHTML(flr.getErrorMessage()),
-			str(config.getMeta("contact.email"))))
+			str(base.getGlobalMeta("contact.email"))))
 	return res.encode("ascii", "ignore")
 
 
